package de.fhg.iais.outdoor.simba.core;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.value.ValueReplacedListener;

public class ParameterElement implements ValueReplacedListener {
	
	private String mPrefix;
	private Hashtable mParameters = new Hashtable();
	protected String mName;
	protected BooleanValue mActive;
	private boolean mAddValuesToValueManager;
	
	
	public ParameterElement(String name, String prefix) {
		this(name, prefix, true);
	}
	
	public ParameterElement(String name, String prefix, boolean addValuesToValueManager) {
		mPrefix = prefix;
		mName = name;
		mAddValuesToValueManager = addValuesToValueManager;
		
		mActive = new BooleanValue(true);
		addParameter(mActive, "Active");	
	}
	
	
	
	public void setName(String name) {
		mName = name;
	}
	
	public String getName() {
		return mName;
	}
	
	public void setPrefix(String prefix) {
		mPrefix = prefix;
	}
	
	public String getPrefix() {
		return mPrefix;
	}
	
	public BooleanValue getActiveValue() {
		return mActive;
	}
	
	public boolean isActive() {
		return mActive.get();
	}
	
	public void setActive(boolean active) {
		mActive.set(active);
	}
	
	public boolean addParameter(Value parameter, String name) {
		if(parameter == null || name == null || mParameters.contains(parameter)) {
			return false;
		}
		mParameters.put(parameter, name);
		if(mAddValuesToValueManager) {
			ValueManager vm = Simulator.getInstance().getValueManager();
			vm.addValue(mPrefix + name, parameter);
			parameter.addValueReplacedListener(this);
		}
		return true;
	}
	
	public Vector getParameters() {
		return new Vector(mParameters.keySet());
	}
	
	public String getParameterName(Value parameter) {
		return (String) mParameters.get(parameter);
	}
	

	public String createParameterLog() {
		//log parameters
		StringBuffer entry = new StringBuffer();
		Enumeration c = mParameters.keys();
		if(!c.hasMoreElements()) {
			//signal that nothing was logged.
			return "";
		}
		
		for(;c.hasMoreElements();) {
			Value v = (Value) c.nextElement();
			entry.append("\t").append(getParamEntry((String) mParameters.get(v), v.getValueAsString()));
		}
		
		return entry.toString();
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mActive = (BooleanValue) ValueManager.synchronizeValue(mActive, oldValue, newValue, this);
	}
	
	public static String getParamEntry(String name, String value) {
		StringBuffer entry = new StringBuffer();
		entry.append("<param name=\"").append(name).append("\" ");
		entry.append("value=\"").append(value).append("\"/>\n");
		return entry.toString();
	}
	
	public static String getParamEntry(String name, int[] values) {
		StringBuffer entry = new StringBuffer();
		entry.append("<param name=\"").append(name).append("\" ");
		entry.append("value=\"");
		for(int i = 0; i < values.length; ++i) {
			entry.append(values[i]);
			if(i < values.length - 1) {
				entry.append(",");
			}
		}
		entry.append("\"/>\n");
		return entry.toString();
	}
	
	public static String getParamEntry(String name, double[] values) {
		StringBuffer entry = new StringBuffer();
		entry.append("<param name=\"").append(name).append("\" ");
		entry.append("value=\"");
		for(int i = 0; i < values.length; ++i) {
			entry.append(values[i]);
			if(i < values.length - 1) {
				entry.append(",");
			}
		}
		entry.append("\"/>\n");
		return entry.toString();
	}
	
	public static String getParamEntry(String name, boolean[] values) {
		StringBuffer entry = new StringBuffer();
		entry.append("<param name=\"").append(name).append("\" ");
		entry.append("value=\"");
		for(int i = 0; i < values.length; ++i) {
			entry.append(values[i]);
			if(i < values.length - 1) {
				entry.append(",");
			}
		}
		entry.append("\"/>\n");
		return entry.toString();
	}
	
	
	
}
