package de.fhg.iais.outdoor.simba.devices;

import java.awt.geom.Point2D;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;


public class DifferentialDriveUnit extends Box {

	private DoubleValue mVelocityRight;
	private DoubleValue mVelocityLeft;
	private DoubleValue mMinVelocity;
	private DoubleValue mMaxVelocity;
	
	public DifferentialDriveUnit(String name, double x, double y, double angle, 
			double width, double height, double minVelocity, double maxVelocity) 
	{
		super(name, x, y, angle, width, height);
		mVelocityRight = new DoubleValue(0.0);
		mVelocityLeft = new DoubleValue(0.0);
		mMinVelocity = new DoubleValue(minVelocity);
		mMaxVelocity = new DoubleValue(maxVelocity);
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		vm.addValue("/SimObjects/" + name + "/VelocityRight", mVelocityRight);
		vm.addValue("/SimObjects/" + name + "/VelocityLeft", mVelocityLeft);
		vm.addValue("/SimObjects/" + name + "/MinVelocity", mMinVelocity);
		vm.addValue("/SimObjects/" + name + "/MaxVelocity", mMaxVelocity);
		
		mVelocityRight.addValueReplacedListener(this);
		mVelocityLeft.addValueReplacedListener(this);
		mMinVelocity.addValueReplacedListener(this);
		mMaxVelocity.addValueReplacedListener(this);
	}
	
	/**
	 * Calculates the distance to the closest obstacle.
	 * 
	 */
	public boolean update() {
		boolean ok = super.update();
		
		
		
		//double th = Math.toRadians(mAngle) + ((mVelocityRight - mVelocityLeft) / mWidth);

		
		/*
		double x = (mWidth * (mVelocityRight + mVelocityLeft) / 2.0 * (mVelocityRight - mVelocityLeft))
					* (Math.sin(th) - Math.sin(Math.toRadians(mAngle)));
		double y = ((mWidth * (mVelocityRight + mVelocityLeft)) / (2.0 * (mVelocityRight - mVelocityLeft)))
				* (Math.cos(th) - Math.cos(Math.toRadians(mAngle)));
		*/
		/*
		double x = (mVelocityRight + mVelocityLeft) / (mVelocityRight - mVelocityLeft) * mWidth / 2 
				* Math.sin((mVelocityRight - mVelocityLeft) / mWidth + Math.toRadians(mAngle));
		double y = (mVelocityRight + mVelocityLeft) / (mVelocityRight - mVelocityLeft) * mWidth / 2
				* Math.cos((mVelocityRight - mVelocityLeft) / mWidth + Math.toRadians(mAngle))
				+ ((mVelocityRight + mVelocityLeft) / (mVelocityRight - mVelocityLeft) * mWidth / 2);
		*/
		double velRight = mVelocityRight.get();
		double velLeft = mVelocityLeft.get();
		
		double x = 0.0;
		double y = 0.0;
		if(velRight == velLeft) {
			double angle = mAngle.get();

			x = (velRight + velLeft) / 2.0 * Math.cos(Math.toRadians(angle + 90.0));
			y = (velRight + velLeft) / 2.0 * Math.sin(Math.toRadians(angle + 90.0));
		}
		else {
			double angle = mAngle.get();
			
			double th = Math.toRadians(angle + 90.0) + ((velRight - velLeft) / mWidth.get());
			
			x = (velRight + velLeft) / 2.0 * Math.cos(th);
			y = (velRight + velLeft) / 2.0 * Math.sin(th);

			
			mAngle.set(Math.toDegrees(th) - 90.0);
		}
		
		Point2D.Double position = mPosition.get();
		mPosition.set(position.x + x, position.y + y);
	
		return ok;
	}
	
	public void setRightVelocity(double velocity) {
		mVelocityRight.set(velocity);
	}
	
	public void setLeftVelocity(double velocity) {
		mVelocityLeft.set(velocity);
	}
	
	public double getRightVelocity() {
		return mVelocityRight.get();
	}
	
	public double getLeftVelocity() {
		return mVelocityLeft.get();
	}
	
	public void setMinVelocity(double velocity) {
		mMinVelocity.set(velocity);
	}
	
	public void setMaxVelocity(double velocity) {
		mMaxVelocity.set(velocity);
	}
	
	public double getMinVelocity() {
		return mMinVelocity.get();
	}
	
	public double getMaxVelocity() {
		return mMaxVelocity.get();
	}
	
	/**
	 * Called whenever an observed Value object is replaced by another one in the 
	 * ValueManager. The object can then change its observation to the new Value object
	 * and cancel the observation of the old Value. When overriding this method
	 * in subclassed do not forget to call this method from the subclass.
	 */
	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mVelocityRight = (DoubleValue) synchronizeValue(mVelocityRight, oldValue, newValue);
		mVelocityLeft = (DoubleValue) synchronizeValue(mVelocityLeft, oldValue, newValue);
		mMaxVelocity = (DoubleValue) synchronizeValue(mMaxVelocity, oldValue, newValue);
		mMinVelocity = (DoubleValue) synchronizeValue(mMinVelocity, oldValue, newValue);
	}
	
}
