package de.fhg.iais.outdoor.simba.devices;

import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.util.Iterator;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;

public class OrthogonalDistanceScanner extends DistanceSensor {
	
	
	public OrthogonalDistanceScanner(String name, double x, double y, double angle, 
					double minScanDistance, double maxScanDistance) 
	{
		super(name, x, y, angle, minScanDistance, maxScanDistance);
	}
	
	/**
	 * Calculates the orthogonal distance to the center point of the closest obstacle.
	 * 
	 */
	public boolean update() {
		boolean ok = super.update();
	
		double maxScanDistance = mMaxScanDistance.get();
		double minScanDistance = mMinScanDistance.get();
		
		double distance = maxScanDistance;
		
		AffineTransform transformation = new AffineTransform();
		Point2D.Double globalPos = getGlobalPosition();
		transformation.translate(globalPos.x, globalPos.y);
		transformation.rotate(Math.toRadians(getGlobalAngle()));
		try {
			transformation = transformation.createInverse();
		}
		catch(NoninvertibleTransformException ex) {
		}
		
		
		SimObjectSet obstacles = mObstacles;
		int setIndex = 0;
		do {
			for(Iterator i = obstacles.getIterator(); i.hasNext();) {
				SimObject obs = (SimObject) i.next();
				if(!obs.isActive() || mIgnorableObjects.contains(obs)) {
					continue;
				}
				
				//Get the object center point in scanner coordinates.
				Point2D.Double objPosition = obs.getGlobalPosition();
				Point2D.Double objPositionTransformed = new Point2D.Double();
				transformation.transform(objPosition, objPositionTransformed);
				if(objPositionTransformed.x >= 0.0 && objPositionTransformed.x <= maxScanDistance) 
				{
					//this scanner only measures the distance in x direction along the scanner line.
					if(objPositionTransformed.x < distance) {
						if(objPositionTransformed.x < minScanDistance) {
							distance = minScanDistance;
							break;
						}
						else {
							distance = objPositionTransformed.x;
						}
					}
				}
			}
			if(setIndex < mObstacleLists.size()) {
				obstacles = (SimObjectSet) mObstacleLists.elementAt(setIndex);
				setIndex++;
			}
			else {
				obstacles =  null;
			}
		}
		while(obstacles != null);
		
		//mDistance -= mMinScanDistance;
		double dist = Math.max(Math.min(addNoise(distance), maxScanDistance), minScanDistance);
		
		mDistance.set(Math.max(0.0, dist));

		return ok;
	}
	
}
