package de.fhg.iais.outdoor.simba.devices;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Sensor;
import de.fhg.iais.outdoor.simba.core.SimObject;

public class ScannerGroup extends Sensor {

	public static final int MODE_MIN = 1;
	public static final int MODE_MAX = 2;
	public static final int MODE_MEAN = 3;
	public static final int MODE_GREATEST_DIFFERENCE = 4;
	
	protected int mMode;
	protected Vector mScanners = new Vector();
	protected InternalSensor mInternalSensor;
	
	public ScannerGroup(String name, double x, double y, double angle, int mode, boolean alignSensors) {
		super(name, x, y, angle);
		mMode = mode;
		mInternalSensor = new InternalSensor(name + "/GroupSensor");
	}
	
	/**
	 * This method is called whenever this SimObject was added to an EnvironmentModel.
	 * Subclasses of SimObject can used this method to register sub components at the
	 * EnvironmentModel.
	 * Here the internal sensor is added to the sensor list.
	 * 
	 * @param model the EnvironmentModel where the child SimObjects should be registered.
	 */
	public void addChildren(EnvironmentModel model) {
		model.addSimObject(mInternalSensor);
	}
	
	/**
	 * Calculates the distance to the closest obstacle.
	 * 
	 */
	public boolean update() {
		boolean ok = super.update();
		
		return ok;
	}
	
	/**
	 * Returns the last measured value of this sensor.
	 * @return
	 */
	public double getSensorReading() {
		return mInternalSensor.getSensorReading();
	}
	
	/**
	 * Adds a scanner object to the group.
	 * 
	 * @param scanner the scanner to add.
	 * @return true if successful.
	 */
	public boolean addScanner(DistanceSensor scanner) {
		if(scanner == null || mScanners.contains(scanner)) {
			return false;
		}
		mScanners.add(scanner);
		mInternalSensor.addDependency(scanner);
		return true;
	}
	
	/**
	 * Adds a scanner object to the group and sets itself as reference object
	 * of that scanner.
	 * @param scanner the scanner to add.
	 * @return true if successful.
	 */
	public boolean addScannerAsChild(DistanceSensor scanner) {
		if(scanner == null) {
			return false;
		}
		addScanner(scanner);
		if(scanner instanceof SimObject) {
			SimObject scanObj = (SimObject) scanner; 
			scanObj.setReferenceObject(this);
			
		}
		return true;
	}
	
	/**
	 * Returns a Vector with all registered Scanner objects.
	 * @return 
	 */
	public Vector getScanners() {
		return new Vector(mScanners);
	}
	
	public class InternalSensor extends Sensor {
		
//		private Vector mSensorValues = new Vector();
		
		public InternalSensor(String name) {
			super(name, 0.0, 0.0, 0.0);
		}
		
		/**
		 * Calculates the distance to the closest obstacle.
		 * TODO - Postponed.
		 */
		public boolean update() {
			boolean ok = super.update();
//			mSensorValues.clear();
//			double[] scanValues = new double[mScanners.size()];
//			
//			//Read all sensor values.
//			for(int i = 0; i < mScanners.size(); ++i) {
//				DistanceSensor sensor = (DistanceSensor) mScanners.elementAt(i);
//				scanValues[i] = sensor.getCurrentValue();
//			}
//			
//			//determin the sensor's value
//			
//			//TODO
//			//aligh the values of the sensors.
//			for(Iterator i = mScanners.iterator(); i.hasNext();) {
//				//DistanceSensor sensor = (DistanceSensor) i.next();
//				//Vector readings = sensor.getSensorReading();
//			}
//			
			return ok;
		}
		
		/**
		 * Returns the last measured value of this sensor.
		 * @return
		 */
		public double getSensorReading() {
			return 0.0;
		}
		
	}
	
}
