package de.fhg.iais.outdoor.simba.devices;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.StringValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class TextLabel extends SimObject {
	
	private StringValue mTextValue;
	private int mMaxLength;
	private static int mLetterSize = 6;
	private boolean mCrossOut;
	
	public TextLabel(String name, double x, double y, double angle, String initText, int maxLength, boolean registerAtValueManager) {
		super(name, x, y, angle, new Polygon(), registerAtValueManager);
		mMaxLength = maxLength;
		mTextValue = new StringValue(initText);
		mCrossOut = false;
		if(registerAtValueManager) {
			ValueManager vm = Simulator.getInstance().getValueManager();
			vm.addValue("/SimObjects/" + getName() + "/Text", mTextValue);
		}
		mTextValue.addValueReplacedListener(this);
		
		Polygon shape = new Polygon();
		shape.addPoint(-(maxLength / 2 * mLetterSize), -7);
		shape.addPoint(-(maxLength / 2 * mLetterSize), 7);
		shape.addPoint((maxLength / 2 * mLetterSize), 7);
		shape.addPoint((maxLength / 2 * mLetterSize), -7);
		
		setShape(shape);
		
	}
	
	public StringValue getTextValue() {
		return mTextValue;
	}
	
	public void setText(String text) {
		mTextValue.set(text);
	}
	
	/**
	 * Determines whether the text should be displayed crossed out.
	 * @param crossOut
	 */
	public void setCrossedOut(boolean crossOut) {
		mCrossOut = crossOut;
	}
	
	public boolean isCrossedOut() {
		return mCrossOut;
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mTextValue = (StringValue) ValueManager.synchronizeValue(mTextValue, oldValue, newValue, this);
	}

	public void paintSelf(Graphics2D g) {
		Color c = g.getColor();
		Composite composite = g.getComposite();
		g.setColor(mShapeColor);
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float) mTransparency.get()));
		
		AffineTransform oldTransform = g.getTransform();
		
		
		AffineTransform transformation = getAffineTransform();
		try {
			transformation = transformation.createInverse();
		}
		catch(NoninvertibleTransformException ex) {
			Logger.getLogger("Simulator")
				.warning("TextLabel.paintSelf: Could not invert AffineTransform.");
			g.setColor(c);
			return;
		}
		
		
		
		g.setColor(getShapeColor());

		
		g.scale(1.0, -1.0);
		String text = mTextValue.get();
		int length = Math.min(mMaxLength, text.length());
		g.translate(((mMaxLength / 2) - (length / 2)) * mLetterSize, 0);
		g.drawChars(text.toCharArray(), Math.max(0, text.length() - mMaxLength), length, -(mMaxLength / 2 * mLetterSize), 5);

		g.scale(1.0, -1.0);
		
		if(mCrossOut && mShape.npoints >= 4) {
			g.setTransform(oldTransform);
			g.drawLine(mShape.xpoints[0], mShape.ypoints[0], mShape.xpoints[2], mShape.ypoints[2]);
			g.drawLine(mShape.xpoints[1], mShape.ypoints[1], mShape.xpoints[3], mShape.ypoints[3]);
		}
		
		g.setComposite(composite);
		g.setColor(c);
	}
	
	

}
