package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;

public class EvolutionConfigLogHandler implements GenerationListener {
	
	public EvolutionConfigLogHandler() {
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		
	}
	
	

	public void generationCompleted(int generationId) {
	}



	public void generationStarted(int generationId) {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		Scheduler scheduler = Simulator.getInstance().getScheduler();
		EvolutionLogger logger = scheduler.getEvolutionLogger();
		
		logger.logXML("<configuration>", "</configuration>");
		
		StringBuffer entry = new StringBuffer();
		
		if(em.getEvolutionRunningValue().get()) {
			//Evolution parameters
			
			entry.append("<evolutionManager meanPopulationSize=\"").append(em.getMeanPopulationSizeValue().get())
					.append("\" preserveParents=\"" + em.getPreserveParentsValue().get() + "\">\n");
			//entry.append("\t<selectionMethod name=\"").append(em.getSelectionMethod().getName())
			//	.append("\"/>\n");
			entry.append("\t<evaluationMethod name=\"").append(em.getEvaluationMethod().getName())
				.append("\"/>\n");
			entry.append("</evolutionManager>\n");
			
			logger.logXML(entry.toString());
			
			
			//EvolutionOperators
			logger.logXML("<evolutionOperators>", "</evolutionOperators>");
			Vector operators = Simulator.getInstance().getEvolutionManager().getEvolutionOperators();
			for(Iterator i = operators.iterator(); i.hasNext();) {
				String opEntry = ((EvolutionOperator) i.next()).createParameterLog();
				logger.logXML(opEntry);
			}
			logger.closeTag();
		}
		
		//Scheduler parameters
		entry = new StringBuffer();
		entry.append("<scheduler numberOfTries=\"").append(scheduler.getNumberOfTries())
			.append("\" numberOfSteps=\"").append(scheduler.getNumberOfSteps()).append("\"/>");
		logger.logXML(entry.toString());
		
		//Fitness functions
		Vector fitnessFunctions = Simulator.getInstance().getFitnessManager().getFitnessFunctions();
		logger.logXML("<fitnessFunctions>", "</fitnessFunctions>");
		for(Iterator i = fitnessFunctions.iterator(); i.hasNext();) {
			String ffEntry = ((FitnessFunction) i.next()).createParameterLog();
			logger.logXML(ffEntry);
		}
		logger.closeTag();
		
		//Environments
		Vector environments = scheduler.getEnvironmentModels();
		logger.logXML("<environments>", "</environments>");
		for(int i = 0; i < environments.size(); ++i) {
			EnvironmentModel model = (EnvironmentModel) environments.elementAt(i);
			logger.logXML("<environment index=\"" + i + "\" name=\"" + model.getName() + "\">", "</environment>");
			logger.logXML(model.createParameterLog());
			logger.logXML(model.createRandomizationParameterLog());
			logger.closeTag();
		}
		logger.closeTag();
		
		
		logger.closeTag();
	}

}
