package de.fhg.iais.outdoor.simba.evolution;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;
import java.util.LinkedList;

public class EvolutionLogger {
	
	private BufferedWriter mFileWriter;
	private boolean mActive;
	private int mIndentation;
	private DateFormat mDateFormatter;
	private LinkedList mClosingTags = new LinkedList();
	private String mFileName;
	
	public EvolutionLogger() {
		mIndentation = 0;
		mDateFormatter = DateFormat.getInstance();
	}
	
	public synchronized boolean startEvolutionLogger(String filename) {
		if(mFileWriter != null) {
			closeEvolutionLogger();
		}
		mIndentation = 0;
		try {
			mFileWriter = new BufferedWriter(new FileWriter(filename));
			mFileName = filename;
			mActive = true;
			addHeader();
			
		} catch (IOException e) {
			return false;
		}
		return true;
	}
	
	public boolean logXML(String xml) {
		return logXML(xml, "");
	}
	
	public synchronized boolean logXML(String xml, String closingTag) {
		
		if(mFileWriter == null) {
			return false;
		}
		String line = "";
		for(int i = 0; i < mIndentation; ++i) {
			line += "\t";
		}
		String[] lines = xml.split("\n");
		try {
			for(int i = 0; i < lines.length; ++i) {
				mFileWriter.write(line + lines[i] + "\n");
			}
			mFileWriter.flush();
		} catch (IOException e) {
			return false;
		}
		
		if(!closingTag.equals("")) {
			mIndentation++;
			mClosingTags.addFirst(closingTag);
		}
		return true;
	}
	
	public synchronized boolean closeTag() {
		if(mClosingTags.isEmpty()) {
			return false;
		}
		mIndentation--;
		logXML((String) mClosingTags.removeFirst(), "");
		return true;
	}
	
	public synchronized boolean logComment(String comment) {
		if(mFileWriter == null) {
			return false;
		}
		String line = "";
		for(int i = 0; i < mIndentation; ++i) {
			line += "\t";
		}
		String[] lines = comment.split("\n");
		try {
			mFileWriter.write(line + "<!-- ");
			for(int i = 0; i < lines.length; ++i) {
				if(i == 0) {
					mFileWriter.write(lines[i]);
				}
				else {
					mFileWriter.write(line + "\t" + lines[i]);
				}
				if(i < lines.length - 1) {
					mFileWriter.write("\n");
				}
				else {
					mFileWriter.write(" -->\n");
				}
			}
			mFileWriter.flush();
		} catch (IOException e) {
			return false;
		}
		return true;
	}
	
	public synchronized void closeEvolutionLogger() {
		if(mFileWriter != null && mActive) {
			try {
				while(!mClosingTags.isEmpty()) {
					closeTag();
				}
				
				addFooter();
				mFileWriter.close();
				mActive = false;
			} catch (IOException e) {
			}
		}
		mFileWriter = null;
	}
	
	protected void addHeader() {
		if(mFileWriter == null) {
			return;
		}
		try {
			mFileWriter.write("<!-- Logger Time: " + mDateFormatter.format(new Date(System.currentTimeMillis())) + " -->\n");
			mFileWriter.write("<evolution>\n");
			mFileWriter.flush();
			mIndentation++;
			
		} catch (IOException e) {
			return;
		}
	}
	
	protected void addFooter() {
		if(mFileWriter == null) {
			return;
		}
		try {
			mFileWriter.write("</evolution>\n");
			mFileWriter.write("<!-- End Time: " + mDateFormatter.format(new Date(System.currentTimeMillis())) + " -->\n");
			mFileWriter.flush();
			mIndentation--;
			
		} catch (IOException e) {
			return;
		}
	}
	
	public boolean isActive() {
		return mActive;
	}
	
	public String getFileName() {
		return mFileName;
	}

}
