package de.fhg.iais.outdoor.simba.evolution;

import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;

public class EvolutionOperatorESMutateStepSize extends EvolutionOperator {

	private NormalizedDoubleValue mProbabilityOfChange;
	private DoubleValue mMaxChange;
	private DoubleValue mMaxStepSize;
	private DoubleValue mMinStepSize;
	
	public EvolutionOperatorESMutateStepSize() {
		super("ESMutateStepSize", 1);
		
		mProbabilityOfChange = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		mMaxChange = new DoubleValue(0.5);
		mMaxStepSize = new DoubleValue(1.0);
		mMinStepSize = new DoubleValue(0.05);
		
		
		addParameter(mProbabilityOfChange, "ChangeProbability");
		addParameter(mMaxChange, "MaxChange");
		addParameter(mMaxStepSize, "MaxStepSize");
		addParameter(mMinStepSize, "MinStepSize");
	}
	
	
	public boolean executeOperation(EvolutionIndividual individual) {

		Double stepSize = (Double) individual.getProperty("ESStepSize");
		if(stepSize == null) {
			stepSize = new Double(mMaxStepSize.get());
			individual.setProperty("ESStepSize", stepSize);
			return true;
		}
		
		if(mRandom.nextDouble() < mProbabilityOfChange.get()) {
			double step = stepSize.doubleValue() + (((mRandom.nextDouble() * 2) - 1) * mMaxChange.get());
			step = Math.max(mMinStepSize.get(), Math.min(mMaxStepSize.get(), step));
			individual.setProperty("ESStepSize", new Double(step));
		}
		
		return true;
	}

}
