package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNAddNeuron extends EvolutionOperator {

	private NormalizedDoubleValue mInsertProbability;
	private IntValue mMaxNumberOfNewNeurons;
	private IntValue mMaxNumberOfHiddenNeurons;
	
	public EvolutionOperatorNNAddNeuron() {
		super("NNAddNeuron", 500);
	
		mInsertProbability = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		addParameter(mInsertProbability, "InsertionProbability");
		
		mMaxNumberOfNewNeurons = new IntValue(1);
		addParameter(mMaxNumberOfNewNeurons, "MaxNumberOfNewNeurons");
		
		mMaxNumberOfHiddenNeurons = new IntValue(100);
		addParameter(mMaxNumberOfHiddenNeurons, "MaxNumberOfHiddenNeurons");
	}

	public boolean executeOperation(EvolutionIndividual individual) {

		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector neurons = net.getNeurons();
		//Determine the number of hidden neurons in the network.
		int numberOfHiddenNeurons = 0;
		for(Iterator i = neurons.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if(n.getType().equals(Neuron.HIDDEN)) {
				numberOfHiddenNeurons++;
			}
		}
		
		int maxNumberOfNewNeurons = mMaxNumberOfNewNeurons.get();
		int maxNumberOfHiddenNeurons = mMaxNumberOfHiddenNeurons.get();
		for(int i = 0; i < maxNumberOfNewNeurons; ++i) {
			//do not insert more neurons that allowed.
			if(numberOfHiddenNeurons >= maxNumberOfHiddenNeurons) {
				break;
			}
			if(mRandom.nextDouble() < mInsertProbability.get()) {
				//insert neuron
				Neuron neuron = new Neuron("HiddenNeuron");
				neuron.setType(Neuron.HIDDEN);
				net.addNeuron(neuron);
				numberOfHiddenNeurons++;
			}
		}

		return true;
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mInsertProbability = (NormalizedDoubleValue) ValueManager.synchronizeValue(mInsertProbability, oldValue, newValue, this);
		mMaxNumberOfNewNeurons = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfNewNeurons, oldValue, newValue, this);
		mMaxNumberOfHiddenNeurons = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfHiddenNeurons, oldValue, newValue, this);
	}
	
	
}
