package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNChangeBias extends EvolutionOperator {
	private NormalizedDoubleValue mChangeProbability;
	private IntValue mMaxNumberOfChangedNeurons;
	private DoubleValue mMaxBiasChange;
	private DoubleValue mMaxBias;
	
	public EvolutionOperatorNNChangeBias() {
		super("NNChangeBias", 500);

		mChangeProbability = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		addParameter(mChangeProbability, "ChangeProbability");
		
		mMaxBiasChange = new DoubleValue(0.0);
		addParameter(mMaxBiasChange, "MaxBiasChange");
		
		mMaxBias = new DoubleValue(4.0);
		addParameter(mMaxBias, "MaxBias");
		
		mMaxNumberOfChangedNeurons = new IntValue(2);
		addParameter(mMaxNumberOfChangedNeurons, "MaxNumberOfChangedNeurons");
	}

	public boolean executeOperation(EvolutionIndividual individual) {

		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector neuronVector = net.getNeurons();
		
		NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		Vector iNeurons = new Vector();
		if(agentInterface != null) {
			iNeurons = agentInterface.getInputNeurons();
		}
		
		//Remove all locked neurons and all input neurons from the list of change candidates. 
		Vector neurons = new Vector();
		for(Iterator i = neuronVector.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if((!n.getProtectionMode().equals(Neuron.LOCKED)) 
					&& (!n.getProtectionMode().equals(Neuron.PROTECTED))
					&& !iNeurons.contains(n.getActivationValue())) 
			{
				neurons.add(n);
			}
		}
		
		LinkedList list = new LinkedList(neurons);
		
		int maxNumberOfChanged = mMaxNumberOfChangedNeurons.get();
		
		for(int i = 0; i < maxNumberOfChanged && !list.isEmpty(); ++i) {
			
			if(mRandom.nextDouble() < mChangeProbability.get()) {
				int index = mRandom.nextInt(list.size());
				Neuron n = (Neuron) list.get(index);
				list.remove(index);

				//change bias
				double maxStrength = mMaxBias.get();
				
				double change = mMaxBiasChange.get() * mRandom.nextDouble();
				if(mRandom.nextBoolean()) {
					change *= -1;
				}
				change += n.getBias();
				if(change > maxStrength) {
					change = maxStrength;
				}
				else if(change < -maxStrength) {
					change = -maxStrength;
				}
				n.setBias(change);
			}
			
		}

		return true;
	}
	

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mChangeProbability = (NormalizedDoubleValue) ValueManager.synchronizeValue(mChangeProbability, oldValue, newValue, this);
		mMaxNumberOfChangedNeurons = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfChangedNeurons, oldValue, newValue, this);
		mMaxBiasChange = (DoubleValue) ValueManager.synchronizeValue(mMaxBiasChange, oldValue, newValue, this);
		mMaxBias = (DoubleValue) ValueManager.synchronizeValue(mMaxBias, oldValue, newValue, this);
		
	}
}
