package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNInitializeZeroSynapses extends
		EvolutionOperator implements ValueChangedListener  {

	private DoubleValue mMinValue;
	private DoubleValue mMaxValue;
	
	public EvolutionOperatorNNInitializeZeroSynapses() {
		super("NNInitZeroSynapses", 400);

		mMinValue = new DoubleValue(-2.0);
		mMaxValue = new DoubleValue(2.0);
		addParameter(mMinValue, "MinValue");
		addParameter(mMaxValue, "MaxValue");
		
		mMinValue.addValueChangedListener(this);
		mMaxValue.addValueChangedListener(this);
	}

	public boolean executeOperation(EvolutionIndividual individual) {

		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector synapses = net.getSynapses();
		double max = mMaxValue.get();
		double min = mMinValue.get();
		
		for(Iterator i = synapses.iterator(); i.hasNext();) {
			Synapse s = (Synapse) i.next();
			
			if(!s.getProtectionMode().equals(Synapse.FIXED)) {
				if(s.getStrength() > -0.00001 && s.getStrength() < 0.00001) {
					double strength = (mRandom.nextDouble() * (max - min)) + min;
					s.setStrength(strength);
				}
			}
			
		}
		
		return true;
	}
	
	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(value == mMinValue) {
			if(mMinValue.get() > mMaxValue.get()) {
				mMinValue.set(mMaxValue.get());
			}
		}
		else if(value == mMaxValue) {
			if(mMaxValue.get() < mMinValue.get()) {
				mMaxValue.set(mMinValue.get());
			}
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mMinValue = (DoubleValue) ValueManager.synchronizeValue(mMinValue, oldValue, newValue, this);
		mMaxValue = (DoubleValue) ValueManager.synchronizeValue(mMaxValue, oldValue, newValue, this);
		
	}

}
