package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNRemoveNeuron extends EvolutionOperator {
	
	private NormalizedDoubleValue mRemoveProbability;
	private IntValue mMaxNumberOfRemovedNeurons;
	
	public EvolutionOperatorNNRemoveNeuron() {
		super("NNRemoveNeuron", 400);

		mRemoveProbability = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		addParameter(mRemoveProbability, "RemoveProbability");
		
		mMaxNumberOfRemovedNeurons = new IntValue(1);
		addParameter(mMaxNumberOfRemovedNeurons, "MaxNumberOfRemovedNeurons");
		
	}

	public boolean executeOperation(EvolutionIndividual individual) {

		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		Vector ioNeuronWrappers = new Vector();
		if(agentInterface != null) {
			ioNeuronWrappers.addAll(agentInterface.getInputNeurons());
			ioNeuronWrappers.addAll(agentInterface.getOutputNeurons());
		}
		
		Vector neuronVector = net.getNeurons();
		LinkedList removeableNeurons = new LinkedList();
		//Ignore all neurons that are locked (not allowed to be source or target of a synapse).
		for(Iterator i = neuronVector.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if(!n.getProtectionMode().equals(Neuron.NORMAL)) {
				continue;
			}
			if(!n.getType().equals(Neuron.HIDDEN)) {
				continue;
			}
			removeableNeurons.add(n);
		}
		//Remove all neurons from the list of candidates if there is a synapse
		//that is protected of locked that is connected to the neuron.
		Vector synapses = net.getSynapses();
		for(Iterator i = synapses.iterator(); i.hasNext();) {
			Synapse s = (Synapse) i.next();
			if(!s.getProtectionMode().equals(Synapse.DYNAMIC)) {
				removeableNeurons.remove(s.getTargetNeuron());
				removeableNeurons.remove(s.getSourceNeuron());
			}
		}
		
		int maxNumberOfRemovedNeurons = mMaxNumberOfRemovedNeurons.get();
		for(int i = 0; i < maxNumberOfRemovedNeurons && !removeableNeurons.isEmpty(); ++i) {
			if(mRandom.nextDouble() < mRemoveProbability.get()) {
				Neuron n = (Neuron) removeableNeurons.get(mRandom.nextInt(removeableNeurons.size()));
				net.removeNeuron(n);
				removeableNeurons.remove(n);
			}
		}
		
		net.validateNetworkConnections();

		return true;
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mRemoveProbability = (NormalizedDoubleValue) ValueManager.synchronizeValue(mRemoveProbability, oldValue, newValue, this);
		mMaxNumberOfRemovedNeurons = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfRemovedNeurons, oldValue, newValue, this);
		
	}

}
