package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNRemoveSynapse extends EvolutionOperator {

	private NormalizedDoubleValue mRemoveProbability;
	private IntValue mMaxNumberOfRemovedSynapses;
	
	
	public EvolutionOperatorNNRemoveSynapse() {
		super("NNRemoveSynapse", 400);

		mRemoveProbability = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		addParameter(mRemoveProbability, "RemoveProbability");
		
		mMaxNumberOfRemovedSynapses = new IntValue(1);
		addParameter(mMaxNumberOfRemovedSynapses, "MaxNumberOfRemovedSynapses");
	}

	public boolean executeOperation(EvolutionIndividual individual) {

		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector neurons = net.getNeurons();
		
		int maxNumberToRemove = mMaxNumberOfRemovedSynapses.get();
		for(int i = 0; i < maxNumberToRemove; ++i) {
			if(mRandom.nextDouble() < mRemoveProbability.get()) {
				
				//search for candidates to remove
				Vector synapses = net.getSynapses();
				Vector candidates = new Vector();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse s = (Synapse) j.next();
					//remove only dynamice synapses (locked and protected synapses should not be deleted)
					if(s.getProtectionMode().equals(Synapse.DYNAMIC)
							&& !s.getTargetNeuron().getProtectionMode().equals(Neuron.LOCKED)
							&& !s.getSourceNeuron().getProtectionMode().equals(Neuron.LOCKED)) 
					{
						candidates.add(s);
					}
				}
				if(candidates.isEmpty()) {
					break;
				}
				
				Synapse synapseToRemove = (Synapse) candidates.elementAt(mRandom.nextInt(candidates.size()));
				
				for(Iterator j = neurons.iterator(); j.hasNext();) {
					Neuron n = (Neuron) j.next();
					Vector localSynapses = n.getSynapses();
					for(Iterator k = localSynapses.iterator(); k.hasNext();) {
						Synapse s = (Synapse) k.next();
						if(s == synapseToRemove) {
							n.removeSynapse(s);
							break;
						}
					}
				}
			}
		}
		
		return true;
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mRemoveProbability = (NormalizedDoubleValue) ValueManager.synchronizeValue(mRemoveProbability, oldValue, newValue, this);
		mMaxNumberOfRemovedSynapses = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfRemovedSynapses, oldValue, newValue, this);
	}
	
}
