package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.IntValue;

public class EvolutionOperatorNNRestrictSynapses extends EvolutionOperator {

	private IntValue mMaxInSynapsesHiddenNeurons;
	private IntValue mMaxInSynapsesOutputNeurons;
	private IntValue mMaxOutSynapsesHiddenNeurons;
	private IntValue mMaxOutSynapsesInputNeurons;
	private IntValue mMaxOutSynapsesOutputNeurons;
	private BooleanValue mPreserveRestrictions;
	
	
	public EvolutionOperatorNNRestrictSynapses() {
		super("NNRestrictSynapses", 500);
		
		mMaxInSynapsesHiddenNeurons = new IntValue(-1);
		mMaxInSynapsesOutputNeurons = new IntValue(-1);
		mMaxOutSynapsesHiddenNeurons = new IntValue(-1);
		mMaxOutSynapsesInputNeurons = new IntValue(-1);
		mMaxOutSynapsesOutputNeurons = new IntValue(-1);
		mPreserveRestrictions = new BooleanValue(false);
		
		addParameter(mMaxInSynapsesHiddenNeurons, "HiddenMaxIn");
		addParameter(mMaxOutSynapsesHiddenNeurons, "HiddenMaxOut");
		addParameter(mMaxInSynapsesOutputNeurons, "OutputMaxIn");
		addParameter(mMaxOutSynapsesOutputNeurons, "OutputMaxOut");
		addParameter(mMaxOutSynapsesInputNeurons, "InputMaxOut");
		addParameter(mPreserveRestrictions, "PreservePreviousRestrictions");
		
	}
	public boolean executeOperation(EvolutionIndividual individual) {
		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector neurons = net.getNeurons();
		
		int hiddenMaxIn = mMaxInSynapsesHiddenNeurons.get();
		int hiddenMaxOut = mMaxOutSynapsesHiddenNeurons.get();
		int inputMaxOut = mMaxOutSynapsesInputNeurons.get();
		int outputMaxIn = mMaxInSynapsesOutputNeurons.get();
		int outputMaxOut = mMaxOutSynapsesOutputNeurons.get();
		boolean preserve = mPreserveRestrictions.get();
		
		for(Iterator i = neurons.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if(n.getType().equals(Neuron.HIDDEN)) {
				
				if(!(preserve && n.getMaxIncomingSynapsesValue().get() != -1)
					&&	n.getMaxIncomingSynapsesValue().get() != hiddenMaxIn) 
				{
					n.getMaxIncomingSynapsesValue().set(hiddenMaxIn);
				}
				if(!(preserve && n.getMaxOutgoingSynapsesValue().get() != -1)
					&&	n.getMaxOutgoingSynapsesValue().get() != hiddenMaxOut) 
				{
					n.getMaxOutgoingSynapsesValue().set(hiddenMaxOut);
				}
			}
			else if(n.getType().equals(Neuron.INPUT)) {
				if(!(preserve && n.getMaxOutgoingSynapsesValue().get() != -1)
					&&	n.getMaxOutgoingSynapsesValue().get() != inputMaxOut) 
				{
					n.getMaxOutgoingSynapsesValue().set(inputMaxOut);
				}
			}
			else if(n.getType().equals(Neuron.OUTPUT)) {
				if(!(preserve && n.getMaxIncomingSynapsesValue().get() != -1)
					&&	n.getMaxIncomingSynapsesValue().get() != outputMaxIn) 
				{
					n.getMaxIncomingSynapsesValue().set(outputMaxIn);
				}
				if(!(preserve && n.getMaxOutgoingSynapsesValue().get() != -1)
					&&	n.getMaxOutgoingSynapsesValue().get() != outputMaxOut) 
				{
					n.getMaxOutgoingSynapsesValue().set(outputMaxOut);
				}
			}
		}
		
		return true;
	}

}
