package de.fhg.iais.outdoor.simba.evolution;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;

public class EvolutionOperatorNNSimpleParentCopy extends EvolutionOperator {

	
	public EvolutionOperatorNNSimpleParentCopy() {
		super("NNSimpleParentCopy", 1);
	}

	/**
	 * Can work with EvolutionIndividuals that provide at least 1 parent.
	 */
	public boolean executeOperation(EvolutionIndividual individual) {
		
		if(individual == null || individual.getParents().isEmpty()) {
			Logger.getLogger("Simulator")
				.warning("Could not find a parent to copy the neural network from!");
			return true;
		}
		//TODO choose the parent with the best fitness.
		EvolutionIndividual parent = (EvolutionIndividual) individual.getParents().firstElement();
		
		NeuralNet net = null;
		try {
			net = (NeuralNet) parent.getGenom();
		}
		catch(ClassCastException e) {
			net = null;
		}
		
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The genom of the parent was not a NeuralNet object!");
			return true;
		}
		
		//Copy the network
		NeuralNet copy = null;
		try {
			copy = (NeuralNet) net.clone();
		} 
		catch (CloneNotSupportedException e) {
			copy = null;
		}
		if(copy == null) {
			Logger.getLogger("Simulator")
			.warning("Copy not clone the NeuralNet of the parent.");
			return true;
		}
		
		if(!individual.setGenom(copy)) {
			Logger.getLogger("Simulator")
			.warning("Could not set the NeuralNet as genom for the child.");
			return true;
		}
		
		return true;
	}

	public void reset() {
		super.reset();
	}
	
	
	
}
