package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.ConnectivityRating;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;

public class EvolutionOperatorNNStructureValidator extends
		EvolutionOperator 
{

	
	public EvolutionOperatorNNStructureValidator() {
		super("NNStructureValidator", 500);
	}

	public boolean executeOperation(EvolutionIndividual individual) {
		
		
		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector inputNeurons = new Vector();
		Vector outputNeurons = new Vector();
		
		NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		synchronized(net) {
			
			Vector inputNeuronWrappers = agentInterface.getInputNeurons();
			Vector outputNeuronWrappers = agentInterface.getOutputNeurons();
			
			Vector neurons = net.getNeurons();
			
			for(Iterator i = neurons.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				for(Iterator j = inputNeuronWrappers.iterator(); j.hasNext();) {
					NeuronWrapper nw = (NeuronWrapper) j.next();
					if(neuron.getActivationValue() == nw) {
						inputNeurons.add(neuron);
					}
				}
				for(Iterator j = outputNeuronWrappers.iterator(); j.hasNext();) {
					NeuronWrapper nw = (NeuronWrapper) j.next();
					if(neuron.getActivationValue() == nw) {
						outputNeurons.add(neuron);
					}
				}
			}
		
			//make sure that all inout neurons are connected to at least on output neuron
			for(Iterator i = inputNeurons.iterator(); i.hasNext();) {
				if(!ConnectivityRating.isConnected((Neuron) i.next(), outputNeurons)) {
					return false;
				}
			}
			
			for(Iterator i = outputNeurons.iterator(); i.hasNext();) {
				if(!ConnectivityRating.isInverseConnected((Neuron) i.next(), inputNeurons)) {
					return false;
				}
			}
		}
		return true;
	}
	
	
}
