package de.fhg.iais.outdoor.simba.evolution;

import java.io.File;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.SystemObject;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.ObjectValue;
import de.fhg.iais.outdoor.simba.value.StringValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class GenerationAutoLogger extends ParameterElement implements ValueChangedListener,
			EvolutionListener, SystemObject {
	
	private DoubleValue mMinFitness;
	private IntValue mFitnessIndex;
	private BooleanValue mLogIncreaseOnly;
	private DoubleValue mBestFitnessValue;
	private String mLogFilePrefix;
	private StringValue mEvoLogFileName;
	private GenerationFileHandler mGenerationFileHandler;
	private double mBestFitness;
	private BooleanValue mResetBestFitness;
	private DoubleValue mMinFitnessDistance;
	
	public GenerationAutoLogger(String name) {
		super("GenerationAutoLogger_" + name, "/Adjustments/Misc/GenerationAutoLogger_" + name + "/");
	
		mEvoLogFileName = Simulator.getInstance().getScheduler().getLogFileName();
		mEvoLogFileName.addValueChangedListener(this);
		
		mGenerationFileHandler = new GenerationFileHandler();
		
		mMinFitness = new DoubleValue(0.0);
		mFitnessIndex = new IntValue(0);
		mLogIncreaseOnly = new BooleanValue(true);
		mResetBestFitness = new BooleanValue(false);
		mMinFitnessDistance = new DoubleValue(0.0);
		
		mResetBestFitness.addValueChangedListener(this);
		
		addParameter(mMinFitness, "MinFitness");
		addParameter(mFitnessIndex, "FitnessIndex");
		addParameter(mLogIncreaseOnly, "LogOnlyIfFitnessIncreases");
		addParameter(mResetBestFitness, "ResetBestFitness");
		addParameter(mMinFitnessDistance, "MinFitnessDistance");
		
		ObjectValue value = new ObjectValue(this);
		Simulator.getInstance().getValueManager().addValue("/Adjustments/Misc/GenerationAutoLogger_" + name, value);
		
		updateLogFilePrefix();
		
		Simulator.getInstance().addSystemObject(this);
		Simulator.getInstance().getEvolutionManager().addEvolutionListener(this);
	}
	
	public void updateLogFilePrefix() {
		File file = new File(mEvoLogFileName.get());
		String parent = file.getParent();
		if(parent != null) {
			mLogFilePrefix = parent + "\\Gen";
		}
	}
	
	public void updateFitnessValue() {
		Vector fitnessFunctions = Simulator.getInstance().getFitnessManager().getFitnessFunctions();
		int fitnessIndex = mFitnessIndex.get();
		if(fitnessFunctions.size() > fitnessIndex) {
			mBestFitnessValue = ((FitnessFunction) fitnessFunctions.elementAt(fitnessIndex))
						.getGenerationBestFitnessValue();
		}
		else {
			mBestFitnessValue = null;
		}
	}

	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(value == mEvoLogFileName) {
			updateLogFilePrefix();
		}
		else if(value == mFitnessIndex) {
			updateFitnessValue();
		}
		else if(value == mResetBestFitness && mResetBestFitness.get()) {
			mBestFitness = 0.0;
			mResetBestFitness.set(false);
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mEvoLogFileName = (StringValue) ValueManager.synchronizeValue(mEvoLogFileName, oldValue, newValue, this);
		
	}

	public void notifyIndividualsCreated(Vector individuals, int generation) {
		
	}

	public void notifyIndividualsEvaluated(Vector individuals, int generation) {
		if(!mActive.get()) {
			return;
		}
		if(mBestFitnessValue == null) {
			return;
		}
		double currentFitness = mBestFitnessValue.get();
		if(currentFitness < mMinFitness.get()) {
			return;
		}
		
		if(mLogIncreaseOnly.get()) {
			if(currentFitness > (mBestFitness + mMinFitnessDistance.get())) {
				mBestFitness = currentFitness;
			}
			else {
				return;
			}
		}
		
		String fileName = mLogFilePrefix + generation + ".gen";
		
		mGenerationFileHandler.saveGeneration(fileName);
	}

	public void notifyParentsSelected(Vector parents, int generation) {
		
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		updateFitnessValue();
		return true;
	}
	
	
	
	

}
