package de.fhg.iais.outdoor.simba.evolution;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.xml.NeuralNetParser;

public class GenerationFileHandler {

	public GenerationFileHandler() {
		
	}
	
	public boolean saveGeneration(String filename) {
		
		EvolutionLogger logger = new EvolutionLogger();
		int mCurrentGeneration = Simulator.getInstance().getScheduler().getCurrentGeneration();
		if(!logger.startEvolutionLogger(filename)) {
			logger.closeEvolutionLogger();
			return false;
		}
		logger.logXML("<generation id=\"" + mCurrentGeneration + "\">", "</generation>");
		
		Vector individuals = Simulator.getInstance().getEvolutionManager().getGeneration();
		for(Iterator i = individuals.iterator(); i.hasNext();) {
			EvolutionIndividual ind = (EvolutionIndividual) i.next();
			if(ind != null && ind.getGenom() instanceof NeuralNet) {
				NeuralNet net = (NeuralNet) ind.getGenom();
				logger.logXML(NeuralNetParser.createPureSimbaXMLFromNet(net));
			}
		}
		
		logger.closeEvolutionLogger();
		
		return true;
	}
	
	public boolean loadGeneration(String filename) {

		Vector generation = new Vector();
		
		NeuralNetAgentInterface agent = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			
			String line = reader.readLine();
			
			while(line != null) {
				while(line != null && line.indexOf("<net") != -1) {
					StringBuffer xml = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
					do {
						xml.append(line + "\n");
						line = reader.readLine();
					} while(line.indexOf("</net") == -1);
					xml.append(line + "\n");
					
					
					NeuralNet net = NeuralNetParser.createNetFromSimbaXML(xml.toString(), agent);
					//System.out.println("Created net: " + net + "\n" + xml.toString());
					if(net != null) {
						EvolutionIndividual ind = new EvolutionIndividual(net);
						generation.add(ind);
					}
					
					line = reader.readLine();
					
				}
				line = reader.readLine();
			}
			
			if(!generation.isEmpty()) {
				Simulator.getInstance().getEvolutionManager().replaceGeneration(generation);
			}
			
		} catch (FileNotFoundException e) {
			return false;
		} catch (IOException e) {
			return false;
		}
		
		
		return true;
	}
}
