package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.fitness.FitnessManager;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;

public class NeuralNetEvaluationMethod extends EvaluationMethod {
	
	private int mNumberOfTries;
	private int mNumberOfSteps;
	
	public NeuralNetEvaluationMethod() {
		super("NeuralNetEvaluationMethod");
	}

	public void evaluateIndividual(EvolutionIndividual individual, int individualId) {
		
		
		
		if(individual.getGenom() == null) {
			individual.clearFitnessValues();
			return;
		}
		
		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
			individual.clearFitnessValues();
			return;
		}
		
		Simulator.getInstance().getNeuralNetManager().setCurrentNet(net);
		Scheduler scheduler = Simulator.getInstance().getScheduler();
		
		scheduler.prepareNextIndividual(individualId);
		for(int tries = 0; tries < mNumberOfTries; ++tries) {
			scheduler.prepareNextTry(tries);
			for(int steps = 0; steps < mNumberOfSteps; ++steps) {
				scheduler.executeSimulationStep(steps);
				if(scheduler.shouldCurrentTryBeTerminated()) {
					break;
				}
			}
		}
		
		//Store the fitness values.
		individual.clearFitnessValues();
		
		FitnessManager fitnessManager = Simulator.getInstance().getFitnessManager();
		Vector fitnessFunctions  = fitnessManager.getFitnessFunctions();
		for(Iterator i = fitnessFunctions.iterator(); i.hasNext();) {
			FitnessFunction f = (FitnessFunction) i.next();
			if(f.isActive()) {
				individual.addFitnessValue(f.getMeanFitness());
			}
		}
		
		individual.setGenom(Simulator.getInstance().getNeuralNetManager().getCurrentNet());
		
	}

	public void prepareNextGeneration(int generationId) {
		mNumberOfSteps = Simulator.getInstance().getScheduler().getNumberOfSteps();
		mNumberOfTries = Simulator.getInstance().getScheduler().getNumberOfTries();
	}
	
	

}
