package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.xml.NeuralNetParser;

public abstract class NeuralNetSelectionMethod extends SelectionMethod {

	public NeuralNetSelectionMethod(String name) {
		super(name);
	}
	
	/**
	 * Logs the Individuals in the Vector to the evolution logger.
	 * All individuals have to have a NeuralNetwork as genom. If not, then these
	 * Individuals are not logged.
	 * 
	 * @param networks a Vector with the Individuals to log.
	 */
	public void logBestNetworks(Vector networks) {
		
		EvolutionLogger logger = Simulator.getInstance().getScheduler().getEvolutionLogger();
		
		int counter = 0;
		for(Iterator i = networks.iterator(); i.hasNext() && counter < getNumberOfGenomsToLog(); counter++) {
			EvolutionIndividual individual = (EvolutionIndividual) i.next();
			
			NeuralNet net = (NeuralNet) individual.getGenom();
			
			if(net != null) {
				logger.logXML("<individual id=\"" + individual.getId() + "\" age=\"" + individual.getAge() 
								+ "\" numberOfChildren=\"" + individual.getNumberOfChildren() + "\">", "</individual>");
				Vector fitnessValues = individual.getFitnessValues();
				for(int j = 0; j < fitnessValues.size(); ++j) {
					logger.logXML("<fitness index=\"" + j + "\" value=\"" + ((Double) fitnessValues.elementAt(j)).doubleValue() + "\"/>");
				}
				logger.logXML(NeuralNetParser.createPureSimbaXMLFromNet(net));
				logger.closeTag();
			}
		}
	}
	
	public void logGenoms(Vector genoms) {
		logBestNetworks(genoms);
	}

}
