package de.fhg.iais.outdoor.simba.fitness;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.SystemObject;

public class FitnessManager implements SystemObject {
	
	
	private Vector mFitnessFunctions = new Vector();
	
	public FitnessManager() {
		
	}
	
	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}


	public void prepareNextTry(int tryId) {
		//reset the try accumulators of the fitness functions
		synchronized (mFitnessFunctions) {
			for(Iterator i = mFitnessFunctions.iterator(); i.hasNext(); ){
				FitnessFunction f = (FitnessFunction) i.next();
				if(f.isActive()) {
					f.prepareNextTry(tryId);
				}
			}
		}
	}

	public void executeFitnessFunctions() {
		synchronized (mFitnessFunctions) {
			for(Iterator i = mFitnessFunctions.iterator(); i.hasNext(); ){
				FitnessFunction f = (FitnessFunction) i.next();
				if(f.isActive()) {
					f.update();
				}
			}
		}
	}
	
	
	public boolean addFitnessFunction(FitnessFunction fitnessFunction) {
		synchronized (mFitnessFunctions) {
			if(fitnessFunction == null || mFitnessFunctions.contains(fitnessFunction)) {
				return false;
			}
			mFitnessFunctions.add(fitnessFunction);
		}
		return true;
	}
	
	public boolean removeFitnessFunction(FitnessFunction fitnessFunction) {
		synchronized (mFitnessFunctions) {
			if(fitnessFunction == null || !mFitnessFunctions.contains(fitnessFunction)) {
				return false;
			}
			mFitnessFunctions.remove(fitnessFunction);
		}
		return true;
	}
	
	public Vector getFitnessFunctions() {
		synchronized (mFitnessFunctions) {
			return new Vector(mFitnessFunctions);
		}
	}
	
	public void removeAllFitnessFunctions() {
		synchronized (mFitnessFunctions) {
			mFitnessFunctions.clear();
		}
	}

}
