package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class DelayedResponseMazeT1Fitness1 extends FitnessFunction {

	private BooleanValue mHitTargetLeft;
	private BooleanValue mHitTargetRight;
	private DoubleValue mDistTargetLeft;
	private DoubleValue mDistTargetRight;
	private DoubleValue mTargetZoneLeft;
	private double mInitDistance;
	private double mMinDistance;
	
	
	public DelayedResponseMazeT1Fitness1() {
		super("DelayedResponseMazeT1Fitness1");
		
	}

	public boolean init() {
		boolean ok = super.init();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mHitTargetLeft = vm.getBooleanValue("/SimObjects/HitTargetLeft/BumpDetected");
		mHitTargetRight = vm.getBooleanValue("/SimObjects/HitTargetRight/BumpDetected");
		mDistTargetLeft = vm.getDoubleValue("/SimObjects/DistanceToLeftArea/Distance");
		mDistTargetRight = vm.getDoubleValue("/SimObjects/DistanceToRightArea/Distance");
		mTargetZoneLeft = vm.getDoubleValue("/FitnessInfo/TargetZoneLeft");
		
		if(mHitTargetLeft == null || mHitTargetRight == null
			|| mDistTargetLeft == null || mDistTargetRight == null
			|| mTargetZoneLeft == null)
		{
			ok = false;
			Logger.getLogger("Simulator")
				.warning("DelayedResponseMazeT1Fitness1.init: Did not find all required Value objects.\n"
						+ mHitTargetLeft + "\n" + mHitTargetRight + "\n" + mDistTargetLeft + "\n" + mDistTargetRight
						+ "\n" + mTargetZoneLeft);
		}
		else {
			mHitTargetLeft.addValueReplacedListener(this);
			mHitTargetRight.addValueReplacedListener(this);
			mDistTargetLeft.addValueReplacedListener(this);
			mDistTargetRight.addValueReplacedListener(this);
			mTargetZoneLeft.addValueReplacedListener(this);
		}
		
		return ok;
	}

	public void reset() {
		mInitDistance = Double.MAX_VALUE;
		mMinDistance = Double.MAX_VALUE;
	}

	public double calculateFitness() {
		
		if(mHitTargetLeft == null || mHitTargetRight == null
				|| mDistTargetLeft == null || mDistTargetRight == null
				|| mTargetZoneLeft == null)
		{
			return 0.0;
		}
		
		//determine target zone
		double dist = Double.MAX_VALUE;
//		double distOther = Double.MAX_VALUE;
		
		if(mTargetZoneLeft.get() == 0.0) {
			dist = mDistTargetRight.get();
//			distOther = mDistTargetLeft.get();
		}
		else {
			dist = mDistTargetLeft.get();
//			distOther = mDistTargetRight.get();
		}
		if(mInitDistance >= Double.MAX_VALUE) {
			mInitDistance = dist;
		}
		
		if(dist <= mMinDistance) {
//			mMinDistance = dist;
		}
//		if(distOther < dist) {
//			mMinDistance = dist; //if the distance to the wrong zone is smaller than the best distance to the correct one, then the mindist is reset to the current distance.
//		}
		
		
//		setFitness( Math.pow((mInitDistance - mMinDistance), 2) / 1000.0 );
		
		return (Math.pow((mInitDistance - dist), 2.0) / 1000.0);
		
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		
		mHitTargetLeft = (BooleanValue) ValueManager.synchronizeValue(mHitTargetLeft, oldValue, newValue, this);
		mHitTargetRight = (BooleanValue) ValueManager.synchronizeValue(mHitTargetRight, oldValue, newValue, this);
		mDistTargetLeft = (DoubleValue) ValueManager.synchronizeValue(mDistTargetLeft, oldValue, newValue, this);
		mDistTargetRight = (DoubleValue) ValueManager.synchronizeValue(mDistTargetRight, oldValue, newValue, this);
		mTargetZoneLeft = (DoubleValue) ValueManager.synchronizeValue(mTargetZoneLeft, oldValue, newValue, this);
	}
	
	
	
	
}
