package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.neuralnet.ConnectivityRating;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class DelayedResponseMazeT1WarpCheckPoint2 extends
		FitnessFunction 
{

	private BooleanValue mHitTargetLeft;
	private BooleanValue mHitTargetRight;
	private DoubleValue mDistTargetLeft;
	private DoubleValue mDistTargetRight;
	private DoubleValue mTargetZoneLeft;
	private SimObjectValue mRobot;
	private SimObjectValue mLeftConnector;
	private double mInitDistance;
	private double mAccu;
	private boolean mFirstOccuranceOfLeft;
	private boolean mFirstOccuranceOfRight;
	private double mMinimalGain;
	private double mFirstOccurenceMinimalGain;
	private boolean mFitnessIsMinimalGain;
	private int mNumberOfCyclesWithoutIncrease;
	private double mBestDistance;
	private IntValue mNumberOfMaximalStepsWithoutIncrease;
	private boolean mNetStructureValid;
	private BooleanValue mTerminateInvalidNets;
	
	
	public DelayedResponseMazeT1WarpCheckPoint2() {
		super("DelayedResponseMazeT1WarpCheckPoint2");
		mAccu = 0.0;
		mInitDistance = Double.MAX_VALUE;
		
		mNumberOfMaximalStepsWithoutIncrease = new IntValue(50);
		addParameter(mNumberOfMaximalStepsWithoutIncrease, "MaximalNumberOfStepsWithoutIncrease");
		
		mTerminateInvalidNets = new BooleanValue(true);
		addParameter(mTerminateInvalidNets, "TerminateInvalidNets");
	}

	public boolean init() {
		boolean ok = super.init();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mHitTargetLeft = vm.getBooleanValue("/SimObjects/HitTargetLeft/BumpDetected");
		mHitTargetRight = vm.getBooleanValue("/SimObjects/HitTargetRight/BumpDetected");
		mDistTargetLeft = vm.getDoubleValue("/SimObjects/DistanceToLeftArea/Distance");
		mDistTargetRight = vm.getDoubleValue("/SimObjects/DistanceToRightArea/Distance");
		mTargetZoneLeft = vm.getDoubleValue("/FitnessInfo/TargetZoneLeft");
		mRobot = vm.getSimObjectValue("/SimObjects/Robot");
		mLeftConnector = vm.getSimObjectValue("/SimObjects/LeftConnector");
		
		if(mHitTargetLeft == null || mHitTargetRight == null
			|| mDistTargetLeft == null || mDistTargetRight == null
			|| mTargetZoneLeft == null || mRobot == null 
			|| mLeftConnector == null)
		{
			ok = false;
			Logger.getLogger("Simulator")
				.warning(getName() + ".init: Did not find all required Value objects.\n"
						+ mHitTargetLeft + "\n" + mHitTargetRight + "\n" + mDistTargetLeft + "\n" + mDistTargetRight
						+ "\n" + mTargetZoneLeft);
		}
		else {
			mHitTargetLeft.addValueReplacedListener(this);
			mHitTargetRight.addValueReplacedListener(this);
			mDistTargetLeft.addValueReplacedListener(this);
			mDistTargetRight.addValueReplacedListener(this);
			mTargetZoneLeft.addValueReplacedListener(this);
			mRobot.addValueReplacedListener(this);
			mLeftConnector.addValueReplacedListener(this);
		}
		return ok;
	}

	public void reset() {
		mInitDistance = Double.MAX_VALUE;
		mAccu = 0.0;
		mFirstOccuranceOfLeft = true;
		mFirstOccuranceOfRight = true;
		mMinimalGain = 0.0;
		mFitnessIsMinimalGain = false;
		mFirstOccurenceMinimalGain = 0.0;
		mNumberOfCyclesWithoutIncrease = 0;
		mBestDistance = Double.MAX_VALUE;
		mNetStructureValid = false;
	}

	public double calculateFitness() {
		
		if(mHitTargetLeft == null || mHitTargetRight == null
				|| mDistTargetLeft == null || mDistTargetRight == null
				|| mTargetZoneLeft == null)
		{
			return 0.0;
		}
		
		if(mTerminateInvalidNets.get() && !mNetStructureValid) {
			//Check netstructure.
			
			Vector inputNeurons = new Vector();
			Vector outputNeurons = new Vector();
			
			NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
			NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
			synchronized(net) {
				Vector inputNeuronWrappers = agentInterface.getInputNeurons();
				Vector outputNeuronWrappers = agentInterface.getOutputNeurons();
				
				Vector neurons = net.getNeurons();
				
				for(Iterator i = neurons.iterator(); i.hasNext();) {
					Neuron neuron = (Neuron) i.next();
					for(Iterator j = inputNeuronWrappers.iterator(); j.hasNext();) {
						NeuronWrapper nw = (NeuronWrapper) j.next();
						if(neuron.getActivationValue() == nw) {
							inputNeurons.add(neuron);
						}
					}
					for(Iterator j = outputNeuronWrappers.iterator(); j.hasNext();) {
						NeuronWrapper nw = (NeuronWrapper) j.next();
						if(neuron.getActivationValue() == nw) {
							outputNeurons.add(neuron);
						}
					}
				}
			}
			mNetStructureValid = true;
			int countFailures = 0;
			//make sure that all inout neurons are connected to at least on output neuron
			for(Iterator i = inputNeurons.iterator(); i.hasNext();) {
				if(!ConnectivityRating.isConnected((Neuron) i.next(), outputNeurons)) {
					mNetStructureValid = false;
					countFailures++;
				}
			}
			
			for(Iterator i = outputNeurons.iterator(); i.hasNext();) {
				if(!ConnectivityRating.isInverseConnected((Neuron) i.next(), inputNeurons)) {
					mNetStructureValid = false;
					countFailures++;
				}
			}
			
			
			if(!mNetStructureValid) {
				Simulator.getInstance().getScheduler().terminateCurrentTry();
				return (10.0 - ((double) countFailures)) / 10000000.0; 
			}
		}
		

		
		//determine target zone
		double dist = Double.MAX_VALUE;
		
		if(mTargetZoneLeft.get() == 0.0) {
			dist = mDistTargetRight.get();
		}
		else {
			dist = mDistTargetLeft.get();
		}
		if(mInitDistance >= Double.MAX_VALUE) {
			mInitDistance = dist;
		}
		mNumberOfCyclesWithoutIncrease++;
		if(dist < (mBestDistance - 0.02)) {
			mBestDistance = dist;
			mNumberOfCyclesWithoutIncrease = 0;
		}
		
		if(mNumberOfCyclesWithoutIncrease > mNumberOfMaximalStepsWithoutIncrease.get()) {
			Simulator.getInstance().getScheduler().terminateCurrentTry();
		}
		
		if(mTargetZoneLeft.get() > 0.0) {
			if(mHitTargetLeft.get()) {
				mAccu += Math.pow((mInitDistance - dist), 2);
				mMinimalGain = 0.0;
				mInitDistance = Double.MAX_VALUE;
				dist = Double.MAX_VALUE;
				mFirstOccuranceOfLeft = false;
			}
			else if(mHitTargetRight.get() && mFirstOccuranceOfLeft) {
				mFitnessIsMinimalGain = true;
			}
		}
		else {
			if(mHitTargetRight.get()) {
				mAccu += Math.pow((mInitDistance - dist), 2);
				mMinimalGain = 0.0;
				mInitDistance = Double.MAX_VALUE;
				dist = Double.MAX_VALUE;
				mFirstOccuranceOfRight = false;
			}
			else if(mHitTargetLeft.get() && mFirstOccuranceOfRight) {
				mFitnessIsMinimalGain = true;
			}
		}
	
		if(mMinimalGain == 0.0) { 
			Box leftConnector = (Box) mLeftConnector.get();
			if(mRobot.get().getGlobalPosition().y >= 
				leftConnector.getGlobalPosition().y + (leftConnector.getHeight() / 2.0))
			{
				mMinimalGain = (mInitDistance - dist);
			}
			if(mFirstOccurenceMinimalGain == 0.0) {
				mFirstOccurenceMinimalGain = mMinimalGain;
			}
		}
	
//		System.out.println("Akku " + mAccu + "  Dist: " + (mInitDistance - dist) + " InitDist: " + mInitDistance);
		if(mFitnessIsMinimalGain) {
			return (Math.pow((mFirstOccurenceMinimalGain), 2)  / 10000.0);
		}
		else {
			return ((mAccu + (Math.pow((Math.max((mInitDistance - dist), mMinimalGain)), 2))) / 10000.0);
		}
		
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		
		mHitTargetLeft = (BooleanValue) ValueManager.synchronizeValue(mHitTargetLeft, oldValue, newValue, this);
		mHitTargetRight = (BooleanValue) ValueManager.synchronizeValue(mHitTargetRight, oldValue, newValue, this);
		mDistTargetLeft = (DoubleValue) ValueManager.synchronizeValue(mDistTargetLeft, oldValue, newValue, this);
		mDistTargetRight = (DoubleValue) ValueManager.synchronizeValue(mDistTargetRight, oldValue, newValue, this);
		mTargetZoneLeft = (DoubleValue) ValueManager.synchronizeValue(mTargetZoneLeft, oldValue, newValue, this);
		mRobot = (SimObjectValue) ValueManager.synchronizeValue(mRobot, oldValue, newValue, this);
		mLeftConnector = (SimObjectValue) ValueManager.synchronizeValue(mLeftConnector, oldValue, newValue, this);
		
	}

}
