package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class DelayedResponseMazeT1WarpFitnessCheckPoints extends
		FitnessFunction 
{

	private BooleanValue mHitTargetLeft;
	private BooleanValue mHitTargetRight;
	private DoubleValue mDistTargetLeft;
	private DoubleValue mDistTargetRight;
	private DoubleValue mTargetZoneLeft;
	private SimObjectValue mRobot;
	private SimObjectValue mLeftConnector;
	private double mInitDistance;
	private double mAccu;
	private boolean mFirstOccuranceOfLeft;
	private boolean mFirstOccuranceOfRight;
	private double mMinimalGain;
	private double mFirstOccurenceMinimalGain;
	private boolean mFitnessIsMinimalGain;
	private int mNumberOfCyclesWithoutIncrease;
	private double mBestDistance;
	private IntValue mNumberOfMaximalStepsWithoutIncrease;
	
	
	public DelayedResponseMazeT1WarpFitnessCheckPoints() {
		super("DelayedResponseMazeT1WarpFitnessCheckPoints");
		mAccu = 0.0;
		mInitDistance = Double.MAX_VALUE;
		
		mNumberOfMaximalStepsWithoutIncrease = new IntValue(100);
		Simulator.getInstance().getValueManager()
			.addValue("/FitnessFunctions/" + getName() + "/MaxStepsWithoutIncrease", mNumberOfMaximalStepsWithoutIncrease);
	}

	public boolean init() {
		boolean ok = super.init();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mHitTargetLeft = vm.getBooleanValue("/SimObjects/HitTargetLeft/BumpDetected");
		mHitTargetRight = vm.getBooleanValue("/SimObjects/HitTargetRight/BumpDetected");
		mDistTargetLeft = vm.getDoubleValue("/SimObjects/DistanceToLeftArea/Distance");
		mDistTargetRight = vm.getDoubleValue("/SimObjects/DistanceToRightArea/Distance");
		mTargetZoneLeft = vm.getDoubleValue("/FitnessInfo/TargetZoneLeft");
		mRobot = vm.getSimObjectValue("/SimObjects/Robot");
		mLeftConnector = vm.getSimObjectValue("/SimObjects/LeftConnector");
		
		if(mHitTargetLeft == null || mHitTargetRight == null
			|| mDistTargetLeft == null || mDistTargetRight == null
			|| mTargetZoneLeft == null || mRobot == null 
			|| mLeftConnector == null)
		{
			ok = false;
			Logger.getLogger("Simulator")
				.warning(getName() + ".init: Did not find all required Value objects.\n"
						+ mHitTargetLeft + "\n" + mHitTargetRight + "\n" + mDistTargetLeft + "\n" + mDistTargetRight
						+ "\n" + mTargetZoneLeft);
		}
		else {
			mHitTargetLeft.addValueReplacedListener(this);
			mHitTargetRight.addValueReplacedListener(this);
			mDistTargetLeft.addValueReplacedListener(this);
			mDistTargetRight.addValueReplacedListener(this);
			mTargetZoneLeft.addValueReplacedListener(this);
			mRobot.addValueReplacedListener(this);
			mLeftConnector.addValueReplacedListener(this);
		}
		return ok;
	}

	public void reset() {
		mInitDistance = Double.MAX_VALUE;
		mAccu = 0.0;
		mFirstOccuranceOfLeft = true;
		mFirstOccuranceOfRight = true;
		mMinimalGain = 0.0;
		mFitnessIsMinimalGain = false;
		mFirstOccurenceMinimalGain = 0.0;
		mNumberOfCyclesWithoutIncrease = 0;
		mBestDistance = Double.MAX_VALUE;
	}

	public double calculateFitness() {
		
		if(mHitTargetLeft == null || mHitTargetRight == null
				|| mDistTargetLeft == null || mDistTargetRight == null
				|| mTargetZoneLeft == null)
		{
			return 0.0;
		}
		

		
		//determine target zone
		double dist = Double.MAX_VALUE;
		
		if(mTargetZoneLeft.get() == 0.0) {
			dist = mDistTargetRight.get();
		}
		else {
			dist = mDistTargetLeft.get();
		}
		if(mInitDistance >= Double.MAX_VALUE) {
			mInitDistance = dist;
		}
		mNumberOfCyclesWithoutIncrease++;
		if(dist < (mBestDistance - 0.01)) {
			mBestDistance = dist;
			mNumberOfCyclesWithoutIncrease = 0;
		}
		
		if(mNumberOfCyclesWithoutIncrease > mNumberOfMaximalStepsWithoutIncrease.get()) {
			Simulator.getInstance().getScheduler().terminateCurrentTry();
		}
		
		if(mTargetZoneLeft.get() > 0.0) {
			if(mHitTargetLeft.get()) {
				mAccu += Math.pow((mInitDistance - dist), 2);
				mMinimalGain = 0.0;
				mInitDistance = Double.MAX_VALUE;
				dist = Double.MAX_VALUE;
				mFirstOccuranceOfLeft = false;
			}
			else if(mHitTargetRight.get() && mFirstOccuranceOfLeft) {
				mFitnessIsMinimalGain = true;
			}
		}
		else {
			if(mHitTargetRight.get()) {
				mAccu += Math.pow((mInitDistance - dist), 2);
				mMinimalGain = 0.0;
				mInitDistance = Double.MAX_VALUE;
				dist = Double.MAX_VALUE;
				mFirstOccuranceOfRight = false;
			}
			else if(mHitTargetLeft.get() && mFirstOccuranceOfRight) {
				mFitnessIsMinimalGain = true;
			}
		}
	
		if(mMinimalGain == 0.0) { 
			Box leftConnector = (Box) mLeftConnector.get();
			if(mRobot.get().getGlobalPosition().y >= 
				leftConnector.getGlobalPosition().y + (leftConnector.getHeight() / 2.0))
			{
				mMinimalGain = (mInitDistance - dist);
			}
			if(mFirstOccurenceMinimalGain == 0.0) {
				mFirstOccurenceMinimalGain = mMinimalGain;
			}
		}
	
//		System.out.println("Akku " + mAccu + "  Dist: " + (mInitDistance - dist) + " InitDist: " + mInitDistance);
		if(mFitnessIsMinimalGain) {
			return (Math.pow((mFirstOccurenceMinimalGain / 10000.0), 2));
		}
		else {
			return ((mAccu + (Math.pow((Math.max((mInitDistance - dist), mMinimalGain)), 2))) / 10000.0);
		}
		
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		
		mHitTargetLeft = (BooleanValue) ValueManager.synchronizeValue(mHitTargetLeft, oldValue, newValue, this);
		mHitTargetRight = (BooleanValue) ValueManager.synchronizeValue(mHitTargetRight, oldValue, newValue, this);
		mDistTargetLeft = (DoubleValue) ValueManager.synchronizeValue(mDistTargetLeft, oldValue, newValue, this);
		mDistTargetRight = (DoubleValue) ValueManager.synchronizeValue(mDistTargetRight, oldValue, newValue, this);
		mTargetZoneLeft = (DoubleValue) ValueManager.synchronizeValue(mTargetZoneLeft, oldValue, newValue, this);
		mRobot = (SimObjectValue) ValueManager.synchronizeValue(mRobot, oldValue, newValue, this);
		mLeftConnector = (SimObjectValue) ValueManager.synchronizeValue(mLeftConnector, oldValue, newValue, this);
		
	}
	
}
