package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class DelayedResponseMazeT1WarpFitnessPunish extends FitnessFunction {

	private BooleanValue mHitTargetLeft;
	private BooleanValue mHitTargetRight;
	private DoubleValue mDistTargetLeft;
	private DoubleValue mDistTargetRight;
	private DoubleValue mTargetZoneLeft;
	private double mInitDistance;
	private double mAccu;
	private boolean mFirstOccuranceOfLeft;
	private boolean mFirstOccuranceOfRight;
	private double mFirstOccurancePunishment;
	
	
	public DelayedResponseMazeT1WarpFitnessPunish() {
		super("DelayedResponseMaze1WarpFitnessPunish");
		mAccu = 0.0;
		mInitDistance = Double.MAX_VALUE;
	}

	public boolean init() {
		boolean ok = super.init();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mHitTargetLeft = vm.getBooleanValue("/SimObjects/HitTargetLeft/BumpDetected");
		mHitTargetRight = vm.getBooleanValue("/SimObjects/HitTargetRight/BumpDetected");
		mDistTargetLeft = vm.getDoubleValue("/SimObjects/DistanceToLeftArea/Distance");
		mDistTargetRight = vm.getDoubleValue("/SimObjects/DistanceToRightArea/Distance");
		mTargetZoneLeft = vm.getDoubleValue("/FitnessInfo/TargetZoneLeft");
		
		if(mHitTargetLeft == null || mHitTargetRight == null
			|| mDistTargetLeft == null || mDistTargetRight == null
			|| mTargetZoneLeft == null)
		{
			ok = false;
			Logger.getLogger("Simulator")
				.warning("DelayedResponseMazeT1Fitness1.init: Did not find all required Value objects.\n"
						+ mHitTargetLeft + "\n" + mHitTargetRight + "\n" + mDistTargetLeft + "\n" + mDistTargetRight
						+ "\n" + mTargetZoneLeft);
		}
		else {
			mHitTargetLeft.addValueReplacedListener(this);
			mHitTargetRight.addValueReplacedListener(this);
			mDistTargetLeft.addValueReplacedListener(this);
			mDistTargetRight.addValueReplacedListener(this);
			mTargetZoneLeft.addValueReplacedListener(this);
		}
		return ok;
	}

	public void reset() {
		mInitDistance = Double.MAX_VALUE;
		mAccu = 0.0;
		mFirstOccuranceOfLeft = true;
		mFirstOccuranceOfRight = true;
		mFirstOccurancePunishment = 1.0; //is a factor, therefore 1.0 means "no punishment", 0.0 < values < 1.0 => punish.
	}

	public double calculateFitness() {
		
		if(mHitTargetLeft == null || mHitTargetRight == null
				|| mDistTargetLeft == null || mDistTargetRight == null
				|| mTargetZoneLeft == null)
		{
			return 0.0;
		}
		

		
		//determine target zone
		double dist = Double.MAX_VALUE;
		
		if(mTargetZoneLeft.get() == 0.0) {
			dist = mDistTargetRight.get();
		}
		else {
			dist = mDistTargetLeft.get();
		}
		if(mInitDistance >= Double.MAX_VALUE) {
			mInitDistance = dist;
		}
		
		if(mTargetZoneLeft.get() > 0.0) {
			if(mHitTargetLeft.get()) {
				mAccu += Math.pow((mInitDistance - dist), 2);
				mInitDistance = Double.MAX_VALUE;
				dist = Double.MAX_VALUE;
				mFirstOccuranceOfLeft = false;
			}
			else if(mHitTargetRight.get() && mFirstOccuranceOfLeft) {
				System.out.println("right");
				mFirstOccurancePunishment = 0.5;
				System.out.println("Punish right");
			}
		}
		else {
			if(mHitTargetRight.get()) {
				mAccu += Math.pow((mInitDistance - dist), 2);
				mInitDistance = Double.MAX_VALUE;
				dist = Double.MAX_VALUE;
				mFirstOccuranceOfRight = false;
			}
			else if(mHitTargetLeft.get() && mFirstOccuranceOfRight) {
				System.out.println("left");
				mFirstOccurancePunishment = 0.5;
			}
		}
	
	
//		System.out.println("Akku " + mAccu + "  Dist: " + (mInitDistance - dist) + " InitDist: " + mInitDistance);
		
		return ((mAccu + (Math.pow((mInitDistance - dist), 2))) / 10000.0 * mFirstOccurancePunishment);
		
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		
		mHitTargetLeft = (BooleanValue) ValueManager.synchronizeValue(mHitTargetLeft, oldValue, newValue, this);
		mHitTargetRight = (BooleanValue) ValueManager.synchronizeValue(mHitTargetRight, oldValue, newValue, this);
		mDistTargetLeft = (DoubleValue) ValueManager.synchronizeValue(mDistTargetLeft, oldValue, newValue, this);
		mDistTargetRight = (DoubleValue) ValueManager.synchronizeValue(mDistTargetRight, oldValue, newValue, this);
		mTargetZoneLeft = (DoubleValue) ValueManager.synchronizeValue(mTargetZoneLeft, oldValue, newValue, this);
	}
	
}
