package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;


public class LightChainFrontFitness1 extends FitnessFunction {

	DoubleValue mHintColorValue;
	DoubleValue mSensorColorValue;
	DoubleValue mHintSensor;
	double mAccu;
	SimObjectValue mHintBeacon;
	SimObjectValue mGoalBeacon;
	OmniDrive mRobot;
	DoubleValue mBeaconDistance;
	double mLastPosition;
	DoubleValue mApproachFactor;
	BooleanValue mWaitForHint;
	boolean mHintReceived;
	
	
	public LightChainFrontFitness1() {
		super("LightChainFrontFitness1");
	
		mApproachFactor = new DoubleValue(1.0);
		addParameter(mApproachFactor, "ApproachFactor");
		
		mWaitForHint = new BooleanValue(true);
		addParameter(mWaitForHint, "WaitForHint");
	}
	
	

	public boolean init() {
		boolean ok =  super.init();
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mHintColorValue = vm.getDoubleValue("/Environment/HintColor");
		mSensorColorValue = vm.getDoubleValue("/SimObjects/LightSensor/Brightness");
		mHintBeacon = vm.getSimObjectValue("/Environment/HintBeacon");
		mGoalBeacon = vm.getSimObjectValue("/Environment/GoalBeacon");
		mBeaconDistance = vm.getDoubleValue("/Environment/LightChain/BeaconDistance");
		mHintSensor = vm.getDoubleValue("/SimObjects/HintSensor/Brightness");
		
		SimObjectValue robotValue = vm.getSimObjectValue("/SimObjects/Robot");
		if(robotValue != null && robotValue.get() != null && (robotValue.get() instanceof OmniDrive)) {
			mRobot = (OmniDrive) robotValue.get();
		}
		
		
		
		
		if(mHintColorValue == null || mSensorColorValue == null || mHintBeacon == null
				|| mGoalBeacon == null || mBeaconDistance == null || mHintSensor == null) 
		{
			Logger.getLogger("Simulator").warning("Could not find the required Value objects");
			return false;
		}
		
		return ok;
	}



	public double calculateFitness() {
		
		if(mHintSensor.get() > 0.05) {
			mHintReceived = true;
		}
		
		if(mHintReceived || !mWaitForHint.get()) {
			double colorDifference = Math.abs(mHintColorValue.get() - mSensorColorValue.get());
			
			if(colorDifference < 0.05) {
				mAccu++;
			}
			else {
				
				double bonus = (mRobot.getGlobalPosition().x - mLastPosition) / (mGoalBeacon.get().getGlobalPosition().x - mHintBeacon.get().getGlobalPosition().x) * mApproachFactor.get();
				if(mRobot.getGlobalPosition().x <= mGoalBeacon.get().getGlobalPosition().x) {
					mAccu += bonus;
				}
				else {
					mAccu -= bonus;
				}
				mLastPosition = mRobot.getGlobalPosition().x;
			}
		}
		return mAccu;
	}

	public void reset() {
		mAccu = 0.0;
		mLastPosition = mHintBeacon.get().getGlobalPosition().x;
		mHintReceived = false;
	}


}
