package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class PathMemorizerFitness1 extends FitnessFunction {
	
	private SimObject mRightBound;
	private SimObject mLeftBound;
	private SimObject mRobot;
	private boolean mMovingLeft;
	private double mLastXPosition;
	private BooleanValue mMovingLeftValue;
	private double mAccumulator;
	
	public PathMemorizerFitness1() {
		super("PathMemorizerFitness1");
		mMovingLeft = true;
		mMovingLeftValue = new BooleanValue();
		addParameter(mMovingLeftValue, "MovingLeft");
	}
	
	

	public boolean init() {
		boolean ok =  super.init();
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		SimObjectValue v = vm.getSimObjectValue("/SimObjects/StartingArea");
		if(v != null) {
			mRightBound = v.get();
		}
		v = vm.getSimObjectValue("/SimObjects/TargetArea");
		if(v != null) {
			mLeftBound = v.get();
		}
		v = vm.getSimObjectValue("/SimObjects/Robot");
		if(v != null) {
			mRobot = v.get();
		}
		if(mRightBound == null || mLeftBound == null || mRobot == null) {
			Logger.getLogger("Simulator").warning("Could not find the required SimObjects");
			return false;
		}
		
		return ok;
	}



	public double calculateFitness() {

		if(mLastXPosition == Double.MAX_VALUE) {
			mLastXPosition = mRobot.getGlobalPosition().x;
		}
		
		if(mMovingLeft) {
			//check if the robot still is moving left
			if(mRobot.getGlobalPosition().x <= mLeftBound.getGlobalPosition().x + 30) {
				mMovingLeft = false;
				mMovingLeftValue.set(false);
			}
			double diff = mLastXPosition - mRobot.getGlobalPosition().x;
			mAccumulator += diff;
		}
		else {
			if(mRobot.getGlobalPosition().x >= mRightBound.getGlobalPosition().x - 30.0) {
				mMovingLeft = true;
				mMovingLeftValue.set(true);
			}
			double diff = mRobot.getGlobalPosition().x - mLastXPosition;
			mAccumulator += diff;
		}
		mLastXPosition = mRobot.getGlobalPosition().x;
		
		return Math.max(0.0, mAccumulator);
	}

	public void reset() {
		mMovingLeft = true;
		mMovingLeftValue.set(true);
		mLastXPosition = Double.MAX_VALUE;
		mAccumulator = 0.0;
	}


}
