package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class SimpleBackAndForthScenarioFitness1 extends FitnessFunction {
	
	
	NormalizedDoubleValue mVelocityX;
	NormalizedDoubleValue mDistLeft;
	NormalizedDoubleValue mDistRight;
	
	private double mLastDirection;
	private double mDirectionFactor;
	private double mAccu;
	
	private DoubleValue mFitnessIncement;
	
	public SimpleBackAndForthScenarioFitness1(double fitnessIncrement) {
		super("SimpleBackAndForthScenarioFitness1");
		mFitnessIncement = new DoubleValue(fitnessIncrement);
		addParameter(mFitnessIncement, "IncrementPerStepAfterTurn");
		mAccu = 0.0;
	}

	public boolean init() {
		boolean ok = super.init();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mVelocityX = (NormalizedDoubleValue) vm.getDoubleValue("/SimObjects/Robot/VelocityX");
		mDistLeft = (NormalizedDoubleValue) vm.getDoubleValue("/SimObjects/DistanceScannerLeft/Distance");
		mDistRight = (NormalizedDoubleValue) vm.getDoubleValue("/SimObjects/DistanceScannerRight/Distance");
		
		if(mVelocityX == null || mDistLeft == null || mDistRight == null) {
			Logger.getLogger("Simulator")
				.warning("SimpleBackAndForthScenarioFitness1.init(): Failed!");
			ok = false;
		}
		else {
			
		}
		
		return ok;
	}

	public void reset() {
		mAccu = 0.0;
	}

	public double calculateFitness() {
		
		if(mVelocityX == null || mDistLeft == null || mDistRight == null) {
			Logger.getLogger("Simulator")
				.warning("SimpleBackAndForthScenarioFitness1.init(): Failed!");
			return 0.0;
		}
		
		double vel = mVelocityX.getNormalized();
		double distLeft = mDistLeft.getNormalized();
		double distRight = mDistRight.getNormalized();
		
		if(mLastDirection > 0.0) {
			if(vel > 0.0) {
				mDirectionFactor += mFitnessIncement.get();
				mDirectionFactor = Math.min(mDirectionFactor, 1.0);
			}
			else {
				mDirectionFactor = 0.0;
			}
		}
		else {
			if(vel <= 0.0) {
				mDirectionFactor += mFitnessIncement.get();
				mDirectionFactor = Math.min(mDirectionFactor, 1.0);
			}
			else {
				mDirectionFactor = 0.0;
			}
		}
		
		distLeft = Math.max(0.0, distLeft);
		distRight = Math.max(0.0, distRight);
		
		mAccu += (Math.abs(vel) * mDirectionFactor * distLeft * distRight);
		
		mLastDirection = vel;
		
		return mAccu;

	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mVelocityX = (NormalizedDoubleValue) ValueManager.synchronizeValue(mVelocityX, oldValue, newValue, this);
		mDistLeft = (NormalizedDoubleValue) ValueManager.synchronizeValue(mDistLeft, oldValue, newValue, this);
		mDistRight = (NormalizedDoubleValue) ValueManager.synchronizeValue(mDistRight, oldValue, newValue, this);
		mFitnessIncement = (DoubleValue) ValueManager.synchronizeValue(mFitnessIncement, oldValue, newValue, this);
	}
	
	
	

}
