package de.fhg.iais.outdoor.simba.neuralnet;

import java.util.Iterator;
import java.util.Vector;

public class ConnectivityRating {
	
	
	private static Vector mVisitedNeurons = new Vector();
	
	public synchronized static boolean isConnected(Neuron neuron, Vector targetNeurons) {
		
		mVisitedNeurons.clear();
		Vector neuronsToCheck = new Vector();
		neuronsToCheck = new Vector(targetNeurons);
		
		return checkRecursivelyIfConnected(neuronsToCheck, neuron);
	}
	
	public static boolean checkRecursivelyIfConnected(Vector neuronsToCheck, Neuron targetNeuron) {
		if(neuronsToCheck.isEmpty()) {
			return false;
		}
		Vector newNeuronList = new Vector();
		for(Iterator i = neuronsToCheck.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			if(neuron == targetNeuron) {
				return true;
			}
			if(!mVisitedNeurons.contains(neuron)) {
				mVisitedNeurons.add(neuron);
				
				Vector synapses = neuron.getSynapses();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					Neuron target = (Neuron) synapse.getSourceNeuron();
					if(target == null) {
						continue;
					}
					if(!mVisitedNeurons.contains(target)) {
						newNeuronList.add(target);
					}
				}
			}
		}
		return checkRecursivelyIfConnected(newNeuronList, targetNeuron);
	}
	
	public synchronized static boolean isInverseConnected(Neuron neuron, Vector targetNeurons) {
		
		mVisitedNeurons.clear();
		Vector neuronsToCheck = new Vector();
		neuronsToCheck.clear();
		neuronsToCheck.add(neuron);
		Vector targets = new Vector(targetNeurons);
		
		return checkRecursivelyIfInverseConnected(neuronsToCheck, targets);
	}
	
	public static boolean checkRecursivelyIfInverseConnected(Vector neuronsToCheck, Vector targetNeurons) {
		
		if(neuronsToCheck.isEmpty()) {
			return false;
		}
		Vector newNeuronList = new Vector();
		for(Iterator i = neuronsToCheck.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			if(targetNeurons.contains(neuron)) {
				return true;
			}
			if(!mVisitedNeurons.contains(neuron)) {
				mVisitedNeurons.add(neuron);
				
				Vector synapses = neuron.getSynapses();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					Neuron target = (Neuron) synapse.getSourceNeuron();
					if(target == null) {
						continue;
					}
					if(!mVisitedNeurons.contains(target)) {
						newNeuronList.add(target);
					}
				}
			}
		}
		return checkRecursivelyIfInverseConnected(newNeuronList, targetNeurons);
	}
	
	

}
