package de.fhg.iais.outdoor.simba.neuralnet;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;

public class NeuronWrapper extends NormalizedDoubleValue {

	String mName;
	
	public NeuronWrapper(String name, double value, double minValue, double maxValue) {
		super(value, minValue, maxValue, 0.0, 1.0);
		mName = name;
	}
	
	
	public static NeuronWrapper createWrapperForDistanceScanner(DistanceScanner scanner) {
		String name = "/SimObjects/" + scanner.getName() + "/Distance";
		NeuronWrapper wrapper = new NeuronWrapper(name, scanner.getSensorReading(), 
				scanner.getMinScanDistance(), scanner.getMaxScanDistance());
		Simulator.getInstance().getValueManager().replaceValue(name, wrapper);
		return wrapper;
	}
	
	public static NeuronWrapper createWrapperForBumpDetector(SimpleBumpDetector bumper) {
		String name = "/SimObjects/" + bumper.getName() + "/BumpDetectedDouble";
		NeuronWrapper wrapper = new NeuronWrapper(name, bumper.getSensorReading(), 
				-1.0, 1.0);
		Simulator.getInstance().getValueManager().replaceValue(name, wrapper);
		return wrapper;
	}
	
	public static NeuronWrapper createWrapperForLightSensor(LightSensor sensor) {
		String name = "/SimObjects/" + sensor.getName() + "/Brightness";
		NeuronWrapper wrapper = new NeuronWrapper(name, sensor.getSensorReading(), 
				sensor.getMinimalBrightness(), sensor.getMaximalBrightness());
		Simulator.getInstance().getValueManager().replaceValue(name, wrapper);
		return wrapper;
	}
	
	
	public void setName(String name) {
		mName = name;
	}
	
	public String getName() {
		return mName;
	}
}
