package de.fhg.iais.outdoor.simba.neuralnet;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;

public class Synapse implements Cloneable {

	public static final String DYNAMIC = "dynamic";
	public static final String PROTECTED = "protected";
	public static final String FIXED = "fixed";
	public static Vector mProtectionModes = new Vector();
	
	private Neuron mSource;
	private DoubleValue mStrength;
	private String mProtection;
	private Neuron mTarget;
	private BooleanValue mEnabled;
	
	public Synapse(Neuron sourceNeuron) {
		this(sourceNeuron, 0.0);
	}
	
	public Synapse(Neuron sourceNeuron, double strength) {
		this(sourceNeuron, strength, Synapse.DYNAMIC);
	}
	
	public Synapse(Neuron sourceNeuron, double strength, String protection) {
		//check if the static protection mode vector is already set up.
		if(mProtectionModes.isEmpty()) {
			mProtectionModes.add(Synapse.DYNAMIC);
			mProtectionModes.add(Synapse.PROTECTED);
			mProtectionModes.add(Synapse.FIXED);
		}
		
		mSource = sourceNeuron;
		mStrength = new DoubleValue(strength);
		mEnabled = new BooleanValue(true);
		mProtection = protection;
		mTarget = null;
	}
	
	
	public double getActivation() {
		if(!mEnabled.get()) {
			return 0.0;
		}
		return (mSource.getLastActivity() * mStrength.get());
	}
	
	public Neuron getSourceNeuron() {
		return mSource;
	}
	
	public Neuron getTargetNeuron() {
		return mTarget;
	}
	
	public void setStrength(double strength) {
		mStrength.set(strength);
	}
	
	public double getStrength() {
		return mStrength.get();
	}
	
	public DoubleValue getStrengthValue() {
		return mStrength;
	}
	
	public void setSourceNeuron(Neuron source) {
		mSource = source;
	}
	
	/**
	 * This method is called by the target neuron itself only!
	 * @param target
	 */
	public void setTargetNeuron(Neuron target) {
		mTarget = target;
	}
	
	public BooleanValue getEnabledValue() {
		return mEnabled;
	}
	
	/**
	 * One of DYNAMIC, PROTECTED, FIXED.
	 * @param type
	 */
	public void setProtectionMode(String type) {
		mProtection = type;
	}
	
	public String getProtectionMode() {
		return mProtection;
	}
	
	public Object clone() throws CloneNotSupportedException {
		Synapse obj = (Synapse) super.clone();
		obj.mStrength = new DoubleValue(mStrength.get());
		obj.mEnabled = new BooleanValue(mEnabled.get());
		obj.mSource = mSource; //the clone works with the same source, this has to be change manually if required.
		obj.mTarget = mTarget; //the clone works with the same target, this has to be change manually if required.
		return obj;
	}
	
}
