package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.util.Random;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.DistanceSensor;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class DirectionEstimation extends SimbaDefaultFrame {
public NeuralNetAgentInterface mMainController;
	
	public DirectionEstimation(String args[]) {
		super("DirectionEstimation", 700, 500);

		setDefaultOffset(-235.0, -190.0);
		setDefaultZoom(1.0);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		//setFitnessFunction(new SimpleBackAndForthScenarioFitness1(0.1), 10000);
		
		
		setMasterEnvironment(new MyEnvironment(10, 8));
		
		Simulator.getInstance().getNeuralNetManager().setNeuralNetInterface(mMainController);
		Simulator.getInstance().getNeuralNetManager().setCurrentNet(NeuralNet.createMinimalNet(mMainController));
		
		startSystem(mMainController, args);
	}

	
	
	
	
	public class MyEnvironment extends EnvironmentModel {
		
		private int mMaxNumberOfObstacles;
		private int mMaxNumberOfSensors;
		private OmniDrive mRobot;
		private Box mRobotBody;
		private Vector mDistanceSensors = new Vector();
		private Vector mObstacles = new Vector();
		private DoubleValue mMinScannerDistance;
		private DoubleValue mMaxScannerDistance;
		private DoubleValue mScannerAngle;
		private IntValue mNumberOfObstacles;
		private DoubleValue mObstacleRadius;
		private DoubleValue mObstacleSize;
		
		public MyEnvironment(int noObstacles, int noSensors) {
			super("BackAndForthScenarioShortScanner");
			
			mMaxNumberOfObstacles = noObstacles;
			mMaxNumberOfSensors = noSensors;
			
			mMinScannerDistance = new DoubleValue(50.0);
			mMaxScannerDistance = new DoubleValue(70.0);
			mScannerAngle = new DoubleValue(3.0);
			addParameter(mMinScannerDistance, "MinScannerDistance");
			addParameter(mMaxScannerDistance, "MaxScannerDistance");
			addParameter(mScannerAngle, "ScannerAngle");
			
			mNumberOfObstacles = new IntValue(15);
			addParameter(mNumberOfObstacles, "NumberOfObstacles");
			
			mObstacleRadius = new DoubleValue(80.0);
			addParameter(mObstacleRadius, "ObstacleRingRadius");
			
			mObstacleSize = new DoubleValue(10.0);
			addParameter(mObstacleSize, "ObstacleSize");
			
		}

		public void resetEnvironmentObjects() {
			double minDist = mMinScannerDistance.get();
			double maxDist = mMaxScannerDistance.get();
			double scannerAngle = mScannerAngle.get();
			
			for(int i = 0; i < mDistanceSensors.size(); ++i) {
				DistanceScanner scanner = (DistanceScanner) mDistanceSensors.elementAt(i);
				scanner.setMinScanDistance(minDist);
				scanner.setMaxScanDistance(maxDist);
				scanner.setAngle((double) ((((mDistanceSensors.size() / 2.0) - 0.5) * -scannerAngle) + (i * scannerAngle)));
			}
			
			double obstacleSize = mObstacleSize.get();
			double obstacleRadius = mObstacleRadius.get();
			int numberOfObstacles = Math.max(0, Math.min(mNumberOfObstacles.get(), mMaxNumberOfObstacles));
			
			for(int i = 0; i < mObstacles.size(); ++i) {
				Box box = (Box) mObstacles.elementAt(i);
				if(i < numberOfObstacles) {
					box.setActive(true);
					
				}
				else {
					box.setActive(false);
				}
			}
			
		}
		
		public void updateRandomizationParameters() {
			
		}
		
		public String createRandomizationParameterLog() {
			
			return "";
		}

		public void createEnvironment() {

			ValueManager vm = Simulator.getInstance().getValueManager();
			
			mRobot = new OmniDrive("Robot", 0.0, 10.0, 0.0, -3.0, 3.0, -3.0, 3.0);
			mRobotBody = new Box("Robot/Body", 0.0, 0.0, 0.0, 20.0, 20.0);
			mRobotBody.setReferenceObject(mRobot);
			mRobotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(mRobotBody);
			//output neuron for motor.
			{
				String name = "/SimObjects/" + mRobot.getName() + "/AngularVelocity";
				NeuronWrapper wrapper = new NeuronWrapper(name, mRobot.getAngularVelocity(), 
						mRobot.getMinAngularVeloctiy(), mRobot.getMaxAngularVelocity());
				vm.replaceValue(name, wrapper);
				mMainController.addOutputNeuron(wrapper);
			}
			
			
			
			SimObjectSet obstacles = new SimObjectSet();
			
			int numberOfObstacles = mMaxNumberOfObstacles;
			for(int i = 0; i < numberOfObstacles; ++i) {
				Box box = new Box("Box" + i, 0.0, 0.0, 0.0, 10.0, 10.0);
				addSimObject(box);
				obstacles.add(box);
				mObstacles.add(box);
			}
			
			int numberOfScanners = mMaxNumberOfSensors;
			for(int i = 0; i < numberOfScanners; ++i) {
				DistanceScanner sensor = new DistanceScanner("Distance" + i, 0.0, 0.0, 0.0, 50.0, 70.0);
				sensor.setReferenceObject(mRobot);
				addSimObject(sensor);
				mDistanceSensors.add(sensor);
				mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(sensor));
				sensor.addDependencies(obstacles);
				sensor.addObstacleSet(obstacles);
			}
		}
		
	}

	
	public static void main(String[] args) {
		new DirectionEstimation(args);
	}
	
}
