package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Random;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.ColorBeacon;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.devices.TextLabel;
import de.fhg.iais.outdoor.simba.fitnessfunctions.LightChainFrontFitness1;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.PointValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class LightChain extends SimbaDefaultFrame  {

public NeuralNetAgentInterface mMainController;
	
	public LightChain(String args[]) {
		super("LightChain", 700, 700);
		
		setDefaultOffset(-430.0, -100.0);
		setDefaultZoom(1.0);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		//Add Fitness function.
		setFitnessFunction(new LightChainFrontFitness1(), 10000);
		
		setMasterEnvironment(new MyEnvironment(10));
		Simulator.getInstance().getNeuralNetManager().setNeuralNetInterface(mMainController);

		startSystem(mMainController, args);
	}

	
public class MyEnvironment extends EnvironmentModel {
		
		private Box mLeftBox;
		private Box mRightBox;
		private Box mFloor;
		private OmniDrive mRobot;
		private Box mRobotBody;
		private SimpleBumpDetector mBumpObstacles;
		private LightSensor mLightSensor;
		private LightSensor mHintSensor;
		private int mMaxNumberOfSpots;
		private IntValue mNumberOfSpots;
		private int mNumberOfSpotsBuffer;
		private double mRobotStartPosition;
		private Vector mLightSignals = new Vector();
		private Hashtable mColorTable = new Hashtable();
		private Vector mColorList = new Vector();
		private DoubleValue mBeaconDistance;
		private Color[] mColorInitialDistribution;
		private IntValue mHintBeaconId;
		private Color mHintColor;
		private DoubleValue mHintColorValue;
		private TextLabel mHintMarker;
		private PointValue mFixedStartingPosition;
		private BooleanValue mRandomizeStartingPosition;
		private BooleanValue mRandomizeBeaconColors;
		private SimObjectValue mHintBeacon;
		private SimObjectValue mGoalBeacon;
		private IntValue mNumberOfColors;

		
		public MyEnvironment(int maxNumberOfSpots) {
			super("LightChain");
			mMaxNumberOfSpots = maxNumberOfSpots;
			
			mNumberOfSpots = new IntValue(4);
			mBeaconDistance = new DoubleValue(100.0);
			
			addParameter(mNumberOfSpots, "NumberOfSpots");
			addParameter(mBeaconDistance, "BeaconDistance");
			
			//initialize color table
			mColorTable.put(Color.YELLOW, new Double(0.15));
			mColorTable.put(Color.RED, new Double(0.3));
			mColorTable.put(Color.GREEN, new Double(0.45));
			mColorTable.put(Color.BLUE, new Double(0.6));
			mColorTable.put(Color.CYAN, new Double(0.75));
			mColorTable.put(Color.MAGENTA, new Double(0.9));
			
			mColorList.add(Color.YELLOW);
			mColorList.add(Color.RED);
			mColorList.add(Color.GREEN);
			mColorList.add(Color.BLUE);
			mColorList.add(Color.CYAN);
			mColorList.add(Color.MAGENTA);
			
			mColorInitialDistribution = new Color[maxNumberOfSpots];
			
			mHintBeaconId = new IntValue(1);
			addParameter(mHintBeaconId, "HintPosition");
			
			mFixedStartingPosition = new PointValue(5.0, 5.0);
			addParameter(mFixedStartingPosition, "FixedStaringPosition");
			
			mRandomizeStartingPosition = new BooleanValue(false);
			addParameter(mRandomizeStartingPosition, "RandomizeStartingPos");
			
			mRandomizeBeaconColors = new BooleanValue(true);
			addParameter(mRandomizeBeaconColors, "RandomizeColors");
			
			mHintColorValue = new DoubleValue();
			Simulator.getInstance().getValueManager().addValue("/Environment/HintColor", mHintColorValue);
			
			mGoalBeacon = new SimObjectValue();
			Simulator.getInstance().getValueManager().addValue("/Environment/GoalBeacon", mGoalBeacon);
			
			mHintBeacon = new SimObjectValue();
			Simulator.getInstance().getValueManager().addValue("/Environment/HintBeacon", mHintBeacon);
			
			mNumberOfColors = new IntValue(6);
			addParameter(mNumberOfColors, "NumberOfColors (1-6)");
		}
		
		public void resetEnvironmentObjects() {
			double beaconDistance = mBeaconDistance.get();
			int hintBeaconId = Math.max(0, Math.min(mNumberOfSpots.get() - 1, mHintBeaconId.get()));
			Double hintColorValue = (Double) mColorTable.get(mHintColor);
			if(hintColorValue == null) {
				Logger.getLogger("Simulator").warning("Could not find color in color table!");
			}
			else {
				mHintColorValue.set(hintColorValue.doubleValue());
			}
			
			if(mRandomizeStartingPosition.get()) { 
				mRobot.setPosition(mRobotStartPosition, mRobot.getPosition().y);
			}
			else {
				mRobot.setPosition(mFixedStartingPosition.get());
			}
			mRightBox.setPosition((mNumberOfSpotsBuffer * beaconDistance) + 10.0, 15.0);
			mFloor.setWidth(30.0 + (beaconDistance * mNumberOfSpotsBuffer));
			mFloor.setPosition((mNumberOfSpotsBuffer) * (beaconDistance / 2.0), -3);
			mHintMarker.setPosition((beaconDistance / 2.0) + (hintBeaconId * beaconDistance), 100.0);
			
			
			int i = 0;
			int colorId = 0;
			
			mLightSensor.removeAllLightSources();
			mHintSensor.removeAllLightSources();
			for(; i < mNumberOfSpotsBuffer; ++i) {
				ColorBeacon beacon = (ColorBeacon) mLightSignals.elementAt(i);
				beacon.setPosition((beaconDistance / 2.0) + (i * beaconDistance), 80.0);
				beacon.setActive(true);
				beacon.getLightArea().setActive(true);
				beacon.getLightArea().setAngle(250.0);
				if(i == hintBeaconId) {
					beacon.setColor(mHintColor);
					mHintSensor.addLightSource(beacon);
					mHintBeacon.set(beacon);
				}
				else {
					beacon.setColor(mColorInitialDistribution[colorId]);
					colorId++;
					mLightSensor.addLightSource(beacon);
					if(beacon.getColor() == mHintColor) {
						mGoalBeacon.set(beacon);
					}
				}
			}
			for(; i < mMaxNumberOfSpots; ++i) {
				ColorBeacon beacon = (ColorBeacon) mLightSignals.elementAt(i);
				beacon.setActive(false);
				beacon.getLightArea().setActive(false);
			}
			
			
		}
		
		public void updateRandomizationParameters() {
			double beaconDistance = mBeaconDistance.get();
			
			mNumberOfSpotsBuffer = Math.min(Math.max(2, mNumberOfSpots.get()), mMaxNumberOfSpots);
			Random rand = new Random();
			mRobotStartPosition = (beaconDistance / 2.0) + (rand.nextInt(mNumberOfSpotsBuffer) * beaconDistance);
			
			LinkedList colors = new LinkedList();
			int numberOfColors = mNumberOfColors.get();
			for(int i = 0; i < numberOfColors; ++i) {
				colors.add(mColorList.elementAt(i));
			}
			if(mRandomizeBeaconColors.get()) {
				for(int i = 0; i < mColorInitialDistribution.length && !colors.isEmpty(); ++i) {
					Color color = (Color) colors.get(rand.nextInt(colors.size()));
					mColorInitialDistribution[i] = color;
					colors.remove(color);
				}
			}
			else {
				for(int i = 0; i < mColorInitialDistribution.length && !colors.isEmpty(); ++i) {
					Color color = (Color) colors.get(0);
					mColorInitialDistribution[i] = color;
					colors.remove(color);
				}
			}
			
			mHintColor = mColorInitialDistribution[rand.nextInt(Math.min(mColorInitialDistribution.length - 1, mNumberOfSpots.get() - 1))];
			
		}
		
	
		public String createRandomizationParameterLog() {
			String log =  super.createRandomizationParameterLog();
			log += getParamEntry("StartingPosition", Double.toString(mRobotStartPosition));
			return log;
		}
		
		

		protected synchronized Object clone() throws CloneNotSupportedException {
			MyEnvironment env = (MyEnvironment) super.clone();
			env.mColorInitialDistribution = new Color[mMaxNumberOfSpots];
			return env;
		}

		public void createEnvironment() {
	
			ValueManager vm = Simulator.getInstance().getValueManager();
			
			//Set up scenario.
			mLeftBox = new Box("LeftBound", -10.0, 15.0, 0.0, 10.0, 30.0);
			mLeftBox.setShapeColor(Color.DARK_GRAY);
			addSimObject(mLeftBox);
			
			mRightBox = new Box("RightBound", 210.0, 15.0, 0.0, 10.0, 30.0);
			mRightBox.setShapeColor(Color.DARK_GRAY);
			addSimObject(mRightBox);
			
			mFloor = new Box("Floor", 100.0, -3.0, 0.0, 230.0, 6.0);
			mFloor.setShapeColor(Color.DARK_GRAY);
			addSimObject(mFloor);
			

			mRobot = new OmniDrive("Robot", 0.0, 10.0, 0.0, -3.0, 3.0, 0.0, 0.0);
			mRobotBody = new Box("Robot/Body", 0.0, 0.0, 0.0, 20.0, 20.0);
			mRobotBody.setReferenceObject(mRobot);
			mRobotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(mRobotBody);
			//output neuron for motor.
			{
				String name = "/SimObjects/" + mRobot.getName() + "/VelocityX";
				NeuronWrapper wrapper = new NeuronWrapper(name, mRobot.getVelocityX(), 
						mRobot.getMinVelocity(), mRobot.getMaxVelocity());
				vm.replaceValue(name, wrapper);
				mMainController.addOutputNeuron(wrapper);
			}
			
			mBumpObstacles = new SimpleBumpDetector("BumpObstacles", 0, 0.0, 0.0);
			mBumpObstacles.setShape(mRobotBody.getShape());
			mBumpObstacles.setReferenceObject(mRobotBody);
			mBumpObstacles.addObstacle(mLeftBox);
			mBumpObstacles.addObstacle(mRightBox);
			addSimObject(mBumpObstacles);
			Simulator.getInstance().getScheduler().addTerminateTryValue(mBumpObstacles.getSensorValueObject());
			
			mLightSensor = new LightSensor("LightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			mLightSensor.setReferenceObject(mRobot);
			addSimObject(mLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(mLightSensor));
			
			mHintSensor = new LightSensor("HintSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			mHintSensor.setReferenceObject(mRobot);
			addSimObject(mHintSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(mHintSensor));
			 
			
			//Color beacons
			for(int i = 0; i < mMaxNumberOfSpots; ++i) {
				ColorBeacon beacon = new ColorBeacon("Beacon" + i, 0.0, 0.0, 0.0, 90.0, 3, 40.0);
				beacon.setColorTable(mColorTable);
				mLightSignals.add(beacon);
				addSimObject(beacon);
				mLightSensor.addDependency(beacon);
				mHintSensor.addDependency(beacon);
				mLightSensor.addLightSource(beacon);
			}
			
			mHintMarker = new TextLabel("HintMarker", 0.0, 0.0, 0.0, "Hint", 4, true);
			addSimObject(mHintMarker);
		}
		
	}


	public static void main(String[] args) {
		new LightChain(args);
	}
	
}