package de.fhg.iais.outdoor.simba.simulators;


import java.awt.Color;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.fitnessfunctions.SimpleBackAndForthScenarioFitness1;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;


public class SimpleBackAndForthScenario  extends SimbaDefaultFrame  {

	public NeuralNetAgentInterface mMainController;
	
	public SimpleBackAndForthScenario(String args[]) {
		super("SimpleBackAndForth", 700, 500);

		setDefaultOffset(-235.0, -190.0);
		setDefaultZoom(1.0);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = new NeuralNetAgentInterface("BackAndForthController");
		
		setFitnessFunction(new SimpleBackAndForthScenarioFitness1(0.1), 10000);

		setMasterEnvironment(new MyEnvironment());
		
		startSystem(mMainController, args);
	}

	
	
	
	
	public class MyEnvironment extends EnvironmentModel {
		
		private Box mLeftBox;
		private Box mRightBox;
		private Box mFloor;
		private OmniDrive mRobot;
		private Box mRobotBody;
		private DistanceScanner mDistanceLeft;
		private DistanceScanner mDistanceRight;
		private SimpleBumpDetector mBumpLeft;
		private SimpleBumpDetector mBumpRight;
		
		public MyEnvironment() {
			super("SimpleBackAndForthScenario");
		}

		public void resetEnvironmentObjects() { //setupInitialState
			//no randomization here.
		}

		public void updateRandomizationParameters() {
			//no randomization parameters needed
		}

		
		public void createEnvironment() {

			ValueManager vm = Simulator.getInstance().getValueManager();
			
			//Set up scenario.
			mLeftBox = new Box("LeftBound", -200.0, 15.0, 0.0, 10.0, 30.0);
			mLeftBox.setShapeColor(Color.DARK_GRAY);
			addSimObject(mLeftBox);
			
			mRightBox = new Box("RightBound", 200.0, 15.0, 0.0, 10.0, 30.0);
			mRightBox.setShapeColor(Color.DARK_GRAY);
			addSimObject(mRightBox);
			
			mFloor = new Box("Floor", 0.0, -3.0, 0.0, 410.0, 6.0);
			mFloor.setShapeColor(Color.DARK_GRAY);
			addSimObject(mFloor);
			
			mRobot = new OmniDrive("Robot", 0.0, 10.0, 0.0, -3.0, 3.0, 0.0, 0.0);
			mRobotBody = new Box("Robot/Body", 0.0, 0.0, 0.0, 20.0, 20.0);
			mRobotBody.setReferenceObject(mRobot);
			mRobotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(mRobotBody);
			//output neuron for motor.
			{
				String name = "/SimObjects/" + mRobot.getName() + "/VelocityX";
				NeuronWrapper wrapper = new NeuronWrapper(name, mRobot.getVelocityX(), 
						mRobot.getMinVelocity(), mRobot.getMaxVelocity());
				vm.replaceValue(name, wrapper);
				mMainController.addOutputNeuron(wrapper);
			}
			
			mDistanceLeft = new DistanceScanner("DistanceScannerLeft", -10.0, 8.0, -180.0, 0.0, 400.0);
			mDistanceLeft.addObstacle(mLeftBox);
			mDistanceLeft.setReferenceObject(mRobotBody);
			mDistanceRight = new DistanceScanner("DistanceScannerRight", 10.0, 8.0, 0.0, 0.0, 400.0);
			mDistanceRight.addObstacle(mRightBox);
			mDistanceRight.setReferenceObject(mRobotBody);
			addSimObject(mDistanceLeft);
			addSimObject(mDistanceRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mDistanceLeft));
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mDistanceRight));
			
			mBumpLeft = new SimpleBumpDetector("BumpLeft", -10, 0.0, 180.0);
			mBumpLeft.setReferenceObject(mRobotBody);
			mBumpLeft.addObstacle(mLeftBox);
			addSimObject(mBumpLeft);
			Simulator.getInstance().getScheduler().addTerminateTryValue(mBumpLeft.getSensorValueObject());
			
			mBumpRight = new SimpleBumpDetector("BumpRight", 10, 0.0, 180.0);
			mBumpRight.setReferenceObject(mRobotBody);
			mBumpRight.addObstacle(mRightBox);
			addSimObject(mBumpRight);
			Simulator.getInstance().getScheduler().addTerminateTryValue(mBumpRight.getSensorValueObject());
			
			mMainController.setNeuronRange(-1.0, 1.0); //tanh
			
		}
		
	}
	
	
	
	public static void main(String[] args) {
		new SimpleBackAndForthScenario(args);
	}
}
