package de.fhg.iais.outdoor.simba.simulators;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.communication.UDPSchedulerConnection;
import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.CircularPolygon;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.BorderLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.BoxLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.ShowDraggedObjectPositionVisualization;
import de.fhg.iais.outdoor.simba.visualization.SimulationDelayController;
import de.fhg.iais.outdoor.simba.visualization.SimulationRecorder;
import de.fhg.iais.outdoor.simba.visualization.SimulationVisualization;
import de.fhg.iais.outdoor.simba.visualization.StatusMessageLabel;
import de.fhg.iais.outdoor.simba.visualization.StepsPerSecondDisplay;
import de.fhg.iais.outdoor.simba.visualization.VisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.VisualizationIntervalController;

public class TestBeerScenario {
	
	public JFrame mFrame;
	public JPanel mContentPanel;
	public Scheduler mScheduler;
	public SimulationVisualization mVisu;
	
	public NeuralNetAgentInterface mMainController;
	
	public TestBeerScenario() {
		mMainController = new NeuralNetAgentInterface("TestBeerScenario");
		init();
	}

	public void init() {
		initializeGUI();		
		Simulator.getInstance().setSimulationDelay(5);
		
		//Instantiate Scheduler.
		Simulator.getInstance().getScheduler().setMasterEnvironment(new MyEnvironment());
		
		//Initialize Simulator.
		Simulator.getInstance().init();
		
		//Show frame.
		mFrame.setVisible(true);
		
	}
	
	public void initializeGUI() {
		
		mFrame = new JFrame("TestSimScenario");
		mFrame.setSize(700, 600);
		
		mContentPanel = new JPanel(new BorderLayout());
		mFrame.getContentPane().setLayout(new BorderLayout());
		
		//Set the root Visualization as frame content.
		mFrame.getContentPane().add(Simulator.getInstance().getVisualizationManager().getRootArea().getPanel(), BorderLayout.CENTER);
		
		
		mFrame.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			}
		);
		
		//Create and install Visualizations.
		
		mVisu = new SimulationVisualization();
		
		//Set offset and scaling of the visualization.
		mVisu.setOffset(-335.0, -350.0);
		mVisu.setScale(1.0);
		mVisu.setVisualizationInterval(1);
		
		VisualizationArea root = Simulator.getInstance().getVisualizationManager().getRootArea();
		
		ShowDraggedObjectPositionVisualization draggedObjVisu = new ShowDraggedObjectPositionVisualization();
		SimulationDelayController delayController = new SimulationDelayController();
		StepsPerSecondDisplay stepsDisplay = new StepsPerSecondDisplay();
		VisualizationIntervalController intervalController = new VisualizationIntervalController("/Visualization/Control/VisualizationInterval");
		StatusMessageLabel statusMessages = new StatusMessageLabel(2500);
		SimulationRecorder recorder = new SimulationRecorder();
		
		root.addVisualization(mVisu, BorderLayout.CENTER);
		
		BorderLayoutVisualizationArea lowerArea = new BorderLayoutVisualizationArea();
		root.addVisualization(lowerArea, BorderLayout.SOUTH);
		
		BoxLayoutVisualizationArea boxArea = new BoxLayoutVisualizationArea();
		lowerArea.addVisualization(boxArea, BorderLayout.CENTER);
		lowerArea.addVisualization(intervalController, BorderLayout.EAST);
		
		boxArea.addVisualization(stepsDisplay, "");
		boxArea.addVisualization(draggedObjVisu, "");
		boxArea.addVisualization(recorder, "");
		boxArea.addVisualization(statusMessages, "");
		
		root.addVisualization(delayController, BorderLayout.EAST);
		
		mFrame.getContentPane().add(Simulator.getInstance().getVisualizationManager().getRootArea().getPanel(), BorderLayout.CENTER);
	}
	
	public class MyEnvironment extends EnvironmentModel {
		
		private Box mLeftBox;
		private Box mRightBox;
		private Box mFloor;
		private OmniDrive mRobot;
		private Box mRobotBody;
		private MovingShape mMovingShape;
		private DistanceScanner mDistanceLeft;
		private DistanceScanner mDistanceRight;
		private DistanceScanner mObjectScanLeft;
		private DistanceScanner mObjectScanMid;
		private DistanceScanner mObjectScanRight;
		private SimpleBumpDetector mBumpShapeOnFloor;
		private SimpleBumpDetector mBumpShapeOnRobot;
		private SimpleBumpDetector mBumpLeft;
		private SimpleBumpDetector mBumpRight;
		
		
		public MyEnvironment() {
			super("TestBeerScenario");
		}

		public void resetEnvironmentObjects() {
			// TODO Auto-generated method stub
			
		}
		
		

		public void updateRandomizationParameters() {
			// TODO Auto-generated method stub
		}



		public void createEnvironment() {

			ValueManager vm = Simulator.getInstance().getValueManager();
			
			//Set up scenario.
			mLeftBox = new Box("LeftBound", -200.0, 15.0, 0.0, 10.0, 30.0);
			mLeftBox.setShapeColor(Color.DARK_GRAY);
			addSimObject(mLeftBox);
			
			mRightBox = new Box("RightBound", 200.0, 15.0, 0.0, 10.0, 30.0);
			mRightBox.setShapeColor(Color.DARK_GRAY);
			addSimObject(mRightBox);
			
			mFloor = new Box("Floor", 0.0, -3.0, 0.0, 410.0, 6.0);
			mFloor.setShapeColor(Color.DARK_GRAY);
			addSimObject(mFloor);
			
			mRobot = new OmniDrive("Robot", 0.0, 10.0, 0.0, -3.0, 3.0, 0.0, 0.0);
			mRobotBody = new Box("Robot/Body", 0.0, 0.0, 0.0, 20.0, 20.0);
			mRobotBody.setReferenceObject(mRobot);
			mRobotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(mRobotBody);
			//TODO output neuron.
			{
				String name = "/SimObjects/" + mRobot.getName() + "/VelocityX";
	//			System.out.println("Motor Value: " + Simulator.getInstance().getValueManager().getValue(name));
				NeuronWrapper wrapper = new NeuronWrapper(name, mRobot.getVelocityX(), 
						mRobot.getMinVelocity(), mRobot.getMaxVelocity());
				vm.replaceValue(name, wrapper);
	//			System.out.println("Motor Value: " + Simulator.getInstance().getValueManager().getValue(name));
				mMainController.addOutputNeuron(wrapper);
			}
			
			mDistanceLeft = new DistanceScanner("DistanceScannerLeft", -10.0, 8.0, -180.0, 0.0, 60.0);
			mDistanceLeft.addObstacle(mLeftBox);
			mDistanceLeft.setReferenceObject(mRobotBody);
			mDistanceRight = new DistanceScanner("DistanceScannerRight", 10.0, 8.0, 0.0, 0.0, 60.0);
			mDistanceRight.addObstacle(mRightBox);
			mDistanceRight.setReferenceObject(mRobotBody);
			addSimObject(mDistanceLeft);
			addSimObject(mDistanceRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mDistanceLeft));
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mDistanceRight));
			
			RobotController controller = new RobotController(2.0);
			controller.setReferenceObject(mRobot);
			controller.addDependency(mRobotBody);
			controller.addDependency(mDistanceLeft);
			controller.addDependency(mDistanceRight);
			addSimObject(controller);
			
			
			mMovingShape = new MovingShape("MovingObject", 20.0, 0.5, 250.0, -10.0, -120.0, 120.0);
			mMovingShape.setShapeColor(Color.BLUE);
			addSimObject(mMovingShape);
			
			mObjectScanLeft = new DistanceScanner("ObjectScanLeft", -2.0, 10.0, 110.0, 0.0, 200.0);
			mObjectScanLeft.setReferenceObject(mRobotBody);
			mObjectScanLeft.addObstacle(mMovingShape);
			addSimObject(mObjectScanLeft);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mObjectScanLeft));
			
			
			mObjectScanMid = new DistanceScanner("ObjectScanMid", 0.0, 10.0, 90.0, 0.0, 200.0);
			mObjectScanMid.setReferenceObject(mRobotBody);
			mObjectScanMid.addObstacle(mMovingShape);
			addSimObject(mObjectScanMid);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mObjectScanMid));
			
			
			mObjectScanRight = new DistanceScanner("ObjectScanRight", 2.0, 10.0, 70.0, 0.0, 200.0);
			mObjectScanRight.setReferenceObject(mRobotBody);
			mObjectScanRight.addObstacle(mMovingShape);
			addSimObject(mObjectScanRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mObjectScanRight));
			
			
			mBumpShapeOnFloor = new SimpleBumpDetector("BumpShapeFloor", 0.0, -20.0, 0.0);
			mBumpShapeOnFloor.setReferenceObject(mMovingShape);
			mBumpShapeOnFloor.addObstacle(mFloor);
			addSimObject(mBumpShapeOnFloor);
			
			mBumpShapeOnRobot = new SimpleBumpDetector("BumpShapeRobot", 0.0, -20.0, 0.0);
			mBumpShapeOnRobot.setReferenceObject(mMovingShape);
			mBumpShapeOnRobot.addObstacle(mRobotBody);
			addSimObject(mBumpShapeOnRobot);
			//mMainController.addInputNeuron(NeuronWrapper.createWrapperForBumpDetector(mBumpShapeOnRobot));
			
			
			mBumpLeft = new SimpleBumpDetector("BumpLeft", -10, 0.0, 180.0);
			mBumpLeft.setReferenceObject(mRobotBody);
			mBumpLeft.addObstacle(mLeftBox);
			addSimObject(mBumpLeft);
			//mMainController.addInputNeuron(NeuronWrapper.createWrapperForBumpDetector(mBumpLeft));
			Simulator.getInstance().getScheduler().addTerminateTryValue(mBumpLeft.getSensorValueObject());
			
			mBumpRight = new SimpleBumpDetector("BumpRight", 10, 0.0, 180.0);
			mBumpRight.setReferenceObject(mRobotBody);
			mBumpRight.addObstacle(mRightBox);
			addSimObject(mBumpRight);
			//mMainController.addInputNeuron(NeuronWrapper.createWrapperForBumpDetector(mBumpRight));
			Simulator.getInstance().getScheduler().addTerminateTryValue(mBumpRight.getSensorValueObject());
			
			mMainController.setNeuronRange(-1.0, 1.0); //tanh
			
		}
		
		public void start() {
			//TCPSchedulerConnection connection = new TCPSchedulerConnection(mMainController);
			UDPSchedulerConnection connection = new UDPSchedulerConnection(mMainController);
			System.out.println("Start server...");
			connection.startServer(50100);
		}
	}
	
	public class MovingShape extends CircularPolygon {
		
		private double mStartingHeight;
		private double mEndingHeight;
		private double mMinX;
		private double mMaxX;
		private double mVelocity;
		
		
		public MovingShape(String name, double radius, double velocity,
				double startingHeight, double endingHeight, double minX, double maxX) 
		{
			super(name, minX, endingHeight - 1, 0.0, radius, 4);
			mStartingHeight = startingHeight;
			mEndingHeight = endingHeight;
			mMinX = minX;
			mMaxX = maxX;
			mVelocity = velocity;
		}
		
		public boolean update() {
			super.update();
			boolean ok = super.update();
			
			Point2D.Double position = mPosition.get();
			if(position.y <= mEndingHeight) {
				//setup the next movement.
				Random r = new Random();
				double x = mMinX + (r.nextDouble() * (mMaxX - mMinX));
				if(r.nextBoolean()) {
					setNumberOfCorners(4);
				}
				else {
					setNumberOfCorners(32);
				}
				mPosition.set(x, mStartingHeight);
			}
			else {
				mPosition.set(position.x, position.y - mVelocity); 
			}
			
			return ok;
		}
		
	}
	
	public class RobotController extends SimObject {
		
		private DistanceScanner mDistScannerLeft;
		private DistanceScanner mDistScannerRight;
		private OmniDrive mDrive;
		private int mCounter;
		
		private double mVelocity;
		private boolean mMoveToTheRight;
		
		public RobotController(double velocity) {
			mVelocity = velocity;
			mMoveToTheRight = true;
			mCounter = 0;
		}
		
		public boolean init() {
			boolean ok = true;
			
			SimObject obj1 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerLeft").get();
			SimObject obj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerRight").get();
			SimObject obj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Robot").get();

			
			if(obj1 ==null || obj2 == null || obj3 == null || 
				!(obj1 instanceof DistanceScanner) || !(obj2 instanceof DistanceScanner)
				|| !(obj3 instanceof OmniDrive)) 
			{
				ok = false;
			}
			else {
				mDistScannerLeft = (DistanceScanner) obj1;
				mDistScannerRight = (DistanceScanner) obj2;
				mDrive = (OmniDrive) obj3;
			}
			return ok;
		}
		
		public boolean cleanUp() {
			return true;
		}
		
		public boolean update() {
			mCounter++;
			
			if(mMoveToTheRight) {
				if(mDistScannerRight.getSensorReading() <= 10.0) {
					mMoveToTheRight = false;
				}
			}
			else {
				if(mDistScannerLeft.getSensorReading() <= 10.0) {
					mMoveToTheRight = true;
				}
			}
			if(mMoveToTheRight) {
				mDrive.setVelocityX(mVelocity);
			}
			else {
				mDrive.setVelocityX((-1 * mVelocity));
			}
			
			return true;
		}
	}
	
	public static void main(String[] args) {
		new TestBeerScenario();
		Simulator.getInstance().emulateEvolutionRun(1, 1, 1, -1);
	
	}
	
}
