package de.fhg.iais.outdoor.simba.simulators;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Polygon;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.AngleToPointScanner;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.CircularPolygon;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.visualization.BorderLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.BoxLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.ShowDraggedObjectPositionVisualization;
import de.fhg.iais.outdoor.simba.visualization.SimulationDelayController;
import de.fhg.iais.outdoor.simba.visualization.SimulationVisualization;
import de.fhg.iais.outdoor.simba.visualization.StatusMessageLabel;
import de.fhg.iais.outdoor.simba.visualization.StepsPerSecondDisplay;
import de.fhg.iais.outdoor.simba.visualization.VisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.VisualizationIntervalController;

public class TestMazeScenario {

	public JFrame mFrame;
	public JPanel mContentPanel;
	public Scheduler mScheduler;
	public SimulationVisualization mVisu;
	
	public TestMazeScenario() {
		init();
	}

	public void init() {
		
		//Set the speed.
		initializeGUI();
		
		Simulator.getInstance().setSimulationDelay(15);
		
		//Instantiate Scheduler.
		mScheduler = Simulator.getInstance().getScheduler();
		mScheduler.setMasterEnvironment(new MyEnvironment());
		
		//Initialize Simulator.
		Simulator.getInstance().init();
		
		//Show frame.
		mFrame.setVisible(true);
		
	}
	
	public void initializeGUI() {
		
		mFrame = new JFrame("TestSimScenario");
		mFrame.setSize(700, 500);
		
		mContentPanel = new JPanel(new BorderLayout());
		mFrame.getContentPane().setLayout(new BorderLayout());
		
		//Set the root Visualization as frame content.
		mFrame.getContentPane().add(Simulator.getInstance().getVisualizationManager().getRootArea().getPanel(), BorderLayout.CENTER);
		
		
		mFrame.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			}
		);
		
		//Create and install Visualizations.
		
		mVisu = new SimulationVisualization();
		
		//Set offset and scaling of the visualization.
		mVisu.setOffset(-700.0, -600.0);
		mVisu.setScale(0.5);
		mVisu.setDefaultView();
		mVisu.setVisualizationInterval(1);
		
		VisualizationArea root = Simulator.getInstance().getVisualizationManager().getRootArea();
		
		ShowDraggedObjectPositionVisualization draggedObjVisu = new ShowDraggedObjectPositionVisualization();
		SimulationDelayController delayController = new SimulationDelayController();
		StepsPerSecondDisplay stepsDisplay = new StepsPerSecondDisplay();
		VisualizationIntervalController intervalController = new VisualizationIntervalController("/Visualization/Control/VisualizationInterval");
		StatusMessageLabel statusMessages = new StatusMessageLabel(2500);
		
		root.addVisualization(mVisu, BorderLayout.CENTER);
		
		BorderLayoutVisualizationArea lowerArea = new BorderLayoutVisualizationArea();
		root.addVisualization(lowerArea, BorderLayout.SOUTH);
		
		BoxLayoutVisualizationArea boxArea = new BoxLayoutVisualizationArea();
		lowerArea.addVisualization(boxArea, BorderLayout.CENTER);
		lowerArea.addVisualization(intervalController, BorderLayout.EAST);
		
		boxArea.addVisualization(stepsDisplay, "");
		boxArea.addVisualization(draggedObjVisu, "");
		boxArea.addVisualization(statusMessages, "");
		
		root.addVisualization(delayController, BorderLayout.EAST);
		
		mFrame.getContentPane().add(Simulator.getInstance().getVisualizationManager().getRootArea().getPanel(), BorderLayout.CENTER);
	}
	
	public class MyEnvironment extends EnvironmentModel {
		
		private DifferentialDriveUnit mRobot;
			
		public MyEnvironment() {
			super("TestMazeScenario");
		}
		
		public void updateRandomizationParameters() {
			// TODO Auto-generated method stub
			
		}

		public void resetEnvironmentObjects() {
			// TODO Auto-generated method stub
			
		}

		public void createEnvironment() {

			//Set up scenario.
			mRobot = new DifferentialDriveUnit("Robot", 0.0, 10.0, 0.0, 20.0, 5.0, -1.5, 1.5);
			Box robotBody = new Box("Robot/Body", 0.0, 0.0, 90.0, 14.0, 20.0);
			robotBody.setReferenceObject(mRobot);
			robotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(robotBody);
			
			Random rand = new Random();
			
			int outerWidth = 200 + (int) (50.0 * rand.nextDouble());
			int innerWidth = outerWidth - 20;
			int outerHeight = 70 + (int) (50.0 * rand.nextDouble());;
			int innerHeight = outerHeight - 20;
			
			Polygon shape = new Polygon();
			shape.addPoint(-innerHeight, outerHeight);
			shape.addPoint(-outerWidth, outerHeight);
			shape.addPoint(-outerWidth, -outerHeight);
			shape.addPoint(outerWidth, -outerHeight);
			shape.addPoint(outerWidth, outerHeight);
			shape.addPoint(innerHeight, outerHeight);
			shape.addPoint(innerHeight, innerHeight);
			shape.addPoint(innerWidth, innerHeight);
			shape.addPoint(innerWidth, -innerHeight);
			shape.addPoint(-innerWidth, -innerHeight);
			shape.addPoint(-innerWidth, innerHeight);
			shape.addPoint(-innerHeight, innerHeight);
			
			SimObject lowerT = new SimObject("LowerT", 0.0, 0.0, 0.0, new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			addSimObject(lowerT);
			
			SimObject rightT = new SimObject("RightT", outerWidth + outerHeight, outerHeight + outerWidth, 90.0, new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			addSimObject(rightT);
			
			SimObject leftT = new SimObject("LeftT", -1 * (outerWidth + outerHeight - 1), outerHeight + outerWidth, -90.0, new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			addSimObject(leftT);
	
			Box upperConnector = new Box("UpperConnector", 0.0, outerHeight + outerWidth + innerHeight + ((outerHeight - innerHeight) / 2), 0.0, outerWidth * 2, outerHeight - innerHeight);
			addSimObject(upperConnector);
			
			Box leftConnector = new Box("LeftConnector", innerHeight + ((outerWidth - innerHeight) / 2), (outerHeight + ((outerWidth - innerHeight) / 2)), 0.0, outerWidth - innerHeight, outerWidth - innerHeight);
			addSimObject(leftConnector);
			leftConnector.setLayer(-5);
			
			Box rightConnector = new Box("RightConnector", (innerHeight + ((outerWidth - innerHeight) / 2)) * -1, (outerHeight + ((outerWidth - innerHeight) / 2)), 0.0, outerWidth - innerHeight, outerWidth - innerHeight);
			addSimObject(rightConnector);
			;rightConnector.setLayer(-5);
			
			
			DistanceScanner scanLeft = new DistanceScanner("DistanceScannerLeft", 0.0, 10.0, 60.0, 0.0, 50.0);
			scanLeft.setReferenceObject(robotBody);
			scanLeft.addObstacle(lowerT);
			scanLeft.addObstacle(leftT);
			scanLeft.addObstacle(rightT);
			scanLeft.addObstacle(upperConnector);
			scanLeft.addObstacle(leftConnector);
			scanLeft.addObstacle(rightConnector);
			addSimObject(scanLeft);
			
			DistanceScanner scanRight = new DistanceScanner("DistanceScannerRight", 0.0, -10.0, -60.0, 0.0, 50.0);
			scanRight.setReferenceObject(robotBody);
			scanRight.addObstacle(lowerT);
			scanRight.addObstacle(leftT);
			scanRight.addObstacle(rightT);
			scanRight.addObstacle(upperConnector);
			scanRight.addObstacle(leftConnector);
			scanRight.addObstacle(rightConnector);
			addSimObject(scanRight);
			
			DistanceScanner scanMid = new DistanceScanner("DistanceScannerFront", 7.0, 0.0, 0.0, 0.0, 40.0);
			scanMid.setReferenceObject(robotBody);
			scanMid.addObstacle(lowerT);
			scanMid.addObstacle(leftT);
			scanMid.addObstacle(rightT);
			scanMid.addObstacle(upperConnector);
			scanMid.addObstacle(leftConnector);
			scanMid.addObstacle(rightConnector);
			addSimObject(scanMid);
			
			
			RobotController controller = new RobotController(2.0);
			controller.setReferenceObject(mRobot);
			controller.addDependency(robotBody);
			controller.addDependency(scanLeft);
			controller.addDependency(scanRight);
			controller.addDependency(scanMid);
			addSimObject(controller);
			
			SimpleBumpDetector bump = new SimpleBumpDetector("BumpRight", 0.0, 0.0, 0.0);
			Polygon p = robotBody.getShape();
			bump.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints));
			bump.setReferenceObject(robotBody);
			bump.setReferenceObject(robotBody);
			bump.addObstacle(lowerT);
			bump.addObstacle(leftT);
			bump.addObstacle(rightT);
			bump.addObstacle(upperConnector);
			bump.addObstacle(leftConnector);
			bump.addObstacle(rightConnector);
			addSimObject(bump);
			
			Beacon beacon1 = new Beacon("Beacon1", -1 * ((outerWidth - innerHeight) / 2.0), 0.0, 135.0, innerHeight * 2.0, 20, 90.0, 0.02, 0.00);
			beacon1.setReferenceObject(leftConnector);
			beacon1.setLayer(5);
			beacon1.getLightArea().setLayer(4);
			int[] sequence = new int[1];
			sequence[0] = 200;
			beacon1.setBlinkPattern(sequence);
			beacon1.getLightArea().setShapeColor(Color.ORANGE);
			addSimObject(beacon1);
			
			Beacon beacon2 = new Beacon("Beacon", (outerWidth - innerHeight) / 2.0, 0.0, 315.0, innerHeight * 2.0, 20, 90.0, 0.02, 1.0);
			beacon2.setReferenceObject(rightConnector);
			beacon2.setLayer(5);
			beacon2.getLightArea().setLayer(4);
			beacon2.setBlinkPattern(sequence);
			beacon2.getLightArea().setShapeColor(Color.YELLOW);
			addSimObject(beacon2);
			
			Beacon beacon3 = new Beacon("Beacon3", -20.0, (outerWidth - innerHeight) / 2.0, 45.0, innerHeight * 2.0, 20, 90.0, 0.9, 0.00);
			int[] sequence2 = new int[1];
			sequence2[0] = 150;
			beacon3.setReferenceObject(leftConnector);
			beacon3.setLayer(5);
			beacon3.getLightArea().setLayer(4);
			beacon3.setBlinkPattern(sequence2);
			beacon3.getLightArea().setShapeColor(Color.BLUE);
			addSimObject(beacon3);
			
			Beacon beacon4 = new Beacon("Beacon4", ((outerWidth - innerHeight) / 2.0) + innerHeight, (outerHeight - innerHeight) / -2.0, 225.0, innerHeight * 2.0, 20, 90.0, 0.9, 1.0);
			beacon4.setBlinkPattern(sequence2);
			beacon4.setReferenceObject(upperConnector);
			beacon4.setLayer(5);
			beacon4.getLightArea().setLayer(4);
			beacon4.getLightArea().setShapeColor(Color.cyan);
			addSimObject(beacon4);
			
			LightSensor lightSensor1 = new LightSensor("LightSensor1", 0.0, 0.0, 0.0, 0.0, 1.0);
			lightSensor1.addLightSource(beacon1);
			lightSensor1.addLightSource(beacon3);
			lightSensor1.setReferenceObject(robotBody);
			addSimObject(lightSensor1);
			
			AngleToPointScanner angleScanner = new AngleToPointScanner("AngleScanner", 0.0, 0.0, 0.0);
			angleScanner.addObstacle(beacon1);
			angleScanner.setReferenceObject(robotBody);
			addSimObject(angleScanner);
			
			
		}
	}
	
	public class MovingShape extends CircularPolygon {
		
		private double mStartingHeight;
		private double mEndingHeight;
		private double mMinX;
		private double mMaxX;
		private double mVelocity;
		
		
		public MovingShape(String name, double radius, double velocity,
				double startingHeight, double endingHeight, double minX, double maxX) 
		{
			super(name, minX, endingHeight - 1, 0.0, radius, 4);
			mStartingHeight = startingHeight;
			mEndingHeight = endingHeight;
			mMinX = minX;
			mMaxX = maxX;
			mVelocity = velocity;
		}
		
		public boolean update() {
			boolean ok = super.update();
			
			Point2D.Double position = mPosition.get();
			if(position.y <= mEndingHeight) {
				//setup the next movement.
				Random r = new Random();
				double x = mMinX + (r.nextDouble() * (mMaxX - mMinX));
				if(r.nextBoolean()) {
					setNumberOfCorners(4);
				}
				else {
					setNumberOfCorners(32);
				}
				mPosition.set(x, mStartingHeight);
			}
			else {
				mPosition.set(position.x, position.y - mVelocity); 
			}
			
			return ok;
		}
		
	}
	
	public class RobotController extends SimObject {
		
		private DistanceScanner mDistScannerLeft;
		private DistanceScanner mDistScannerRight;
		private DistanceScanner mDistScannerMid;
		private DifferentialDriveUnit mDrive;
		private int mCounter;
		
		private double mVelocity;
		
		public RobotController(double velocity) {
			mVelocity = velocity;
			mCounter = 0;
		}
		
		public boolean init() {
			boolean ok = true;
			
			SimObject obj1 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerLeft").get();
			SimObject obj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerRight").get();
			SimObject obj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerFront").get();
			SimObject obj4 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Robot").get();

			
			if(obj1 ==null || obj2 == null || obj3 == null || obj4 == null ||
				!(obj1 instanceof DistanceScanner) || !(obj2 instanceof DistanceScanner)
				|| !(obj3 instanceof DistanceScanner) || !(obj4 instanceof DifferentialDriveUnit)) 
			{
				ok = false;
			}
			else {
				mDistScannerLeft = (DistanceScanner) obj1;
				mDistScannerRight = (DistanceScanner) obj2;
				mDistScannerMid = (DistanceScanner) obj3;
				mDrive = (DifferentialDriveUnit) obj4;
			}
			return ok;
		}
		
		public boolean cleanUp() {
			return true;
		}
		
		public boolean update() {
			double left = mVelocity;
			double right = mVelocity;
			
			if(mCounter > 0) {
				mCounter--;
				return true;
			}
			
			if((mDistScannerLeft.getSensorReading() <= 30.0 
					|| mDistScannerRight.getSensorReading() <= 30.0))
			{
				if(mDistScannerLeft.getSensorReading() - mDistScannerRight.getSensorReading() < 0.0)
				{
					right *= -1;
				}
				else {
					left *= -1;
				}
			}
			if(mDistScannerMid.getSensorReading() <= 10.0) {
				if(Math.abs(mDistScannerLeft.getSensorReading() - mDistScannerRight.getSensorReading()) <= 1.0)
				{
					right *= -1;
					mCounter = 5;
				}
			}
			mDrive.setLeftVelocity(left);
			mDrive.setRightVelocity(right);
			
			return true;
		}
	}
	
	public static void main(String[] args) {
		new TestMazeScenario();
		Simulator.getInstance().emulateEvolutionRun(1, 1, 1, -1);
	}

}
