package de.fhg.iais.outdoor.simba.simulators;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.visualization.BorderLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.BoxLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.ShowDraggedObjectPositionVisualization;
import de.fhg.iais.outdoor.simba.visualization.SimulationDelayController;
import de.fhg.iais.outdoor.simba.visualization.SimulationVisualization;
import de.fhg.iais.outdoor.simba.visualization.StatusMessageLabel;
import de.fhg.iais.outdoor.simba.visualization.StepsPerSecondDisplay;
import de.fhg.iais.outdoor.simba.visualization.VisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.VisualizationIntervalController;


public class TestSimulationScenario {
	
	public JFrame mFrame;
	public JPanel mContentPanel;
	public Scheduler mScheduler;
	public SimulationVisualization mVisu;
	
	public TestSimulationScenario() {
		init();
	}
	
	public void init() {
		
		//Set the speed.
		initializeGUI();
		
		Simulator.getInstance().setSimulationDelay(30);
		
		Simulator.getInstance().getScheduler().setMasterEnvironment(
				new MyEnvironment());
		
		
		//Initialize Simulator.
		Simulator.getInstance().init();
		
		//Show frame.
		mFrame.setVisible(true);
		
		Enumeration e = Simulator.getInstance().getValueManager().getValueNames();
		for(; e.hasMoreElements();) {
			System.out.println(e.nextElement().toString());
		}
		
	}
	
	public void initializeGUI() {
		
		mFrame = new JFrame("TestSimScenario");
		mFrame.setSize(700, 300);
		
		mContentPanel = new JPanel(new BorderLayout());
		mFrame.getContentPane().setLayout(new BorderLayout());
		
		//Set the root Visualization as frame content.
		mFrame.getContentPane().add(Simulator.getInstance().getVisualizationManager().getRootArea().getPanel(), BorderLayout.CENTER);
		
		
		mFrame.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			}
		);
		
		//Create and install Visualizations.
		
		mVisu = new SimulationVisualization();
		
		//Set offset and scaling of the visualization.
		mVisu.setOffset(-335.0, -150.0);
		mVisu.setScale(1.0);
		mVisu.setVisualizationInterval(1);
		
		VisualizationArea root = Simulator.getInstance().getVisualizationManager().getRootArea();
		
		ShowDraggedObjectPositionVisualization draggedObjVisu = new ShowDraggedObjectPositionVisualization();
		SimulationDelayController delayController = new SimulationDelayController();
		StepsPerSecondDisplay stepsDisplay = new StepsPerSecondDisplay();
		VisualizationIntervalController intervalController = new VisualizationIntervalController("/Visualization/Control/VisualizationInterval");
		StatusMessageLabel statusMessages = new StatusMessageLabel(2500);
		
		root.addVisualization(mVisu, BorderLayout.CENTER);
		
		BorderLayoutVisualizationArea lowerArea = new BorderLayoutVisualizationArea();
		root.addVisualization(lowerArea, BorderLayout.SOUTH);
		
		BoxLayoutVisualizationArea boxArea = new BoxLayoutVisualizationArea();
		lowerArea.addVisualization(boxArea, BorderLayout.CENTER);
		lowerArea.addVisualization(intervalController, BorderLayout.EAST);
		
		boxArea.addVisualization(stepsDisplay, "");
		boxArea.addVisualization(draggedObjVisu, "");
		boxArea.addVisualization(statusMessages, "");
		
		root.addVisualization(delayController, BorderLayout.EAST);
		
		mFrame.getContentPane().add(Simulator.getInstance().getVisualizationManager().getRootArea().getPanel(), BorderLayout.CENTER);
	}
	
	public class MyEnvironment extends EnvironmentModel {
		
		private Box mLeftBox;
		private Box mRightBox;
		private Box mFloor;
		private OmniDrive mRobot;
		private Box mRobotBody;
		private Beacon mEnergySource1;
		private Beacon mEnergySource2;
		private Beacon mEnergySource3;
//		private OrthogonalDistanceScanner mOrthogonalScanner1;
//		private OrthogonalDistanceScanner mOrthogonalScanner2;
//		private DistanceToPointScanner mDistanceToPointScanner1;
		private DistanceScanner mDistanceLeft;
		private DistanceScanner mDistanceRight;
		
		public MyEnvironment() {
			super("TestSimulationScenario");
		}
		
		public void resetEnvironmentObjects() {
			
		}
		
		public void updateRandomizationParameters() {
			// TODO Auto-generated method stub
			
		}
		
		public void createEnvironment() {
			
			//Set up scenario.
			mLeftBox = new Box("LeftBound", -200.0, 15.0, 0.0, 10.0, 30.0);
			addSimObject(mLeftBox);
			
			mRightBox = new Box("RightBound", 200.0, 15.0, 0.0, 10.0, 30.0);
			addSimObject(mRightBox);
			
			mFloor = new Box("Floor", 0.0, -1.0, 0.0, 410.0, 2.0);
			addSimObject(mFloor);
			
			mRobot = new OmniDrive("Robot", 0.0, 10.0, 0.0);
			mRobotBody = new Box("Robot/Body", 0.0, 0.0, 0.0, 20.0, 20.0);
			mRobotBody.setReferenceObject(mRobot);
			addSimObject(mRobot);
			addSimObject(mRobotBody);
			
			mDistanceLeft = new DistanceScanner("DistanceScannerLeft", -10.0, 8.0, -180.0, 0.0, 60.0);
			mDistanceLeft.addObstacle(mLeftBox);
			mDistanceLeft.setReferenceObject(mRobotBody);
			mDistanceRight = new DistanceScanner("DistanceScannerRight", 10.0, 8.0, 0.0, 0.0, 60.0);
			mDistanceRight.addObstacle(mRightBox);
			mDistanceRight.setReferenceObject(mRobotBody);
			addSimObject(mDistanceLeft);
			addSimObject(mDistanceRight);
			
			RobotController controller = new RobotController(1.0);
			controller.setReferenceObject(mRobot);
			controller.addDependency(mRobotBody);
			controller.addDependency(mDistanceLeft);
			controller.addDependency(mDistanceRight);
			addSimObject(controller);
			

			Random rand = new Random();
			
			mEnergySource1 = new Beacon("Light1", -60.0 - (50.0 * rand.nextDouble()), 0.0, 0.0, 30.0, 10, 180.0, 0.1, 1.0);
			int[] frequency = new int[2];
			frequency[0] = 40;
			frequency[1] = 140;
			mEnergySource1.setBlinkPattern(frequency);
			addSimObject(mEnergySource1);
			
			mEnergySource2 = new Beacon("Light2", 80.0 + (50.0 * rand.nextDouble()), 0.0, 0.0, 30.0, 10, 180.0, 0.1, 0.0);
			int[] frequency3 = new int[4];
			frequency3[0] = 40;
			frequency3[1] = 80;
			frequency3[2] = 60;
			frequency3[3] = -1;
			mEnergySource2.setBlinkPattern(frequency3);
			addSimObject(mEnergySource2);
			
			mEnergySource3 = new Beacon("Light3", 10.0, 40.0, 0.0, 40.0, 20, 360.0, 0.1, 0.0);
			int[] frequency2 = new int[2];
			frequency2[0] = 120;
			frequency2[1] = 60;
			
			mEnergySource3.setBlinkPattern(frequency2);
			addSimObject(mEnergySource3);

		}
	}
	
	public class RobotController extends SimObject {
		
		private DistanceScanner mDistScannerLeft;
		private DistanceScanner mDistScannerRight;
		private OmniDrive mDrive;
		private int mCounter;
		
		private double mVelocity;
		private boolean mMoveToTheRight;
		
		public RobotController(double velocity) {
			mVelocity = velocity;
			mMoveToTheRight = true;
			mCounter = 0;
		}
		
		public boolean init() {
			boolean ok = true;
			
			SimObject obj1 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerLeft").get();
			SimObject obj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerRight").get();
			SimObject obj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Robot").get();
			
			if(obj1 ==null || obj2 == null || obj3 == null || 
				!(obj1 instanceof DistanceScanner) || !(obj2 instanceof DistanceScanner)
				|| !(obj3 instanceof OmniDrive)) 
			{
				ok = false;
			}
			else {
				mDistScannerLeft = (DistanceScanner) obj1;
				mDistScannerRight = (DistanceScanner) obj2;
				mDrive = (OmniDrive) obj3;
			}
			return ok;
		}
		
		public boolean cleanUp() {
			return true;
		}
		
		public boolean update() {
			mCounter++;
			if(mMoveToTheRight) {
				if(mDistScannerRight.getSensorReading() <= 10.0) {
					mMoveToTheRight = false;
				}
			}
			else {
				if(mDistScannerLeft.getSensorReading() <= 10.0) {
					mMoveToTheRight = true;
				}
			}
			if(mMoveToTheRight) {
				mDrive.setVelocityX(mVelocity);
			}
			else {
				mDrive.setVelocityX((-1 * mVelocity));
			}
			
			return true;
		}
		
		
	}
	
	/*
	public class TestController implements Controller {
		
		private DistanceScanner mDistScanner1;
		private DistanceScanner mDistScanner2;
		private OmniDrive mDrive;
		
		private double mVelocity;
		private boolean mMoveToTheRight;
		
		public TestController(double velocity) {
			mVelocity = velocity;
			mMoveToTheRight = true;
		}
		
		public boolean init() {
			boolean ok = true;
			
			SimObject obj1 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerLeft").get();
			SimObject obj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerRight").get();
			SimObject obj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Robot").get();

			
			if(obj1 ==null || obj2 == null || obj3 == null || 
				!(obj1 instanceof DistanceScanner) || !(obj2 instanceof DistanceScanner)
				|| !(obj3 instanceof OmniDrive)) 
			{
				ok = false;
			}
			else {
				mDistScanner1 = (DistanceScanner) obj1;
				mDistScanner2 = (DistanceScanner) obj2;
				mDrive = (OmniDrive) obj3;
			}
			return ok;
		}
		
		public boolean cleanUp() {
			return true;
		}
		
		public void updateInput() {
			
		}
		
		public void updateOutput() {
		}
		
		public void updateGlobalEvents() {
			
		}
		
		public void nextStep(int stepNumber) {
		}
		
		public void nextTry(int tryId) {
		}
		
		public void nextGeneration(int generationId) {
		}
		
		
	}
	*/
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new TestSimulationScenario();
		Simulator.getInstance().emulateEvolutionRun(1, 1, 1, -1);
	}


}
