package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.awt.Polygon;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.DistanceSensor;
import de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class TryOutReplicators extends SimbaDefaultFrame {
	
	public TryOutReplicators() {
		super("TryOutReplicators", 650, 900);
		
		setDefaultOffset(-309.0, -307.0);
		setDefaultZoom(1.0);
		setVisualizationInterval(1);
		setSimulationDelay(0);

		setMasterEnvironment(new MyEnvironment());
		Simulator.getInstance().init();
		
		String args[] = new String[1];
		args[0] = "sim";
		startSystem(Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface(), args);
	}
	
	

	
	public class MyEnvironment extends EnvironmentModel {
		
		private Vector mColors = new Vector();
		private Box mPlayGround;
		
		public MyEnvironment() {
			super("TryOutReplicators");
			
			mColors.add(Color.RED);
			mColors.add(Color.BLUE);
			mColors.add(Color.CYAN);
			mColors.add(Color.YELLOW);
			mColors.add(Color.MAGENTA);
		}
		

		public void updateRandomizationParameters() {
			
		}
		

		public void resetEnvironmentObjects() {
			
			mActiveIndividuals = 1;
			if(mAllCells.getSize() > 0) {
				CellA cell1 = (CellA) mAllCells.getElements().elementAt(0);
				cell1.setPosition(0.0, 0.0);
				cell1.setAngle(0.0);
				cell1.setActive(true);
				cell1.mDistScanner.setActive(true);
				cell1.mBumpDetector.setActive(false);
				cell1.mDistanceToPointScanner.setActive(true);
				changeColorGroup(cell1, cell1.mDistanceToPointScanner, cell1.getShapeColor());
				
				for(Iterator i = mAllCells.getIterator(); i.hasNext();) {
					CellA cell = (CellA) i.next();
					if(cell != cell1) {
						cell.setActive(false);
					}
				}
			}
			
			
		}

		public void createEnvironment() {
			
			for(int i = 0; i < mColors.size(); ++i) {
				mColorGroups.add(new SimObjectSet());
			}
			

			mPlayGround = new Box("", 0.0, 0.0, 0.0, 500.0, 600.0);
			mPlayGround.setLayer(-5);
			mPlayGround.setShapeColor(Color.GREEN);
			addSimObject(mPlayGround);
			

			for(int i = 0; i < 50; ++i) {
				CellA cell1 = new CellA("A", 0.0, 0.0, 0.0, Color.BLUE);
				cell1.setActive(false);
				addSimObject(cell1);
			}
		}
		
		public void changeColorGroup(SimObject object, DistanceSensor scanner, Color color) {
			scanner.clearObstacleSets();
			for(int i = 0; i < mColors.size(); ++i) {
				if(((Color) mColors.elementAt(i)) == color) {
					object.setShapeColor(color);
					SimObjectSet colorSet = (SimObjectSet) mColorGroups.elementAt(i);
					colorSet.add(object);
					scanner.addObstacleSet(colorSet);
					scanner.setObstacleDetectedColor(color);
				}
				else {
					SimObjectSet colorSet = (SimObjectSet) mColorGroups.elementAt(i);
					colorSet.remove(object);
				}
			}
		}
		
		private SimObjectSet mAllCells = new SimObjectSet();
		private Vector mColorGroups = new Vector();
		private int mActiveIndividuals = 0;
		
		
		
		
		
		public class CellA extends SimObject {
			
			private DistanceScanner mDistScanner;
			private int mReplicationCounter;
			private int mVelocityCounter;
			private int mAngularVelocityCounter;
			private Random mRandom = new Random();
			private double mVelocity;
			private double mAngularVelocity;
			private SimpleBumpDetector mBumpDetector;
			private DistanceToPointScanner mDistanceToPointScanner;
			private DistanceToPointScanner mDistanceToAllScanner;
			
			
			
			public CellA(String name, double x, double y, double angle, Color color) {
				super(name, x, y, angle, new Polygon());
				
				Polygon p = new Polygon();
				p.addPoint(12, 0);
				p.addPoint(0, -4);
				p.addPoint(0, 4);
				setShape(p);
				
				setShapeColor(color);
				
				mDistScanner = new DistanceScanner(getName() + "/Scanner", 0.0, 0.0, 0.0, 0.0, 130.0);
				mDistScanner.setReferenceObject(this);
				mDistScanner.addIgnorableObject(this);
				
				mBumpDetector = new SimpleBumpDetector(getName() + "/Bump", 0.0, 0.0, 0.0);
				mBumpDetector.setReferenceObject(this);
				mBumpDetector.addObstacle(mPlayGround);
				mBumpDetector.setActive(false);
				mBumpDetector.setLayer(10);
				mBumpDetector.setVisible(false);
				
				mDistanceToPointScanner = new DistanceToPointScanner(getName() + "/DistToPoint", 0.0, 0.0, 0.0, 0.0, 1000.0);
				mDistanceToPointScanner.setReferenceObject(this);
				mDistanceToPointScanner.addIgnorableObject(this);
				mDistanceToPointScanner.setLayer(5);
				
				mDistanceToAllScanner = new DistanceToPointScanner(getName() + "/DistToAll", 0.0, 0.0, 0.0, 0.0, 200.0);
				mDistanceToAllScanner.setReferenceObject(this);
				mDistanceToAllScanner.addIgnorableObject(this);
				mDistanceToAllScanner.setShapeColor(Color.WHITE);
				mDistanceToAllScanner.setObstacleDetectedColor(Color.WHITE);
				mDistanceToAllScanner.setLayer(4);
				
				mReplicationCounter = 10 + mRandom.nextInt(100); 
				mVelocity = mRandom.nextDouble();
				
				mDistScanner.setActive(true);
				mAllCells.add(this);
				
//				mDistanceToAllScanner.setVisible(false);
//				mDistanceToPointScanner.setVisible(false);
				mDistScanner.setVisible(false);
			}
			
			
			
			public void addChildren(EnvironmentModel model) {
				super.addChildren(model);
				model.addSimObject(mDistScanner);
				model.addSimObject(mBumpDetector);
				model.addSimObject(mDistanceToPointScanner);
				model.addSimObject(mDistanceToAllScanner);
			}



			public boolean init() {
				boolean ok = super.init();
				
				for(Iterator i = mAllCells.getIterator(); i.hasNext();) {
					SimObject o = (SimObject) i.next();
					if(o != this) {
						mDistanceToPointScanner.addDependency(o);
						mDistanceToAllScanner.addDependency(o);
						mDistScanner.addDependency(o);
					}
				}
				mDistanceToAllScanner.addObstacleSet(mAllCells);
				mDistScanner.addObstacleSet(mAllCells);
				mDistScanner.addObstacle(mPlayGround);
				mDistScanner.setTreatObstaclesAsMassive(false);
				
				return ok;
			}
			
			

			public void reset() {
				super.reset();
				mActiveIndividuals = 1;
			}

			public boolean update() {
				boolean ok = super.update();
				
				if(mBumpDetector.isActive() && mBumpDetector.getSensorReading() < 1.0) {
					setActive(false);
					mActiveIndividuals--;
				}
				else {
					if(!mBumpDetector.isActive()) {
						mBumpDetector.setActive(true);
					}
				}
				
//				if(this == mAllCells.getIterator().next()) {
//					Simulator.getInstance().reportStatus("Running: " + mActiveIndividuals);
//				}
				
				
				mVelocityCounter--;
				mAngularVelocityCounter--;
				mReplicationCounter--;
				
				double angleBias = 0.0;
				
				double maxVel = 1.0;
				if(mVelocityCounter <= 0) {
					mVelocity += ((mRandom.nextDouble()) * maxVel);
					if(mVelocity > maxVel) {
						mVelocity = maxVel;
					}
					if(mVelocity < -maxVel) {
						mVelocity = -maxVel;
					}
					mVelocityCounter = mRandom.nextInt(50);
				}
				if(mAngularVelocityCounter <= 0) {
					double modification = (- 0.5 + mRandom.nextDouble());
					if(modification > 0.0 && angleBias > 0.0) {
						modification *= -1;
					}
					else if(modification < 0.0 && angleBias < 0.0) {
						modification *= -1;
					}
					mAngularVelocity += modification;
					if(mAngularVelocity > 2.0) {
						mAngularVelocity = 2.0;
					}
					if(mAngularVelocity < -2.0) {
						mAngularVelocity = -2.0;
					}
					mAngularVelocityCounter = mRandom.nextInt(5);
				}
				//mAngularVelocity = 0.5;
				
				setAngle(getAngle() + mAngularVelocity);
				setPosition(getPosition().x + Math.cos(Math.toRadians(getAngle())) * mVelocity,
							getPosition().y + Math.sin(Math.toRadians(getAngle())) * mVelocity);
				
				
				//Replication
				if(mReplicationCounter < 0) {
					//replicate
					Color color = (mRandom.nextDouble() > 0.80) ? (Color) mColors.elementAt(mRandom.nextInt(mColors.size() - 1)) : getShapeColor();
					
					CellA cellA = null;
					
					for(Iterator i = mAllCells.getIterator(); i.hasNext();) {
						CellA obj = (CellA) i.next();
						if(!obj.isActive()) {
							cellA = obj;
							break;
						}
					}
					if(cellA != null) {
						cellA.setPosition(getPosition().x + (Math.cos(Math.toRadians(getAngle())) * 12.0),
								getPosition().y + (Math.sin(Math.toRadians(getAngle())) * 12.0));
						cellA.setAngle(getAngle());
						changeColorGroup(cellA, cellA.mDistanceToPointScanner, color);
						cellA.setActive(true);
						cellA.mDistScanner.setActive(true);
						cellA.mReplicationCounter = /*(mActiveIndividuals + 1) * 5 +*/ 50 + mRandom.nextInt(200);
						cellA.mBumpDetector.setActive(false);
						cellA.mDistanceToPointScanner.setActive(true);
						cellA.mDistanceToAllScanner.setActive(true);
						cellA.mVelocity = 0;
						cellA.mVelocityCounter = 10 + mRandom.nextInt(50);
					}
					mReplicationCounter = /*(mActiveIndividuals + 1) * 5 +*/ 50 + mRandom.nextInt(200);
					mActiveIndividuals = Math.min(mActiveIndividuals + 1, mAllCells.getSize());
					
				}
				
				
				return ok;
			}


			public void setActive(boolean active) {
				if(!active) {
					mDistScanner.setActive(false);
					mBumpDetector.setActive(false);
					mDistanceToPointScanner.setActive(false);
					mDistanceToAllScanner.setActive(false);
				}
				super.setActive(active);
			}
			
		}
	}
	
	
	
	public static void main(String[] args) {
		new TryOutReplicators();
		//Simulator.getInstance().emulateEvolutionRun(10, 2, 2, 2500);
	}
	
}