package de.fhg.iais.outdoor.simba.statistic;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionIndividual;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;
import de.fhg.iais.outdoor.simba.value.HistoryValue;

public class EvolutionStatistics extends StatisticCalculator implements GenerationListener {

	
	private HistoryValue mMeanAge;
	private HistoryValue mExecutionTime;
	private HistoryValue mNumberOfIndividuals;
	private long mStartingTime;
	
	
	public EvolutionStatistics() {
		super("EvolutionStatistics");
		
		mMeanAge = new HistoryValue();
		mExecutionTime = new HistoryValue();
		mNumberOfIndividuals = new HistoryValue();
		
		addParameter(mMeanAge, "MeanAge");
		addParameter(mExecutionTime, "ExecutionTime");
		addParameter(mNumberOfIndividuals, "NumberOfIndividuals");
		
		mStartingTime = 0;
		
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		Simulator.getInstance().getStatisticManager().addCalculator(this);
		
	}
	
	public void calculate() {
		
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		
		//Execution Time
		generationCompleted(0);
		
		//Mean Age
		Vector individuals = em.getGeneration();
		int ageSum = 0;
		int individualCount = 0;
		for(Iterator i = individuals.iterator(); i.hasNext();) {
			EvolutionIndividual ind = (EvolutionIndividual) i.next();
			ageSum += ind.getAge();
			individualCount++;
		}
		if(individualCount == 0) {
			mMeanAge.set(0.0);
		}
		else {
			mMeanAge.set(((double) ageSum) / ((double) individualCount));
		}
		
		//NumberOfIndividuals
		mNumberOfIndividuals.set((double) individualCount);
		
	}

	public void generationCompleted(int generationId) {
		if(mStartingTime != 0) {
			long endTime = System.currentTimeMillis();
			mExecutionTime.set(((double) (endTime - mStartingTime)) / 1000.0);
			mStartingTime = 0;
		}
	}

	public void generationStarted(int generationId) {
		mStartingTime = System.currentTimeMillis();
	}
	
	

}
