package de.fhg.iais.outdoor.simba.statistic;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.SystemObject;
import de.fhg.iais.outdoor.simba.evolution.EvolutionLogger;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;

public class StatisticManager implements GenerationListener, SystemObject {
	
	private Vector mCalculators = new Vector();
	
	public StatisticManager() {
		
	}
	
	public synchronized void calculateStatistics() {
		for(Iterator i = mCalculators.iterator(); i.hasNext();) {
			StatisticCalculator calc = (StatisticCalculator) i.next();
			if(calc.isActive()) {
				calc.calculate();
			}
		}
	}
	
	public synchronized void logStatistics() {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		Scheduler scheduler = Simulator.getInstance().getScheduler();
		EvolutionLogger logger = scheduler.getEvolutionLogger();
		
		logger.logXML("<statistics>", "</statistics>");
		
		if(em.getEvolutionRunningValue().get()) {
			//Statistics
			for(Iterator i = mCalculators.iterator(); i.hasNext();) {
				StatisticCalculator calc = (StatisticCalculator) i.next();
				if(calc.isActive()) {
					logger.logXML("<calculator name=\"" + calc.getName() + "\">", "</calculator>");
					String cEntry = calc.createParameterLog();
					logger.logXML(cEntry);
					logger.closeTag(); //calculator
				}
			}
		}
		logger.closeTag(); //statistics
	}
	
	public synchronized boolean addCalculator(StatisticCalculator calculator) {
		if(calculator == null || mCalculators.contains(calculator)) {
			return false;
		}
		mCalculators.add(calculator);
		return true;
	}

	public synchronized Vector getCalculators() {
		return new Vector(mCalculators);
	}

	public void generationCompleted(int generationId) {
		calculateStatistics();
		logStatistics();
	}

	public void generationStarted(int generationId) {
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		return true;
	}
	
	
	
	
}
