package de.fhg.iais.outdoor.simba.test;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import junit.framework.TestCase;

public class TestBoxSimObject extends TestCase {
	
	
	protected void setUp() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.BoxSimObject.BoxSimObject(double, double, double, double, double)'
	 */
	public void testBoxSimObject() {
		ValueManager vm = Simulator.getInstance().getValueManager();
		int valueCount = vm.getValues().size();
		
		Box box = new Box("Box1", 1.0, 2.0, 40.0, 10.0, 20.0);
		assertEquals(box.getName(), "Box1");
		assertEquals(box.getPosition(), new Point2D.Double(1.0, 2.0));
		assertEquals(box.getAngle(), 40.0, 0.0001);
		assertEquals(box.getWidth(), 10.0, 0.0001);
		assertEquals(box.getHeight(), 20.0, 0.0001);
		Polygon shape = box.getShape();
		assertEquals(shape.npoints, 4);
		assertEquals(shape.xpoints[0], -5);
		assertEquals(shape.ypoints[0], 10);
		assertEquals(shape.xpoints[1], 5);
		assertEquals(shape.ypoints[1], 10);
		assertEquals(shape.xpoints[2], 5);
		assertEquals(shape.ypoints[2], -10);
		assertEquals(shape.xpoints[3], -5);
		assertEquals(shape.ypoints[3], -10);
		
		int numberOfAttributes = 10;
		assertEquals(vm.getValues().size(), 2 + numberOfAttributes + valueCount);
		assertNotNull(vm.getSimObjectValue("/SimObjects/Box1"));
		assertNotNull(vm.getDoubleValue("/SimObjects/Box1/Transparency"));
		assertSame(vm.getSimObjectValue("/SimObjects/Box1").get(), box);
	}
	
	
	public void testGetShapeAsGlobalLineSegments() {
		Box box = new Box("Box1", 1.0, 2.0, 0.0, 4.0, 6.0);
		
		Vector segments = box.getShapeAsGlobalLineSegments();
		assertEquals(segments.size(), 4);
		Line2D.Double l1 = (Line2D.Double) segments.elementAt(0);
		Line2D.Double l2 = new Line2D.Double(-1.0, 5.0, 3.0, 5.0);
		
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(1);
		l2 = new Line2D.Double(3.0, 5.0, 3.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(2);
		l2 = new Line2D.Double(3.0, -1.0, -1.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(3);
		l2 = new Line2D.Double(-1.0, -1.0, -1.0, 5.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		
		Box box2 = new Box("Box2", -1.0, -2.0, -90.0, 1.0, 1.0);
		box.setReferenceObject(box2);
		
		//Without a call to update, the values are unchanged.
		segments = box.getShapeAsGlobalLineSegments();
		assertEquals(segments.size(), 4);
		l1 = (Line2D.Double) segments.elementAt(0);
		l2 = new Line2D.Double(-1.0, 5.0, 3.0, 5.0);
		
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(1);
		l2 = new Line2D.Double(3.0, 5.0, 3.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(2);
		l2 = new Line2D.Double(3.0, -1.0, -1.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(3);
		l2 = new Line2D.Double(-1.0, -1.0, -1.0, 5.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		
		//now with update
		box.update();
		
		segments = box.getShapeAsGlobalLineSegments();
		assertEquals(segments.size(), 4);
		l1 = (Line2D.Double) segments.elementAt(0);
		l2 = new Line2D.Double(4.0, -1.0, 4.0, -5.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(1);
		l2 = new Line2D.Double(4.0, -5.0, -2.0, -5.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(2);
		l2 = new Line2D.Double(-2.0, -5.0, -2.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) segments.elementAt(3);
		l2 = new Line2D.Double(-2.0, -1.0, 4.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.BoxSimObject.setWidth(double)'
	 */
	public void testSetWidth() {
		Box box = new Box("Box", -1.0, -2.0, -23.0, 2.0, 4.0);
		Polygon shape = box.getShape();
		assertEquals(shape.npoints, 4);
		assertEquals(shape.xpoints[0], -1);
		assertEquals(shape.ypoints[0], 2);
		assertEquals(shape.xpoints[1], 1);
		assertEquals(shape.ypoints[1], 2);
		assertEquals(shape.xpoints[2], 1);
		assertEquals(shape.ypoints[2], -2);
		assertEquals(shape.xpoints[3], -1);
		assertEquals(shape.ypoints[3], -2);
		
		box.setWidth(6.0);
		shape = box.getShape();
		assertEquals(shape.npoints, 4);
		assertEquals(shape.xpoints[0], -3);
		assertEquals(shape.ypoints[0], 2);
		assertEquals(shape.xpoints[1], 3);
		assertEquals(shape.ypoints[1], 2);
		assertEquals(shape.xpoints[2], 3);
		assertEquals(shape.ypoints[2], -2);
		assertEquals(shape.xpoints[3], -3);
		assertEquals(shape.ypoints[3], -2);
		
		box.setHeight(9.0);
		shape = box.getShape();
		assertEquals(shape.npoints, 4);
		assertEquals(shape.xpoints[0], -3);
		assertEquals(shape.ypoints[0], 5); //rounding up
		assertEquals(shape.xpoints[1], 3);
		assertEquals(shape.ypoints[1], 5);
		assertEquals(shape.xpoints[2], 3);
		assertEquals(shape.ypoints[2], -4); //rounding up
		assertEquals(shape.xpoints[3], -3);
		assertEquals(shape.ypoints[3], -4);
	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.BoxSimObject.createLine(Double, double, double, double)'
	 */
	public void testCreateLine() {
		Box boxObj = new Box("",
				1.0, 1.0, 45.0, 10.0, 10.0);
		
		
		double angle = 180.0 - Math.toDegrees(Math.atan(0.5)); //negative
		double width = 4.0 * 2.0 / Math.cos(angle * Math.PI / 180.0);
		double height = 2.0 * 2.0 / Math.sin((angle + 90.0) * Math.PI / 180.0); 
		
		Line2D.Double line = boxObj.createLine(
				new Point2D.Double(2.0, 3.0),
				width, 
				height, 
				angle);
		
		assertEquals(line.x1, 7.0, 0.001);
		assertEquals(line.y1, 3.0, 0.001);
		assertEquals(line.x2, 5.0, 0.001);
		assertEquals(line.y2, -1.0, 0.001);
		
		
		angle -= 180.0; //positive, so the line on the other side is calculated.
		
		line = boxObj.createLine(
				new Point2D.Double(2.0, 3.0),
				width, 
				height, 
				angle);
		
		assertEquals(line.x1, -3.0, 0.001);
		assertEquals(line.y1, 3.0, 0.001);
		assertEquals(line.x2, -1, 0.001);
		assertEquals(line.y2, 7.0, 0.001);
		
		//exchange width and height.
		angle += 180.0; //positive
		
		line = boxObj.createLine(
				new Point2D.Double(2.0, 3.0), 
				height,
				width,
				angle + 90.0); //this flips the calculation


		assertEquals(line.x1, -1.0, 0.001);
		assertEquals(line.y1, 7.0, 0.001);
		assertEquals(line.x2, 7.0, 0.001);
		assertEquals(line.y2, 3.0, 0.001);
		
		//this time negative
		angle -= 180.0; //positive
		
		line = boxObj.createLine(
				new Point2D.Double(2.0, 3.0), 
				height,
				width,
				angle + 90.0);

		assertEquals(line.x1, 5.0, 0.001);
		assertEquals(line.y1, -1.0, 0.001);
		assertEquals(line.x2, -3.0, 0.001);
		assertEquals(line.y2, 3.0, 0.001);
		
		
		//System.out.println("(" + angle + ")got: " + line.x1 + " / " + line.y1 + "  and " + line.x2 + " / " + line.y2);
		
		//Perpendicular example.
		line = boxObj.createLine(
				new Point2D.Double(0.0, 0.0), 2.0, 4.0, 0.0);
		
		assertEquals(line.x1, 1.0, 0.001);
		assertEquals(line.y1, 2.0, 0.001);
		assertEquals(line.x2, 1.0, 0.001);
		assertEquals(line.y2, -2.0, 0.001);
		
		line = boxObj.createLine(
				new Point2D.Double(0.0, 0.0), 2.0, 4.0, 180.0);
		
		assertEquals(line.x1, -1.0, 0.001);
		assertEquals(line.y1, -2.0, 0.001);
		assertEquals(line.x2, -1.0, 0.001);
		assertEquals(line.y2, 2.0, 0.001);
		
		line = boxObj.createLine(
				new Point2D.Double(0.0, 0.0), 4.0, 2.0, 90.0); //exchange width and height
		
		assertEquals(line.x1, -1.0, 0.001);
		assertEquals(line.y1, 2.0, 0.001);
		assertEquals(line.x2, 1.0, 0.001);
		assertEquals(line.y2, 2.0, 0.001);
		
		line = boxObj.createLine(
				new Point2D.Double(0.0, 0.0), 4.0, 2.0, 270.0);
		
		assertEquals(line.x1, 1.0, 0.001);
		assertEquals(line.y1, -2.0, 0.001);
		assertEquals(line.x2, -1.0, 0.001);
		assertEquals(line.y2, -2.0, 0.001);
	}

}
