package de.fhg.iais.outdoor.simba.test;

import java.awt.Polygon;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.CircularPolygon;
import junit.framework.TestCase;

public class TestCircularPolygon extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
		Simulator.getInstance().clearInstance();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}

	public void testConstructor() {
		CircularPolygon c1 = new CircularPolygon("Circ1", 1.0, 2.0, 3.0, 10.0, 9, 180.0);
		
		assertEquals(c1.getName(), "Circ1");
		assertEquals(c1.getPosition().x, 1.0, 0.001);
		assertEquals(c1.getPosition().y, 2.0, 0.001);
		assertEquals(c1.getAngle(), 3.0, 0.001);
		assertEquals(c1.getMaximalAngle(), 180.0, 0.001);
		assertEquals(c1.getRadius(), 10.0, 0.001);
		assertEquals(c1.getNumberOfCorners(), 9);
		
		
		CircularPolygon c2 = new CircularPolygon("Circ2", 5.0, 10.0, 15.0, 20.0, 30);
		
		assertEquals(c2.getName(), "Circ2");
		assertEquals(c2.getPosition().x, 5.0, 0.001);
		assertEquals(c2.getPosition().y, 10.0, 0.001);
		assertEquals(c2.getAngle(), 15.0, 0.001);
		assertEquals(c2.getMaximalAngle(), 360.0, 0.001);
		assertEquals(c2.getRadius(), 20.0, 0.001);
		assertEquals(c2.getNumberOfCorners(), 30);
		
	}

	public void testUpdateShape() {
		CircularPolygon c1 = new CircularPolygon("Circ1", 1.0, 2.0, 3.0, 10.0, 9, 180.0);
		
		Polygon shape = c1.getShape();
		
		assertEquals(shape.npoints, 9);
		assertEquals(shape.xpoints[0], 10);
		assertEquals(shape.ypoints[0], 0);
		assertEquals(shape.xpoints[1], 9);
		assertEquals(shape.ypoints[1], 4);
		assertEquals(shape.xpoints[2], 6);
		assertEquals(shape.ypoints[2], 8);
		assertEquals(shape.xpoints[3], 2);
		assertEquals(shape.ypoints[3], 10);
		assertEquals(shape.xpoints[4], -2);
		assertEquals(shape.ypoints[4], 10);
		assertEquals(shape.xpoints[5], -6);
		assertEquals(shape.ypoints[5], 8);
		assertEquals(shape.xpoints[6], -9);
		assertEquals(shape.ypoints[6], 4);
		assertEquals(shape.xpoints[7], -10);
		assertEquals(shape.ypoints[7], 0);
		assertEquals(shape.xpoints[8], 0);
		assertEquals(shape.ypoints[8], 0);
		
		
		c1 = new CircularPolygon("Circ2", 1.0, 2.0, 3.0, 10.0, 5);
		
		shape = c1.getShape();
		
		assertEquals(shape.npoints, 6);
		assertEquals(shape.xpoints[0], 10);
		assertEquals(shape.ypoints[0], 0);
		assertEquals(shape.xpoints[1], 3);
		assertEquals(shape.ypoints[1], 10);
		assertEquals(shape.xpoints[2], -8);
		assertEquals(shape.ypoints[2], 6);
		assertEquals(shape.xpoints[3], -8);
		assertEquals(shape.ypoints[3], -6);
		assertEquals(shape.xpoints[4], 3);
		assertEquals(shape.ypoints[4], -10);
		assertEquals(shape.xpoints[5], 10);
		assertEquals(shape.ypoints[5], 0);
		
		
		Vector lines = c1.getShapeAsGlobalLineSegments();
		assertEquals(lines.size(), 5); //lines between identical points are ignored ([0] and [5]).
		
	}

}


