package de.fhg.iais.outdoor.simba.test;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import de.fhg.iais.outdoor.simba.core.GeometricMath;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import junit.framework.TestCase;

public class TestGeometricMath extends TestCase {

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	/*
	 * TODO There should also be a testGetintersectionPoint with pure lines only.
	 */
	public void testGetIntersectionPointWithDistanceScanner() {
		DistanceScanner scanObj = new DistanceScanner("",
				1.0, 1.0, 45.0, 0.0, 4.5);
	
		
		Line2D.Double line = new Line2D.Double(7.0, 1.0, -1.0, 9.0);
		Point2D.Double intersection = new Point2D.Double();
		assertTrue(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		assertEquals(intersection.x, 4.0, 0.001);
		assertEquals(intersection.y, 4.0, 0.001);
		
		scanObj.setMaxScanDistance(4.0); //maxdist too short
		assertFalse(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		
		scanObj.setMaxScanDistance(5.0);
		scanObj.setMinScanDistance(4.5); //mindist too far (still is measured.)
		assertTrue(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		assertEquals(intersection.x, 4.0, 0.001);
		assertEquals(intersection.y, 4.0, 0.001);
		
		
		scanObj.setAngle(90.0); //angle 90 degree, but maxscandist too short
		scanObj.update();
		assertFalse(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		
		scanObj.setMaxScanDistance(6.0); //maxScanDist far enough, so intersection (at differnt pos)
		scanObj.update();
		assertTrue(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		assertEquals(intersection.x, 1.0, 0.001);
		assertEquals(intersection.y, 7.0, 0.001);
		
		line = new Line2D.Double(7.0, 1.0, 2.0, 6.0); //line now to short, no intersection.
		assertFalse(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		
		//direction: positive (line is on the wrong side
		line = new Line2D.Double(2.0, -6.0, -1.0, 0.0); 
		assertFalse(GeometricMath.getIntersectionPoint(line, scanObj.getGlobalScanLine(),intersection));
		
		//TODO test with different reference objects.
	}
	
	public void testDet() {
		assertEquals(GeometricMath.det(1.0, 1.0, 50.0, 100.0), 50.0, 0.001);
		assertEquals(GeometricMath.det(0.0, 0.0, 50.0, 100.0), 0.0, 0.001);
		assertEquals(GeometricMath.det(1.0, 1.0, -50.0, -100.0), -50.0, 0.001);
		assertEquals(GeometricMath.det(-1.0, 1.0, -50.0, 100.0), -50.0, 0.001);
		assertEquals(GeometricMath.det(0.0, 0.0, -50.0, -100.0), 0.0, 0.001);
		assertEquals(GeometricMath.det(-123.0, 1.0, -50.0, 12.0), -1426.0, 0.001);
		assertEquals(GeometricMath.det(27.0, 99.5, -8.3, -18.0), 339.85, 0.001);
	}
	
}
