package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.HistoryValue;
import junit.framework.TestCase;

public class TestHistoryValue extends TestCase {

	protected void setUp() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testHistory() {
		HistoryValue value = new HistoryValue();
		
		assertEquals(value.get(), 0.0, 0.0001);
		assertEquals(value.getHistory().size(), 1);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 0.0, 0.0001);
		
		value.set(99.23);
		assertEquals(value.get(), 99.23, 0.0001);
		assertEquals(value.getHistory().size(), 1);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		
		value.generationStarted(3);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 0.0, 0.0001);
		
		
		value.set(11.22);
		assertEquals(value.get(), 11.22, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 11.22, 0.0001);
		
		value.set(22.11);
		assertEquals(value.get(), 22.11, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 22.11, 0.0001);
		
		value.generationStarted(0);
		value.set(5.55);
		assertEquals(value.get(), 5.55, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 5.55, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 22.11, 0.0001);
		
		value.set(654.765, 2);
		assertEquals(value.get(), 5.55, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 5.55, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertEquals(((Double) value.getHistory().elementAt(2)).doubleValue(), 654.765, 0.0001);
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 22.11, 0.0001);
		
	}
	
	public void testGenerationListener() {
		HistoryValue value = new HistoryValue(100.5);
		assertEquals(value.getHistory().size(), 1);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 100.5, 0.0001);
		
		Simulator.getInstance().getScheduler().prepareNextGeneration(4, 10, 10);
		
		value.set(11.22);
		assertEquals(value.get(), 11.22, 0.0001);
		assertEquals(value.getHistory().size(), 5);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 100.5, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertNull(value.getHistory().elementAt(3));
		assertEquals(((Double) value.getHistory().elementAt(4)).doubleValue(), 11.22, 0.0001);
		
	}
}
