package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import junit.framework.TestCase;

public class TestNeuralNetAgentInterface extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
		Simulator.getInstance().clearInstance();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testConstructor() {
		NeuralNetAgentInterface controller = new NeuralNetAgentInterface("NeuralNetController1");
		assertEquals(controller.getName(), "NeuralNetController1");
		assertEquals(controller.getInputNeurons().size(), 0);
		assertEquals(controller.getOutputNeurons().size(), 0);
	}

	public void testAddInputOutputNeuron() {
		NeuralNetAgentInterface controller = new NeuralNetAgentInterface("");
		
		assertEquals(controller.getInputNeurons().size(), 0);
		assertEquals(controller.getOutputNeurons().size(), 0);
		
		NeuronWrapper w1 = new NeuronWrapper("Name", 1.0, 0.5, 1.5);
		NeuronWrapper w2 = new NeuronWrapper("Name2", 50.0, 25.0, 125.0);
		
		assertFalse(controller.addInputNeuron(null));
		assertTrue(controller.addInputNeuron(w1));
		assertEquals(controller.getInputNeurons().size(), 1);
		assertFalse(controller.addInputNeuron(w1));
		
		assertTrue(controller.addOutputNeuron(w1));
		assertEquals(controller.getInputNeurons().size(), 1);
		assertEquals(controller.getOutputNeurons().size(), 1);
		
		assertTrue(controller.addInputNeuron(w2));
		assertEquals(controller.getInputNeurons().size(), 2);
		
		assertTrue(controller.addOutputNeuron(w2));
		assertEquals(controller.getOutputNeurons().size(), 2);
		
	}

	

	public void testSetNeuronRange() {
		NeuralNetAgentInterface controller = new NeuralNetAgentInterface("");
		NeuronWrapper w1 = new NeuronWrapper("Name", 1.0, 0.5, 1.5);
		NeuronWrapper w2 = new NeuronWrapper("Name2", 50.0, 25.0, 125.0);
		
		controller.addInputNeuron(w1);
		controller.addOutputNeuron(w2);
		
		assertEquals(w1.get(), 1.0, 0.001);
		assertEquals(w1.getNormalized(), 0.5, 0.001);
		assertEquals(w1.getMinNormalizedValue(), 0.0, 0.001);
		assertEquals(w1.getMaxNormalizedValue(), 1.0, 0.001);
		
		assertEquals(w2.get(), 50.0, 0.001);
		assertEquals(w2.getNormalized(), 0.25, 0.001);
		assertEquals(w2.getMinNormalizedValue(), 0.0, 0.001);
		assertEquals(w2.getMaxNormalizedValue(), 1.0, 0.001);
		
		controller.setNeuronRange(-2.0, 2.0);
		
		assertEquals(w1.get(), 1.0, 0.001);
		assertEquals(w1.getNormalized(), 0.0, 0.001);
		assertEquals(w1.getMinNormalizedValue(), -2.0, 0.001);
		assertEquals(w1.getMaxNormalizedValue(), 2.0, 0.001);
		
		assertEquals(w2.get(), 50.0, 0.001);
		assertEquals(w2.getNormalized(), -1.0, 0.001);
		assertEquals(w2.getMinNormalizedValue(), -2.0, 0.001);
		assertEquals(w2.getMaxNormalizedValue(), 2.0, 0.001);
		
		
	}

}
