package de.fhg.iais.outdoor.simba.test;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;

public class TestTCPNeuroConnectionClient {
	
	public static final int FAILURE = -1;
	public static final int COMMAND_COMPLETE = 1;
	public static final int COMMAND_PREPARE_GENERATION = 2;
	public static final int COMMAND_PREPARE_INDIVIDUAL = 3;
	public static final int COMMAND_PREPARE_TRY = 4;
	public static final int COMMAND_EXECUTE_STEP = 5;
	public static final int COMMAND_TERMINATE = 6;
	public static final int COMMAND_RESET_SYSTEM = 7;
	public static final int COMMAND_GET_SYSTEM_INFO = 8;
	
	Socket mSocket;
	
	private int mCurrentTry;
	private int mCurrentGeneration;
	private int mCurrentIndividual;
	private int mCurrentStep;
	private int mNumberOfTries;
	private int mNumberOfInputNeurons;
	private int mNumberOfOutputNeurons;
	private int mPort;
	private DataInputStream mInput;
	private DataOutputStream mOutput;
	
	public TestTCPNeuroConnectionClient() {
		mCurrentTry = 0;
		mCurrentGeneration = 0;
		mCurrentIndividual = 0;
		mCurrentStep = 0;
		mNumberOfTries = 5;
	}
	
	public void start(int port) {
		mPort = port;
		
		connect();
		
		for(int g = 0; g < 10; ++g) {
			if(g != 0) {
				setNewStartPosition();
			}
			for(int i = 0; i < 100; ++i) {
				if(i != 0) {
					reset();
				}
				for(int t = 0; t < mNumberOfTries; ++t) {
					if(t != 0) {
						nextTry();
					}
					for(int s = 0; s < 1000; ++s) {
						send();
					}
				}
			}
		}

		disconnect();
		
	}
	

	public String getConfigFileName() {
		return "xml/simulator/yars.xml";
	}

	public String getSimName() {
		return "Simba:Maze";
	}

	public void send() {
		
		if(mCurrentGeneration == 0) {
			setNewStartPosition();
		}
		if(mCurrentTry == 0) {
			nextTry();
		}
			
		if(mInput != null && mOutput != null) {
			try {
				mOutput.writeInt(COMMAND_EXECUTE_STEP);
				mOutput.writeInt(mCurrentStep);

				for(int i = 0; i < mNumberOfOutputNeurons; ++i) {
					mOutput.writeDouble(0.2);
				}
				mOutput.flush();
				
				for(int i = 0; i < mNumberOfInputNeurons; ++i) {
					mInput.readDouble();
				}
				
				
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for executeStep command.");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void connect() {
		mSocket = new Socket();
		try {
			mSocket.connect(new InetSocketAddress("127.0.0.1", mPort));
			mInput = new DataInputStream(mSocket.getInputStream());
			mOutput = new DataOutputStream(mSocket.getOutputStream());
			
			mOutput.writeInt(COMMAND_GET_SYSTEM_INFO);
			mOutput.flush();
			
			mNumberOfInputNeurons = mInput.readInt();
			mNumberOfOutputNeurons = mInput.readInt();
			if(!(mInput.readInt() == COMMAND_COMPLETE)) {
				System.out.println("Communication problem: Did not receive CommandComplete signal for info command.");
			}
			System.out.println("Have: Input: " + mNumberOfInputNeurons + "  Output: " + mNumberOfOutputNeurons);
			
		} catch (IOException e) {
			System.out.println("Could not connect.");
			System.exit(0);
		}
	}

	public void disconnect() {
		if(mInput != null && mOutput != null) {
			try {
				mOutput.writeInt(COMMAND_TERMINATE);
				mOutput.flush();
				
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for terminate command.");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		try {
			mSocket.close();
			mInput = null;
			mOutput = null;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void nextTry() {
		if(mInput != null && mOutput != null) {
			try {
				mOutput.writeInt(COMMAND_PREPARE_TRY);
				mOutput.writeInt(mCurrentTry);
				mOutput.flush();
				
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for nextTry command.");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentStep = 0;
		mCurrentTry++;
	}

	public void reset() {
		if(mInput != null && mOutput != null) {
			try {
				mOutput.writeInt(COMMAND_PREPARE_INDIVIDUAL);
				mOutput.writeInt(mCurrentIndividual);
				mOutput.flush();
				
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for nextIndividual command.");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentTry = 0;
		mCurrentIndividual++;
	}

	public void setNewStartPosition() {
		
		if(mInput != null && mOutput != null) {
			try {
				mOutput.writeInt(COMMAND_PREPARE_GENERATION);
				mOutput.writeInt(mCurrentGeneration);
				mOutput.writeInt(mNumberOfTries);
				mOutput.flush();
				
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for nextGeneration command.");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentIndividual = 0;
		mCurrentGeneration++;
	}

	public void stop() {
		//nothing to do.
	}

	
	
	public static void main(String[] arg) {
		TestTCPNeuroConnectionClient client = new TestTCPNeuroConnectionClient();
		client.start(50100);
	}

}
