package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import junit.framework.TestCase;

public class TestValueManager extends TestCase {

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	
	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.ValueManager.ValueManager()'
	 */
	public void testValueManager() {
		ValueManager vm = new ValueManager();
		assertEquals(vm.getValues().size(), 0);
	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.ValueManager.getValue(String)'
	 */
	public void testAddAndGetValue() {
		ValueManager vm = new ValueManager();
		
		assertEquals(vm.getValues().size(), 0);
		assertFalse(vm.nameExists("/test/objects/obj1"));
		assertEquals(vm.getValuesStartingWith("/test/").size(), 0);
		
		DoubleValue dv1 = new DoubleValue(11.22);
		assertTrue(vm.addValue("/test/objects/obj1", dv1));
		assertTrue(vm.nameExists("/test/objects/obj1"));
		assertSame(vm.getValue("/test/objects/obj1"), dv1);
		assertNull(vm.getIntValue("/test/objects/obj1"));
		assertSame(vm.getDoubleValue("/test/objects/obj1"), dv1);
		assertEquals(vm.getValues().size(), 1);
		assertSame(vm.getValues().elementAt(0), dv1);
		assertEquals(vm.getValuesStartingWith("/test/").size(), 1);
		
		assertFalse(vm.addValue("/test/objects/obj1", dv1)); //adding with the same name again fails
		assertEquals(vm.getValues().size(), 1);
		
		assertTrue(vm.addValue("/test/objects/obj1Alias", dv1));
		assertEquals(vm.getValues().size(), 2);
		assertSame(vm.getValues().elementAt(0), dv1);
		assertSame(vm.getValues().elementAt(1), dv1);
		
		assertEquals(vm.getValuesStartingWith("/test/").size(), 1);
		assertSame(vm.getValuesStartingWith("/test/").elementAt(0), dv1);
		
		IntValue iv1 = new IntValue(100);
		assertFalse(vm.addValue("/test/objects/obj1", iv1)); //adding with the same name
		assertEquals(vm.getValues().size(), 2);
		
		assertTrue(vm.addValue("/test/objects/obj2", iv1));
		assertSame(vm.getValue("/test/objects/obj2"), iv1);
		assertNull(vm.getDoubleValue("/test/objects/obj2"));
		assertSame(vm.getIntValue("/test/objects/obj2"), iv1);
		assertEquals(vm.getValues().size(), 3);
		assertTrue(vm.getValues().contains(dv1));
		assertTrue(vm.getValues().contains(iv1));
		assertEquals(vm.getValuesStartingWith("/test/").size(), 2);
		assertTrue(vm.getValuesStartingWith("/test/").contains(iv1));
		assertTrue(vm.getValuesStartingWith("/test/").contains(dv1));
		
		BooleanValue bv = new BooleanValue(false);
		assertTrue(vm.addValue("/test/boolean/obj1", bv));
		assertSame(vm.getValue("/test/boolean/obj1"), bv);
		
		assertEquals(vm.getValuesMatchingPattern(".*obj1").size(), 2);
		assertTrue(vm.getValuesMatchingPattern(".*obj1").contains(bv));
		assertTrue(vm.getValuesMatchingPattern(".*obj1").contains(dv1));
		
		assertEquals(vm.getValuesMatchingPattern("/.*/object.*/obj.*").size(), 2);
		assertTrue(vm.getValuesMatchingPattern("/.*/object.*/obj.*").contains(iv1));
		assertTrue(vm.getValuesMatchingPattern("/.*/object.*/obj.*").contains(dv1));
		
		assertEquals(vm.getValuesMatchingPattern("/test/.*/obj.*").size(), 3);
		assertTrue(vm.getValuesMatchingPattern("/test/.*/obj.").contains(iv1));
		assertTrue(vm.getValuesMatchingPattern("/test/.*/obj.").contains(dv1));
		assertTrue(vm.getValuesMatchingPattern("/test/.*/obj.").contains(bv));
		
		assertEquals(vm.getValuesMatchingPattern(".*").size(), 3);
		assertTrue(vm.getValuesMatchingPattern(".*").contains(iv1));
		assertTrue(vm.getValuesMatchingPattern(".*").contains(dv1));
		assertTrue(vm.getValuesMatchingPattern(".*").contains(bv));
		
		assertEquals(vm.getValuesMatchingPattern("nothing").size(), 0);
		assertEquals(vm.getValuesMatchingPattern("*").size(), 0);
		
	}
	
	public void testValueChangedListeners() {
		DoubleValue dv1 = new DoubleValue(123.45);
		IntValue iv1 = new IntValue(999);
	
		TestValueChangedListener listener1 = new TestValueChangedListener();
		TestValueChangedListener listener2 = new TestValueChangedListener();
		
		dv1.addValueChangedListener(listener1);
		iv1.addValueChangedListener(listener1);
		dv1.addValueChangedListener(listener2);
		
		assertEquals(dv1.get(), 123.45, 0.0001);
		assertEquals(iv1.get(), 999);
		
		assertNull(listener1.mLastReportedValue);
		assertNull(listener2.mLastReportedValue);
		
		dv1.set(66.5);
		assertEquals(dv1.get(), 66.5, 0.0);
		assertSame(listener1.mLastReportedValue, dv1);
		assertSame(listener2.mLastReportedValue, dv1);
		
		iv1.set(1020);
		assertEquals(iv1.get(), 1020);
		assertSame(listener1.mLastReportedValue, iv1);
		assertSame(listener2.mLastReportedValue, dv1); // did not change
	}
	
	public void testValueReplacements() {
		DoubleValue dv1 = new DoubleValue(123.45);
		IntValue iv1 = new IntValue(999);
		
		ValueManager vm = new ValueManager();
		vm.addValue("DV1", dv1);
	
		TestValueChangedListener listener1 = new TestValueChangedListener();
		TestValueChangedListener listener2 = new TestValueChangedListener();
		
		dv1.addValueChangedListener(listener1);
		dv1.addValueReplacedListener(listener2);
		
		assertEquals(dv1.get(), 123.45, 0.0001);
		assertEquals(iv1.get(), 999);
		
		assertNull(listener1.mLastReportedValue);
		assertNull(listener1.mLastOldValue);
		assertNull(listener2.mLastReportedValue);
		assertNull(listener2.mLastOldValue);
		
		dv1.set(91.91);
		assertSame(listener1.mLastReportedValue, dv1);
		assertNull(listener1.mLastOldValue);
		assertNull(listener2.mLastReportedValue);
		assertNull(listener2.mLastOldValue);

		iv1.set(8234);
		assertSame(listener1.mLastReportedValue, dv1);
		assertNull(listener1.mLastOldValue);
		
		listener1.mLastReportedValue = null;
		assertNull(listener1.mLastReportedValue);
		
		assertFalse(vm.replaceValue("notContained", null));
		
		assertTrue(vm.replaceValue("DV1", iv1));
		assertSame(listener1.mLastReportedValue, iv1);
		assertSame(listener1.mLastOldValue, dv1);
		assertSame(listener2.mLastReportedValue, iv1); //this is the only case where the value was replaced.
		assertSame(listener2.mLastOldValue, dv1);
		
		
		//TODO Test the change value mechanism more closly (consider add / remove listener during notificatino)
		
	}
	
	public void testSynchronizeValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		DoubleValue d1 = new DoubleValue(15.0);
		DoubleValue d2 = new DoubleValue(55.1);
		IntValue i1 = new IntValue(20);
		
		d1.addValueReplacedListener(listener);
		assertEquals(d1.getValueReplacedListeners().size(), 1);
		assertSame(d1.getValueReplacedListeners().elementAt(0), listener);
		assertEquals(d2.getValueReplacedListeners().size(), 0);
		
		//nothing happens
		assertSame(ValueManager.synchronizeValue(d1, d2, d1, listener), d1);
		assertEquals(d1.getValueReplacedListeners().size(), 1);
		assertSame(d1.getValueReplacedListeners().elementAt(0), listener);
		assertEquals(d2.getValueReplacedListeners().size(), 0);
		
		//wrong type
		assertSame(ValueManager.synchronizeValue(d1, d1, i1, listener), d1);
		assertEquals(d1.getValueReplacedListeners().size(), 1);
		assertSame(d1.getValueReplacedListeners().elementAt(0), listener);
		assertEquals(d2.getValueReplacedListeners().size(), 0);
		
		//null
		assertSame(ValueManager.synchronizeValue(d1, d1, null, listener), d1);
		assertEquals(d1.getValueReplacedListeners().size(), 1);
		assertSame(d1.getValueReplacedListeners().elementAt(0), listener);
		assertEquals(d2.getValueReplacedListeners().size(), 0);
		
		//other double value //listeners change
		assertSame(ValueManager.synchronizeValue(d1, d1, d2, listener), d2);
		assertEquals(d1.getValueReplacedListeners().size(), 0);
		assertEquals(d2.getValueReplacedListeners().size(), 1);
		assertSame(d2.getValueReplacedListeners().elementAt(0), listener);
		
	}
	
	public class TestValueChangedListener implements ValueChangedListener {
		public Value mLastReportedValue;
		public Value mLastOldValue;
		
		public void valueChanged(Value value) {
			mLastReportedValue = value;
		}
		
		/**
		 * Called if the one of the registered Value objects were replaced in the ValueManager.
		 */
		 public void valueReplaced(Value oldValue, Value newValue) {
			mLastOldValue = oldValue;
			mLastReportedValue = newValue;
		 }
	}
	
	
	
	
}
