package de.fhg.iais.outdoor.simba.test;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Polygon;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.StepListener;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.CircularPolygon;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.OmniDrive;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.visualization.SimulationVisualization;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.VisualizationArea;

public class TestVisuFrame {
	
	public JFrame mFrame;
	public JPanel mContentPanel;
	public Scheduler mScheduler;
	public SimulationVisualization mVisu;
	
	public TestVisuFrame() {
		init();
	}
	
	public void init() {
		mFrame = new JFrame("TestVisuFrame");
		mFrame.setSize(300, 200);
		
		mContentPanel = new JPanel(new BorderLayout());
		mFrame.getContentPane().setLayout(new BorderLayout());
		mFrame.getContentPane().add(mContentPanel, BorderLayout.CENTER);
	
		mScheduler = Simulator.getInstance().getScheduler();
		Simulator.getInstance().getScheduler().setMasterEnvironment(new MyEnvironment());
		
		mVisu = new SimulationVisualization();
		mScheduler.addStepListener(mVisu);
		VisualizationArea root = Simulator.getInstance().getVisualizationManager().getVisualizationArea("root");
		
		root.addVisualization(mVisu, BorderLayout.CENTER);
		mFrame.getContentPane().add(root.getPanel(), BorderLayout.CENTER);
		
		TestController controller = new TestController();
		mScheduler.addStepListener(controller);
		
		mFrame.addWindowListener(
				new WindowAdapter() {
					public void windowClosing(WindowEvent e) {
						System.exit(0);
					}
					
				}
			);
		
		Vector visus = Simulator.getInstance().getVisualizationManager().getVisualizations();
		for(int i = 0; i < visus.size(); ++i) {
			Visualization v = (Visualization) visus.elementAt(i);
			if(v instanceof SimulationVisualization) {
				((SimulationVisualization) v).setOffset(-250.0, -200.0);
			}
		}
		
		Simulator.getInstance().init();
		
		mFrame.setVisible(true);
		
	}
	
	public class MyEnvironment extends EnvironmentModel {
		


		public void resetEnvironmentObjects() {
			// TODO Auto-generated method stub
			
		}
		
		



		public void updateRandomizationParameters() {
			// TODO Auto-generated method stub
			
		}





		public void createEnvironment() {

			Polygon p = new Polygon();
			p.addPoint(-40, 0);
			p.addPoint(20, 20);
			p.addPoint(0, 80);
			p.addPoint(40, 0);
			SimObject box1 = new SimObject("MyBox", 100.0, 100.0, 10.0, p);
			Box box2 = new Box("MyBox2", 250.0, 50.0, 90.0, 40.0, 20.0);
			Box box3 = new Box("MyBox3", -10.0, -20.0, -45.0, 40.0, 20.0);
			Box box4 = new Box("MyBox4", 150.0, 100.0, 0.0, 60.0, 40.0);
			box4.setShapeTransparency(0.5);
			CircularPolygon circular = new CircularPolygon("Circ", 200.0, 200.0, 0.0, 40.0, 9, 720.0);
			circular.setShapeColor(Color.ORANGE);
			TestActuator act1 = new TestActuator("TestActuator", 100.0, 100.0, 0.0, 4.0, 4.0, 5.0);
			act1.setSteps(10);
			DistanceScanner scan1 = new DistanceScanner("Scanner1", 0.0, 0.0, 0.0, 0.0, 300.0);
			scan1.setReferenceObject(act1);
			scan1.addObstacle(box1);
			scan1.addObstacle(box2);
			scan1.addObstacle(circular);
			DistanceScanner scan2 = new DistanceScanner("Scanner2", 0.0, 0.0, 45.0, 0.0, 200.0);
			scan2.setReferenceObject(scan1);
			scan2.addObstacle(box2);
			scan2.addObstacle(circular);
			DifferentialDriveUnit drive = new DifferentialDriveUnit("Driver", 150.0, 150.0, 0.0, 20.0, 4.0, -10.0, 10.0);
			drive.setRightVelocity(5.0);
			drive.setLeftVelocity(2.0);
			drive.setLayer(50);
			drive.setShapeColor(Color.MAGENTA);
			Polygon p2 = new Polygon();
			p2.addPoint(-10, -5);
			p2.addPoint(0, 5);
			p2.addPoint(10, -5);
			SimObject obj7 = new SimObject("RobotBody", 0.0, 0.0, 0.0, p2);
			obj7.setReferenceObject(drive);
			obj7.setLayer(51);
			
			OmniDrive omni = new OmniDrive("Omni", 0.0, 0.0, 0.0);
			Box omnicassis = new Box("Cassis", 0.0, 0.0, 0.0, 10.0, 20.0);
			omnicassis.setReferenceObject(omni);
			omni.setVelocityX(2.0);
			omni.setVelocityY(2.0);
			omni.setAngularVelocity(2.0);
			
			Box test = new Box("Testt", 0.0, 0.0, 45.0, 100.0, 100.0);
			
			SimpleBumpDetector bump1 = new SimpleBumpDetector("Bump1", 0.0, 0.0, 0.0);
			bump1.setReferenceObject(omnicassis);
			bump1.setLayer(100);
			bump1.addObstacle(test);
			bump1.addObstacle(circular);
			
			
			

			//scan2.addObstacle(box3);
			box3.setReferenceObject(act1);
			box1.setShapeColor(Color.GREEN);
			box2.setShapeColor(Color.YELLOW);
			box3.setShapeColor(Color.CYAN);
			box3.setFillShape(false);
			act1.setFillShape(true);
			act1.setShapeColor(Color.RED);
			box4.setShapeColor(Color.blue);
			
			box1.setLayer(-5);
			box4.setLayer(1);
			box3.setLayer(2);
			
			
			addSimObject(box1);
			addSimObject(box2);
			addSimObject(box3);
			addSimObject(box4);
			addSimObject(act1);
			addSimObject(scan1);
			addSimObject(scan2);
			addSimObject(drive);
			addSimObject(circular);
			addSimObject(obj7);
			
			
			addSimObject(test);
			
			addSimObject(omni);
			addSimObject(omnicassis);
			addSimObject(bump1);

		}
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new TestVisuFrame();
		SimObject obj = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Scanner1").get();
		SimObject obj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Scanner2").get();
		SimObject obj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/MyBox2").get();
		SimObject obj4 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/TestActuator").get();
		
		Scheduler s = Simulator.getInstance().getScheduler();
		s.prepareNextGeneration(0, 1, -1);
		s.prepareNextIndividual(0);
		s.prepareNextTry(0);
		
		
		boolean boxActive = true;
		int counter = 0;
		int sign = 1;
		int sign2 = 1;
		while(true) {
			try {
				Thread.sleep(100);
			}
			catch(InterruptedException e) {
				
			}
			obj.setAngle(obj.getAngle() + 3.0);
			obj2.setAngle(obj2.getAngle() + 1.0);
			obj4.setAngle(obj4.getAngle() + 2.0);
			//visu.mVisu.setScale(visu.mVisu.getScale() + (0.01 * sign));
			//visu.mVisu.setOffset(visu.mVisu.getOffset().x + (-5 * sign2), visu.mVisu.getOffset().y + (0.2 * sign2));
			if(counter % 27 == 0) {
				sign2 = sign2 * -1;
			}
			if(counter++ % 40 == 0) {
				sign = sign * -1;
				boxActive = !boxActive;
				obj3.setActive(boxActive);
				//visu.mVisu.setScale(visu.mVisu.getScale() * 0.8);
				
			}
			Simulator.getInstance().getScheduler().executeSimulationStep(0);
			
		}
	}
	
	public class TestController implements StepListener {
		
		private SimObject mObj1;
		private SimObject mObj2;
		private SimObject mOBj3;
		private int mCounter;
		private int mSign;
		
		public TestController() {
			mCounter = 0;
			mSign = 1;
		}
		
		public boolean init() {
			boolean ok = true;
			
			mObj1 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/MyBox").get();
			mObj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/MyBox2").get();
			mOBj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Circ").get();
			if(mObj1 ==null || mObj2 == null) {
				ok = false;
			}
			return ok;
		}
		
		public boolean cleanUp() {
			return true;
		}
		
		
		public void stepStarted(int stepNumber) {
			mCounter++;
			if(mCounter % 6 == 0) {
				Color c = mObj1.getShapeColor();
				mObj1.setShapeColor(mObj2.getShapeColor());
				mObj2.setShapeColor(c);
			}
			if(mCounter % 10 == 0) {
				
				int corners = ((CircularPolygon) mOBj3).getNumberOfCorners() + mSign;
				
				if(corners <= 3 || corners >= 12) {
					mSign = mSign * -1;
				}
				((CircularPolygon) mOBj3).setNumberOfCorners(corners);
			}
		}
		
		public void stepCompleted(int stepNumber) {
			Point2D.Double pos1 = mObj1.getPosition();
			if(pos1.x > 300.0) {
				pos1.x = 30.0;
			}
			mObj1.setPosition(pos1.x + 1, pos1.y);
			mOBj3.setAngle(mOBj3.getAngle() + 4.0);
		}
		
		public void tryStarted(int tryId) {
			
		}
		
		public void tryCompleted(int tryId) {
			
		}
		
		public void generationStarted(int generationId) {
			
		}
		
		public void generationCompleted(int generationId) {
			
		}
		
		
	}

}
