package de.fhg.iais.outdoor.simba.value;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Simulator;

public class HistoryValue extends DoubleValue implements GenerationListener {


	private Vector mHistory = new Vector();
	private int mCurrentGeneration;
	
	public HistoryValue() {
		this(0.0);
	}
	
	public HistoryValue(double initValue) {
		mCurrentGeneration = 0;
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		set(initValue);
	}

	public synchronized void set(double value) {
		super.set(value);
		while(mHistory.size() < mCurrentGeneration) {
			mHistory.add(null);
		}
		if(mCurrentGeneration >= 0) {
			mHistory.add(mCurrentGeneration, new Double(value));
			if(mHistory.size() > mCurrentGeneration + 1) {
				mHistory.remove(mCurrentGeneration + 1);
			}
		}
	}
	
	public synchronized void set(double value, int generation) {
		while(mHistory.size() < generation) {
			mHistory.add(null);
		}
		mHistory.add(generation, new Double(value));
		if(mHistory.size() > generation + 1) {
			mHistory.remove(generation + 1);
		}
	}
	
	/**
	 * Returns the internal Vector with the history.
	 * Modifications to this Vector will have an effect on the internal data representation.
	 * Warning: This is not a copy due to performance considerations.
	 * @return
	 */
	public Vector getHistory() {
		return mHistory;
	}

	public void generationCompleted(int generationId) {
	}

	public void generationStarted(int generationId) {
		mCurrentGeneration = generationId;
		set(0.0);
	}
	
	
	
	
	
	
	
	
}
