package de.fhg.iais.outdoor.simba.value;

public class NormalizedDoubleWrapper extends Value implements ValueChangedListener {

protected NormalizedDoubleValue mValue;
	
	public NormalizedDoubleWrapper(NormalizedDoubleValue value) {
		super();
		if(value == null) {
			throw new IllegalArgumentException("The parameter value may not be null");
		}
		mValue = value;
		mValue.addValueChangedListener(this);
	}
	
	/**
	 * Returns the double value represented by this DoubelValue object.
	 * 
	 * @return the value of this DoubleValue.
	 */
	public synchronized double get() {
		return mValue.getNormalized();
	}
	
	/**
	 * Changes the encapsulated double value to the value given as parameter.
	 * 
	 * @param value the new value of this DoubleValue.
	 */
	public synchronized void set(double value) {
		mValue.setNormalized(value);
		notifyValueChanged();
	}
	
	/**
	 * Returns the content of the Value as String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return the string representation of the Values content.
	 */
	public String getValueAsString() {
		return Double.toString(mValue.getNormalized());
	}
	
	
	/**
	 * Sets the content of the Value from a String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return true if the parameter could successfully be parsed to set the value.
	 */
	public boolean setValueFromString(String value) {
		return mValue.setNormalizedValueFromString(value);
	}

	public void valueChanged(Value value) {
		if(value != null && value == mValue) {
			notifyValueChanged();
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mValue = (NormalizedDoubleValue) ValueManager.synchronizeValue(mValue, oldValue, newValue, this);
	}
	
	
	
}
