package de.fhg.iais.outdoor.simba.value;

import java.util.Iterator;
import java.util.Vector;


public class Value {
	
	private Vector mValueChangedListeners = new Vector();
	private Vector mValueReplacedListeners = new Vector();
	private Vector mListenerBufferVector = new Vector();
	private int mNotifyCount;
	
	public Value() {
		mNotifyCount = 0;
	}
	
	/**
	 * Returns the content of the Value as String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return the string representation of the Values content.
	 */
	public String getValueAsString() {
		return "";
	}
	
	
	/**
	 * Sets the content of the Value from a String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return true if the parameter could successfully be parsed to set the value.
	 */
	public boolean setValueFromString(String value) {
		return false;
	}
	
	
	/**
	 * Adds a ValueChangedListener.
	 * @param listener the listener to add.
	 * @return true if successful, false if already contained.
	 */
	public synchronized boolean addValueChangedListener(ValueChangedListener listener) {
		if(listener == null || mValueChangedListeners.contains(listener)) {
			return false;
		}
		mValueChangedListeners.add(listener);
		return true;
	}
	
	
	/**
	 * Removes a ValueChangedListener from a Value object.
	 * 
	 * @param listener the listener to remove.
	 * @return true if successful. False if the listener was not contained.
	 */
	public synchronized boolean removeValueChangedListener(ValueChangedListener listener) {
		if(listener == null || !mValueChangedListeners.contains(listener)) {
			return false;
		}
		mValueChangedListeners.remove(listener);
		return true;
	}
	
	/**
	 * Adds a ValueReplacedListener.
	 * @param listener the listener to add.
	 * @return true if successful, false if already contained.
	 */
	public synchronized boolean addValueReplacedListener(ValueReplacedListener listener) {
		if(listener == null || mValueReplacedListeners.contains(listener)) {
			return false;
		}
		mValueReplacedListeners.add(listener);
		return true;
	}
	
	
	/**
	 * Removes a ValueReplacedListener from a Value object.
	 * 
	 * @param listener the listener to remove.
	 * @return true if successful. False if the listener was not contained.
	 */
	public synchronized boolean removeValueReplacedListener(ValueReplacedListener listener) {
		if(listener == null || !mValueReplacedListeners.contains(listener)) {
			return false;
		}
		mValueReplacedListeners.remove(listener);
		return true;
	}
	
	/**
	 * Returns a Vector with all registered ValueChangedListeners.
	 * @return all ValueChangedListeners.
	 */
	public Vector getValueChangedListeners() {
		return new Vector(mValueChangedListeners);
	}
	
	/**
	 * Returns a Vector with all registered ValueChangedListeners.
	 * @return all ValueChangedListeners
	 */
	public Vector getValueReplacedListeners() {
		return new Vector(mValueReplacedListeners);
	}
	
	/**
	 * Notifies all listeners that the value of this Value changed.
	 * @return
	 */
	public  synchronized void notifyValueChanged() {
		
		if(mNotifyCount <= 0) {
			mListenerBufferVector.clear();
			mListenerBufferVector.addAll(mValueChangedListeners);
			mNotifyCount = 0;
		}
		mNotifyCount++;
		for(Iterator i = mListenerBufferVector.iterator(); i.hasNext();) {
			((ValueChangedListener) i.next()).valueChanged(this);
		}
		mNotifyCount--;
	}

	/**
	 * Notifies all listeners that the value of this Value was replaced by a new one.
	 * @return
	 */
	public synchronized void notifyValueReplaced(Value newValue) {
		mListenerBufferVector.clear();
		mListenerBufferVector.addAll(mValueChangedListeners);
		for(Iterator i = mListenerBufferVector.iterator(); i.hasNext();) {
			((ValueChangedListener) i.next()).valueReplaced(this, newValue);
		}
		mListenerBufferVector.clear();
		mListenerBufferVector.addAll(mValueReplacedListeners);
		for(Iterator i = mListenerBufferVector.iterator(); i.hasNext();) {
			((ValueReplacedListener) i.next()).valueReplaced(this, newValue);
		}
	}
	
}
