package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;


public class FrameVisualizationArea extends VisualizationArea {

	
	private JFrame mDialog;
	private Action mTriggerAction;
	private JPanel mTriggerButtonArea;
	private JButton mTriggerDialogButton;
	private String mName;
	private boolean mVisible;
	
	
	public FrameVisualizationArea(String name, int width, int height, int x, int y) {
		
		mName = name;
		
		mDialog = new JFrame();
		mDialog.setTitle(name + " Dialog");
		mDialog.setSize(width, height);
		mDialog.setLocation(x, y);
		mDialog.getContentPane().setLayout(new BorderLayout());
		
		mVisible = false;
		
		mTriggerAction = new AbstractAction(name) {
			public static final long serialVersionUID = 1000L;
			
			public void actionPerformed(ActionEvent e) {
				mDialog.setVisible(!mDialog.isVisible());
				mVisible = mDialog.isVisible() 
				&& (mDialog.getExtendedState() & JFrame.ICONIFIED) == 0;
				notifyVisibilityChanged(mVisible);
			}
		};
		
		mDialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				mVisible = false;
				notifyVisibilityChanged(false);
			}
			public void windowIconified(WindowEvent e) {
				mVisible = false;
				notifyVisibilityChanged(false);
			}
			public void windowDeiconified(WindowEvent e) {
				mVisible = true;
				notifyVisibilityChanged(true);
			}
		});
		
		
		//Trigger button area.
		mTriggerButtonArea = new JPanel();
		mTriggerDialogButton = new JButton(mTriggerAction);
		mTriggerButtonArea.add(mTriggerDialogButton);
		
	
	}
	
	
	/**
	 * Installs a visualization to the JDialog. The layout of the JDialogs content panel
	 * is the BorderLayout, so the position identifiers of BorderLayout can be used to
	 * specify the desired location.
	 */
	protected boolean installVisualization(Visualization visu, String positionIdentifier) {
		mDialog.getContentPane().add(visu.getPanel(), positionIdentifier);
		return true;
	}

	/**
	 * Returns a JPanel containing a JButton that encapsulates the trigger Action
	 * to change the visibility of the JDialog.
	 */
	public JPanel getPanel() {
		return mTriggerButtonArea;
	}
	
	/**
	 * Returns the Action object that triggers the visibility change of the 
	 * JDialog.
	 * @return the visibility change trigger Action.
	 */
	public Action getTriggerAction() {
		return mTriggerAction;
	}


	public void visibilityChanged(boolean visible, Object source) {
		//do nothing, intercept the notification chain here, because each Frame has its own visibility.
	}


	public boolean cleanUp() {
		boolean ok = super.cleanUp();
		
		Properties props = Simulator.getInstance().getSystemProperties();
		String visible = mVisible ? "T" : "F";
		props.put("/FrameVisualizations/" + mName, 
				Integer.toString(mDialog.getWidth()) + "," + 
				Integer.toString(mDialog.getHeight()) + "," + 
				Integer.toString(mDialog.getX()) + "," + 
				Integer.toString(mDialog.getY()) + "," +
				visible);
		
		return ok;
	}


	public boolean init() {
		boolean ok = super.init();
		
		Properties props = Simulator.getInstance().getSystemProperties();
		String locParams = props.getProperty("/FrameVisualizations/" + mName);
		if(locParams != null) {
			String locs[] = locParams.split(",");
			if(locs.length == 5) {
				try {
					int width = Integer.parseInt(locs[0]);
					int height = Integer.parseInt(locs[1]);
					int x = Integer.parseInt(locs[2]);
					int y = Integer.parseInt(locs[3]);
					
					if(locs[4].equals("T")) {
						mDialog.setVisible(true);
						mVisible = mDialog.isVisible() && (mDialog.getExtendedState() & JFrame.ICONIFIED) == 0;
						notifyVisibilityChanged(mVisible);
					}
					else {
						mDialog.setVisible(false);
						mVisible = mDialog.isVisible() && (mDialog.getExtendedState() & JFrame.ICONIFIED) == 0;
						notifyVisibilityChanged(mVisible);
					}
					
					mDialog.setSize(width, height);
					mDialog.setLocation(x, y);
					mDialog.validate();
					
				} catch (NumberFormatException e) {
				}
			}
		}
		
		return ok;
	}

	
	
	
	

}
