package de.fhg.iais.outdoor.simba.visualization;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.Simulator;

public class OptimizePerformanceButton implements Visualization, OptimizePerformanceListener {

	
	private JPanel mDisplay;
	private JToggleButton mOptimizeButton;
	private Timer mTimer;
	private int mAutoOptimizeDelay;

	public OptimizePerformanceButton() {
		mDisplay = new JPanel();
		mOptimizeButton = new JToggleButton("Performance");
		mDisplay.add(mOptimizeButton);
		
		mOptimizeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mOptimizeButton.isSelected()) {
					if(!Simulator.getInstance().isPerformanceOptimized()) {
						Simulator.getInstance().optimizePerformance(true);
					}
				}
				else {
					if(Simulator.getInstance().isPerformanceOptimized()) {
						Simulator.getInstance().optimizePerformance(false);
					}
				}
			}
		});
		
		mAutoOptimizeDelay = 1000 * 60 * 15; //15 Minuten.
		mTimer = new Timer();
		mTimer.schedule(new AutoPerformanceTimerTask(), mAutoOptimizeDelay, mAutoOptimizeDelay); //15 minuten
	
		
		Simulator.getInstance().addOptimizePerformanceListener(this);
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}

	public void optimizePerformance(boolean optimize) {
		mOptimizeButton.setSelected(optimize);
		if(mTimer != null) {
			mTimer.cancel();
		}
		if(!optimize) {
			mTimer = new Timer();
			mTimer.schedule(new AutoPerformanceTimerTask(), mAutoOptimizeDelay, mAutoOptimizeDelay); //15 minuten
		}
	}

	public class AutoPerformanceTimerTask extends TimerTask {
		
		public void run() {
			if(!Simulator.getInstance().isPerformanceOptimized()) {
				Simulator.getInstance().optimizePerformance(true);
				Simulator.getInstance().reportStatus("Automatic optimization delay of " + mAutoOptimizeDelay / 60 / 1000 + " minutes expired.");
			}
		}
	}
	
	
	
}
