package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.geom.Point2D;
import java.text.DecimalFormat;
import java.util.logging.Logger;

import javax.swing.JLabel;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;

public class ShowDraggedObjectPositionVisualization implements Visualization, ValueChangedListener {

	private JPanel mDisplay;
	private JLabel mLabel;
	private SimObjectValue mDraggedObject;
	private boolean mHasDraggedObject;
	
	public ShowDraggedObjectPositionVisualization() {
		mDisplay = new JPanel(new BorderLayout());
		mLabel = new JLabel("<no object selected>");
		mDisplay.add(BorderLayout.CENTER, mLabel);
		
	}
	public boolean init() {
		boolean ok = true;
		
		acquireValueObjects();
		
		return ok;
	}
	
	/**
	 * Acquires the Value object containing the DraggedObject from the ValueManager.
	 */
	private void acquireValueObjects() {
		mDraggedObject = Simulator.getInstance().getValueManager()
		.getSimObjectValue("/Visualization/Control/DraggedObject");

		if(mDraggedObject == null) {
			Logger.getLogger("Simulator")
				.warning("ShowDraggedObjectPositionVisualization: Could not find Value object:\n" +
						"[/Visualization/Control/DraggedObject]");
		}
		else {
			mDraggedObject.addValueChangedListener(this);
		}
	}
	
	public boolean cleanUp() {
		boolean ok = true;
		
		return ok;
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}
	
	/**
	 * Called when the Slide value changed. The simulation delay will be updated
	 * according to the current slider value. If the slider is set to its maximum,
	 * then the simulation is paused (the IntValue /simulation/pause is set to 1).
	 */
	public void valueChanged(Value value) {
		if(mDraggedObject != null && value == mDraggedObject) {
			if(mDraggedObject.get() == null) {
				if(mHasDraggedObject) {
					mLabel.setText("Last " + mLabel.getText());
					mHasDraggedObject = false;
				}
			}
			else {
				SimObject obj = mDraggedObject.get();
				Point2D.Double local = obj.getPosition();
				Point2D.Double global = obj.getGlobalPosition();
				mLabel.setText("Object: " + obj.getName() + " local (" + 
						DecimalFormat.getNumberInstance().format(local.x) + "/" + 
						DecimalFormat.getNumberInstance().format(local.y) + "/" + 
						DecimalFormat.getNumberInstance().format(obj.getAngle())
						+ ") global (" + 
						DecimalFormat.getNumberInstance().format(global.x) + "/" + 
						DecimalFormat.getNumberInstance().format(global.y) + "/" + 
						DecimalFormat.getNumberInstance().format(obj.getGlobalAngle()) + ")");
				mHasDraggedObject = true;
			}
		}
	}
	
	/**
	 * Called if the one of the registered Value objects were replaced in the ValueManager.
	 */
	public void valueReplaced(Value oldValue, Value newValue) {
		if(oldValue != newValue && oldValue != null) {
			oldValue.removeValueChangedListener(this);
			if(mDraggedObject == oldValue) {
				acquireValueObjects();
			}
		}
	}

}
