package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.StepListener;
import de.fhg.iais.outdoor.simba.core.TryListener;

public class SimulationRecorder implements Visualization, StepListener, TryListener, GenerationListener {

	private JPanel mDisplay;
//	private JPanel mControlPlaybackPanel;
	private JPanel mControlMainPanel;
	private JPanel mFilePanel;
	private JButton mRecordButton;
	private JButton mStopButton;
	private JButton mPlaybackButton;
	private JTextField mFileNameTextField;
	private JButton mSelectFileButton;
	private File mWorkingDirectory;
	private boolean mRecording;
	private FileWriter mFileWriter;
	
	
	
	public SimulationRecorder() {
		mDisplay = new JPanel(new BorderLayout());
		mRecordButton = new JButton("Rec");
		mRecordButton.setEnabled(false);
		
		mPlaybackButton = new JButton("Play");
		mPlaybackButton.setEnabled(false);
		
		mStopButton = new JButton("Stop");
		mStopButton.setEnabled(false);
		
		mControlMainPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		mControlMainPanel.add(mPlaybackButton);
		mControlMainPanel.add(mStopButton);
		mControlMainPanel.add(mRecordButton);
		
		mFileNameTextField = new JTextField("No file selected");
		mFileNameTextField.setEditable(false);
		mSelectFileButton = new JButton("File");
		mSelectFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleFileButton();
			}
		});
	
		mFilePanel = new JPanel(new BorderLayout());
		mFilePanel.add(BorderLayout.CENTER, mFileNameTextField);
		mFilePanel.add(BorderLayout.EAST, mSelectFileButton);
		
		mDisplay.add(BorderLayout.NORTH, mControlMainPanel);
		mDisplay.add(BorderLayout.SOUTH, mFilePanel);
		mDisplay.setBorder(BorderFactory.createTitledBorder("Recorder"));
		
		mRecording = false;
	}
	
	public boolean cleanUp() {
		if(mRecording) {
			stopRecording();
		}
		return true;
	}

	public boolean init() {
		return true;
	}

	public JPanel getPanel() {
		return mDisplay;
	}
	
	public void generationCompleted(int generationId) {
		// TODO Auto-generated method stub
		
	}

	public void generationStarted(int generationId) {
		// TODO Auto-generated method stub
		
	}

	public void tryCompleted(int tryId) {
		// TODO Auto-generated method stub
		
	}

	public void tryStarted(int tryId) {
		// TODO Auto-generated method stub
		
	}

	public void stepCompleted(int stepNumber) {
		// TODO Auto-generated method stub
		
	}

	public void stepStarted(int stepNumber) {
		// TODO Auto-generated method stub
		
	}
	
	private void handleFileButton() {
		JFileChooser chooser = new JFileChooser();
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int returnVal = chooser.showOpenDialog(null);
		String fileName = "";
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	       System.out.println("You chose to open this file: " +
	            chooser.getCurrentDirectory() + File.separator + chooser.getSelectedFile().getName());
	      fileName = chooser.getCurrentDirectory() + File.separator + chooser.getSelectedFile().getName();
	    }
	    if(!fileName.equals("")) {
	    	mWorkingDirectory = new File(fileName);
	    	if(mWorkingDirectory.exists()) {
	    		mPlaybackButton.setEnabled(true);
	    		mRecordButton.setEnabled(false);
	    	}
	    	else {
	    		mPlaybackButton.setEnabled(false);
	    		mRecordButton.setEnabled(true);
	    	}
	    	mFileNameTextField.setText(mWorkingDirectory.getAbsolutePath());
	    }
	}

//	private void startRecording() {
//		
//		if(mWorkingDirectory.exists()) {
//			//if the directory already exists, then cancle recording!
//			mRecordButton.setEnabled(false);
//			return;
//		}
//		if(!mWorkingDirectory.mkdir()) {
//			mRecordButton.setEnabled(false);
//		}
//		Simulator.getInstance().getScheduler().addGenerationListener(this);
//		Simulator.getInstance().getScheduler().addTryListener(this);
//		Simulator.getInstance().getScheduler().addStepListener(this);
//		
//		mRecording = true;
//	}
	
	private void stopRecording() {
		Simulator.getInstance().getScheduler().removeGenerationListener(this);
		Simulator.getInstance().getScheduler().removeTryListener(this);
		Simulator.getInstance().getScheduler().removeStepListener(this);
		if(mFileWriter != null) {
			try {
				//TODO add finishing data.
				mFileWriter.flush();
				mFileWriter.close();
				mFileWriter = null;
			}
			catch(IOException e) {
			}
		}
		mRecording = false;
	}
	

}
