package de.fhg.iais.outdoor.simba.visualization;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.IndividualListener;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.TryListener;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class SkipVisualizationButtonPanel implements Visualization,
						TryListener, IndividualListener, GenerationListener,
						ValueChangedListener
{

	private JPanel mDisplay;
	private JToggleButton mSkipTryButton;
	private JToggleButton mSkipIndividualButton;
	private JToggleButton mSkipGenerationButton;
	private boolean mSkipTry;
	private boolean mSkipIndividual;
	private boolean mSkipGeneration;
	private BooleanValue mOptimizePerformanceValue;
	
	public SkipVisualizationButtonPanel() {
		
		mDisplay = new JPanel();
		
		mSkipTryButton = new JToggleButton("SkipTry");
		mSkipIndividualButton = new JToggleButton("SkipInd");
		mSkipGenerationButton = new JToggleButton("SkipGen");
		
		mDisplay.add(mSkipTryButton);
		mDisplay.add(mSkipIndividualButton);
		mDisplay.add(mSkipGenerationButton);
		
		mSkipIndividualButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mSkipIndividualButton.isSelected()) {
					mSkipGenerationButton.setEnabled(false);
					mSkipTryButton.setEnabled(false);
					mSkipIndividual = true;
					Simulator.getInstance().optimizePerformance(true);
				}
				else {
					mSkipGenerationButton.setEnabled(true);
					mSkipTryButton.setEnabled(true);
					mSkipIndividual = false;
					Simulator.getInstance().optimizePerformance(false);
				}
			}
		});
		
		mSkipTryButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mSkipTryButton.isSelected()) {
					mSkipGenerationButton.setEnabled(false);
					mSkipIndividualButton.setEnabled(false);
					mSkipTry = true;
					Simulator.getInstance().optimizePerformance(true);
				}
				else {
					mSkipGenerationButton.setEnabled(true);
					mSkipIndividualButton.setEnabled(true);
					mSkipTry = false;
					Simulator.getInstance().optimizePerformance(false);
				}
			}
		});
		
		mSkipGenerationButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mSkipGenerationButton.isSelected()) {
					mSkipIndividualButton.setEnabled(false);
					mSkipTryButton.setEnabled(false);
					mSkipGeneration = true;
					Simulator.getInstance().optimizePerformance(true);
				}
				else {
					mSkipIndividualButton.setEnabled(true);
					mSkipTryButton.setEnabled(true);
					mSkipGeneration = false;
					Simulator.getInstance().optimizePerformance(false);
				}
			}
		});
		
		Simulator.getInstance().getScheduler().addTryListener(this);
		Simulator.getInstance().getScheduler().addIndividualListener(this);
		Simulator.getInstance().getScheduler().addGenerationListener(this);
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		ValueManager vm = Simulator.getInstance().getValueManager();
		mOptimizePerformanceValue = vm.getBooleanValue("/Visualization/OptimizePerformance");
		if(mOptimizePerformanceValue != null) {
			mOptimizePerformanceValue.addValueChangedListener(this);
		}
		return true;
	}

	public void individualCompleted(int individualId) {
		if(mSkipIndividual) {
			mSkipIndividual = false;
			mSkipGenerationButton.setEnabled(true);
			mSkipTryButton.setEnabled(true);
			Simulator.getInstance().optimizePerformance(false);
			mSkipIndividualButton.setSelected(false);
		}
	}

	public void individualStarted(int individualId) {
	}

	public void generationCompleted(int generationId) {
		if(mSkipGeneration) {
			mSkipGeneration = false;
			mSkipIndividualButton.setEnabled(true);
			mSkipTryButton.setEnabled(true);
			Simulator.getInstance().optimizePerformance(false);
			mSkipGenerationButton.setSelected(false);
		}
	}

	public void generationStarted(int generationId) {
	}

	public void tryCompleted(int tryId) {
		if(mSkipTry) {
			mSkipTry = false;
			mSkipGenerationButton.setEnabled(true);
			mSkipIndividualButton.setEnabled(true);
			Simulator.getInstance().optimizePerformance(false);
			mSkipTryButton.setSelected(false);
		}
	}

	public void tryStarted(int tryId) {
	}

	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(value == mOptimizePerformanceValue) {
			if(!mOptimizePerformanceValue.get()) {
				mSkipGeneration = false;
				mSkipIndividual = false;
				mSkipTry = false;
				mSkipGenerationButton.setEnabled(true);
				mSkipIndividualButton.setEnabled(true);
				mSkipTryButton.setEnabled(true);
				mSkipGenerationButton.setSelected(false);
				mSkipIndividualButton.setSelected(false);
				mSkipTryButton.setSelected(false);
			}
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mOptimizePerformanceValue = (BooleanValue) ValueManager.synchronizeValue(mOptimizePerformanceValue, oldValue, newValue, this);
	}
	
	
	
	

}
