package de.fhg.iais.outdoor.simba.visualization;


import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class StaseButton implements Visualization, ValueChangedListener {

	private JPanel mDisplay;
	private JToggleButton mStaseButton;
	private BooleanValue mStaseValue;
	private BooleanValue mEvolutionRunning;
	private Color mButtonBackgroundColor;
	
	public StaseButton() {
		mDisplay = new JPanel();
		mStaseButton = new JToggleButton("Stase");
		mDisplay.add(mStaseButton);
		mButtonBackgroundColor = mDisplay.getBackground();
		
		mStaseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mStaseValue != null) {
					mStaseValue.set(mStaseButton.isSelected());
					if(mStaseButton.isSelected()) {
						mDisplay.setBackground(Color.RED);
					}
					else {
						mDisplay.setBackground(mButtonBackgroundColor);
					}
				}
			}
		});
	}
	
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {

		mStaseValue = Simulator.getInstance().getValueManager().getBooleanValue("/Evolution/Control/StaseMode");
		mEvolutionRunning = Simulator.getInstance().getValueManager().getBooleanValue("/Evolution/Status/Running");
		
		if(mStaseValue != null) {
			mStaseValue.addValueChangedListener(this);
		}
		if(mEvolutionRunning != null) {
			mEvolutionRunning.addValueChangedListener(this);
		}

		mStaseButton.setEnabled(mStaseValue != null && mEvolutionRunning != null && mEvolutionRunning.get());
		if(!mStaseButton.isEnabled()) {
			mDisplay.setBackground(mButtonBackgroundColor);
		}
		return true;
	}


	public void valueChanged(Value value) {
		if(value != null && value == mStaseValue) {
			mStaseButton.setSelected(mStaseValue.get());
			if(mStaseButton.isSelected()) {
				mDisplay.setBackground(Color.RED);
			}
			else {
				mDisplay.setBackground(mButtonBackgroundColor);
			}
		}
		mStaseButton.setEnabled(mStaseValue != null && mEvolutionRunning != null && mEvolutionRunning.get());
		if(!mStaseButton.isEnabled()) {
			mDisplay.setBackground(mButtonBackgroundColor);
		}
	}


	public void valueReplaced(Value oldValue, Value newValue) {
		mStaseValue = (BooleanValue) ValueManager.synchronizeValue(mStaseValue, oldValue, newValue, this);
		mEvolutionRunning = (BooleanValue) ValueManager.synchronizeValue(mEvolutionRunning, oldValue, newValue, this);
		
		mStaseButton.setEnabled(mStaseValue != null && mEvolutionRunning != null && mEvolutionRunning.get());
		if(!mStaseButton.isEnabled()) {
			mDisplay.setBackground(mButtonBackgroundColor);
		}
	}

	
}
