package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import javax.swing.JLabel;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.StringValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;

public class StatusMessageLabel implements Visualization, ValueChangedListener {
	
	
	private JPanel mDisplay;
	private JLabel mLabel;
	private StringValue mStatusMessages;
	private Timer mTimer;
	private int mMessageDuration;
	
	
	public StatusMessageLabel(int messageDuration) {
		mDisplay = new JPanel(new BorderLayout());
		mLabel = new JLabel(".");
		mDisplay.add(BorderLayout.CENTER, mLabel);
		mTimer = new Timer();
		mMessageDuration = messageDuration;
	}
	
	public boolean init() {
		boolean ok = true;

		acquireValueObjects();
		
		return ok;
	}
	
	public void acquireValueObjects() {
		mStatusMessages = Simulator.getInstance().getValueManager().getStringValue("/StatusMessages");
		if(mStatusMessages == null) {
			Logger.getLogger("Simulator")
				.warning("VisualizationIntervalController: Could not find StringValue:\n" +
						"[/StatusMessages]");
		}
		else {
			mStatusMessages.addValueChangedListener(this);
		}
	}
	
	public boolean cleanUp() {
		boolean ok = true;
		
		return ok;
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}
	
	public synchronized void valueChanged(Value value) {
		if(mStatusMessages != null && value == mStatusMessages) {
			mTimer.cancel();
			mTimer = new Timer();
			mLabel.setText(mStatusMessages.get());
			mTimer.schedule(new ResetTextTask(), mMessageDuration);
		}
	}
	
	/**
	 * Called if the one of the registered Value objects were replaced in the ValueManager.
	 */
	 public void valueReplaced(Value oldValue, Value newValue) {
		if(oldValue != newValue && oldValue != null) {
			oldValue.removeValueChangedListener(this);
			if(mStatusMessages  == oldValue) {
				acquireValueObjects();
			}
		}
	 }
	
	private class ResetTextTask extends TimerTask {
		
		public synchronized void run() {
			mLabel.setText(".");
		}
	
	}
}
