package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class TabbedVisualizationArea extends VisualizationArea {
	
	private JPanel mDisplay;
	private JTabbedPane mTabbedPane;
	private boolean mVisible;
	private int mLastVisibleIndex;
	
	public TabbedVisualizationArea() {
		this(JTabbedPane.TOP);
		
		mTabbedPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if(mLastVisibleIndex >= 0 && mLastVisibleIndex != mTabbedPane.getSelectedIndex()) {
					Visualization visu = (Visualization) mVisualizations.elementAt(mLastVisibleIndex);
					if(visu instanceof VisibilityListener) {
						((VisibilityListener) visu).visibilityChanged(false, this);
					}
				}
				if(mVisible) {
					Visualization visu = (Visualization) mVisualizations.elementAt(mTabbedPane.getSelectedIndex());
					if(visu instanceof VisibilityListener) {
						((VisibilityListener) visu).visibilityChanged(true, this);
					}
				}
				mLastVisibleIndex = mTabbedPane.getSelectedIndex();
			}
		});
	}
	
	public TabbedVisualizationArea(int tabPlacement) {
		mDisplay = new JPanel(new BorderLayout());
		mTabbedPane = new JTabbedPane();
		mDisplay.add(mTabbedPane, BorderLayout.CENTER);
	}

	protected boolean installVisualization(Visualization visu, String positionIdentifier) {
		mTabbedPane.add(visu.getPanel(), positionIdentifier);
		mLastVisibleIndex = mTabbedPane.getSelectedIndex();
		return true;
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public void visibilityChanged(boolean visible, Object source) {
		mVisible = visible;
		for(int i = 0; i < mVisualizations.size(); ++i) {
			Object obj = mVisualizations.elementAt(i);
			if(obj instanceof VisibilityListener) {
				if(i != mTabbedPane.getSelectedIndex()) {
					((VisibilityListener) obj).visibilityChanged(false, this);
				}
				else {
					((VisibilityListener) obj).visibilityChanged(visible, this);
				}
			}
			
		}
	}
	
	

}
