package de.fhg.iais.outdoor.simba.visualization;

import java.util.Hashtable;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class VisualizationManager {
	
	private Hashtable mVisualizationAreas = new Hashtable();
	private VisualizationArea mRootArea;
	private Vector mVisualizations = new Vector();
	
	public VisualizationManager(ValueManager valueManager) {
		mRootArea = new BorderLayoutVisualizationArea();
		mVisualizationAreas.put("root", mRootArea);
	}
	
	/**
	 * Registers a VisualizationArea with a new name. A visualizationArea can be 
	 * associated with several names.
	 * 
	 * @param name the name the VisualizationArea is associated with.
	 * @param area the area to associate with the name.
	 * @return
	 */
	public boolean addVisualizationArea(String name, VisualizationArea area) {
		if(area == null || name == null || mVisualizationAreas.containsKey(name)) {
			return false;
		}
		mVisualizationAreas.put(name, area);
		Simulator.getInstance().addSystemObject(area);
		return true;
	}
	
	/**
	 * Adds a Visualisation. Visualizations that are added to a VisualizationArea
	 * are automatically added here.
	 * 
	 * @param visu the Visualization to add.
	 * @return true if successful, otherwise false.
	 */
	public boolean addVisualization(Visualization visu) {
		if(visu == null || mVisualizations.contains(visu)) {
			return false;
		}
		mVisualizations.add(visu);
		return true;
	}
	
	/**
	 * Returns a Vector with all known Visualization objects.
	 * 
	 * @return a Vector with all Visualizations.
	 */
	public Vector getVisualizations() {
		return new Vector(mVisualizations);
	}
	
	/**
	 * Returns the VisualiationArea object that is associated with the given name.
	 * If no such object exists, then null is returned. 
	 * @param name the associated name of the desired object.
	 * @return the desired VisualizationArea object if found, otherwise null.
	 */
	public VisualizationArea getVisualizationArea(String name) {
		Object area = mVisualizationAreas.get(name);
		if(area != null) {
			return (VisualizationArea) area;
		}
		return null;
	}
	
	/**
	 * Returns the default root area. 
	 * @return the root area.
	 */
	public VisualizationArea getRootArea() {
		return mRootArea;
	}
	


}
