package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JScrollPane;


import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanel;

public class EvolutionControlPanel implements Visualization {

	private JPanel mDisplay;
	
	private ValuePanel mStepPanel;
	private ValuePanel mTriesPanel;
	private ValuePanel mPreserveParentPanel;
	private ValuePanel mPopulationSizePanel;
	private ValuePanel mNetIterationsPerStep;
	private ChooseLogFileNameVisualization mLogFileNamePanel;
//	private ValuePanel mCurrentSizePanel;
	private Box mContent;
	private JScrollPane mScrollPanel;
	private Vector mValuePanels = new Vector();
	private RestartGenerationButton mRestartGenerationButton;
	
	
	public EvolutionControlPanel() {
		mDisplay = new JPanel(new BorderLayout());
		
		//Prepare main panel
		JPanel controlPanel = new JPanel(new BorderLayout());
		
		controlPanel.setBorder(BorderFactory.createTitledBorder("Control Parameters"));
		
		mContent = Box.createVerticalBox();
		
		JPanel mainPanel = new JPanel(new BorderLayout());
		mScrollPanel = new JScrollPane(mainPanel);
		mainPanel.add(mContent, BorderLayout.NORTH);
		mDisplay.add(mScrollPanel, BorderLayout.CENTER);
		mDisplay.add(mainPanel, BorderLayout.CENTER);
		
		
		//Control parameter panel
		JPanel controlParameterPanel = new JPanel(new GridLayout(0, 3));
		controlParameterPanel.setBorder(BorderFactory.createTitledBorder("Control Parameters"));
		controlPanel.add(controlParameterPanel, BorderLayout.CENTER);
		
		mStepPanel = new ValuePanel(null, "/Simulation/NumberOfSteps", "Steps", 6, false, true);
		mStepPanel.setBorder(BorderFactory.createEtchedBorder());
		controlParameterPanel.add(mStepPanel);
		mValuePanels.add(mStepPanel);
		
		
		mTriesPanel = new ValuePanel(null, "/Simulation/NumberOfTries", "Tries", 6, false, true);
		mTriesPanel.setBorder(BorderFactory.createEtchedBorder());
		controlParameterPanel.add(mTriesPanel);
		mValuePanels.add(mTriesPanel);
		
		mPreserveParentPanel = new ValuePanel(null, "/Evolution/Control/PreserveParents", "Preserve Parents", 4, false, true);
		mPreserveParentPanel.setBorder(BorderFactory.createEtchedBorder());
		controlParameterPanel.add(mPreserveParentPanel);
		mValuePanels.add(mPreserveParentPanel);
		
		mPopulationSizePanel = new ValuePanel(null, "/Evolution/Control/PopulationSize", "Population Size", 6, false, true);
		mPopulationSizePanel.setBorder(BorderFactory.createEtchedBorder());
		mValuePanels.add(mPopulationSizePanel);
		controlParameterPanel.add(mPopulationSizePanel);
		
		mNetIterationsPerStep = new ValuePanel(null, "/NeuralNetManager/Control/IterationsPerStep", "NetIterations", 4, false, true);
		mNetIterationsPerStep.setBorder(BorderFactory.createEtchedBorder());
		mValuePanels.add(mNetIterationsPerStep);
		controlParameterPanel.add(mNetIterationsPerStep);
		
		mContent.add(controlParameterPanel);
		
		mLogFileNamePanel = new ChooseLogFileNameVisualization();
		mContent.add(mLogFileNamePanel.getPanel());
		mLogFileNamePanel.getPanel().setBorder(BorderFactory.createEmptyBorder());
		//do not add to valuePanels. This prevents the logFileName from being saved.
		
		JPanel generationButtonPanel = new JPanel();
		mRestartGenerationButton = new RestartGenerationButton();
		generationButtonPanel.add(mRestartGenerationButton.getPanel());
		
		SaveAndLoadGenerationButton saveAndLoadGenButton = new SaveAndLoadGenerationButton();
		generationButtonPanel.add(saveAndLoadGenButton.getPanel());
		
		EvolutionEvaluateOnlyButton evaluateOnlyButton = new EvolutionEvaluateOnlyButton();
		generationButtonPanel.add(evaluateOnlyButton.getPanel());
		
		mContent.add(generationButtonPanel);
		
	}

	public JPanel getPanel() {
		return mDisplay;
	}


	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		//Connect Value objects with their corresponding ValuePanels.
		mStepPanel.setValue(Simulator.getInstance().getScheduler().getNumberOfStepsValue());
		mTriesPanel.setValue(Simulator.getInstance().getScheduler().getNumberOfTriesValue());
		mPreserveParentPanel.setValue(Simulator.getInstance().getEvolutionManager().getPreserveParentsValue());
		mPopulationSizePanel.setValue(Simulator.getInstance().getEvolutionManager().getMeanPopulationSizeValue());
		mNetIterationsPerStep.setValue(Simulator.getInstance().getNeuralNetManager().getIterationsPerStepValue());
		return true;
	}
	
	public Vector getValuePanels() {
		return new Vector(mValuePanels);
	}
	
	
	
	
}
