package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class EvolutionEvaluateOnlyButton implements Visualization, ValueChangedListener {

	private JPanel mDisplay;
	private JToggleButton mEvaluateOnlyButton;
	private BooleanValue mEvaluateOnlyValue;
	private Color mButtonBackgroudColor;
	
	public EvolutionEvaluateOnlyButton() {
		mDisplay = new JPanel();
		mEvaluateOnlyButton = new JToggleButton("EvaluateOnly");
		mButtonBackgroudColor = mDisplay.getBackground();
		
		mDisplay.add(mEvaluateOnlyButton);
		
		mEvaluateOnlyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mEvaluateOnlyValue != null) {
					mEvaluateOnlyValue.set(mEvaluateOnlyButton.isSelected());
					if(mEvaluateOnlyButton.isSelected()) {
						mDisplay.setBackground(Color.RED);
					}
					else {
						mDisplay.setBackground(mButtonBackgroudColor);
					}
				}
			}
		});
		
		Simulator.getInstance().addSystemObject(this);
	}
	
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {

		mEvaluateOnlyValue = Simulator.getInstance().getEvolutionManager().getEvaluateOnlyModeValue();
		
		if(mEvaluateOnlyValue != null) {
			mEvaluateOnlyValue.addValueChangedListener(this);
		}

		mEvaluateOnlyButton.setEnabled(mEvaluateOnlyValue != null);
		return true;
	}


	public void valueChanged(Value value) {
		if(value != null && value == mEvaluateOnlyValue) {
			mEvaluateOnlyButton.setSelected(mEvaluateOnlyValue.get());
			if(mEvaluateOnlyButton.isSelected()) {
				mDisplay.setBackground(Color.RED);
			}
			else {
				mDisplay.setBackground(mButtonBackgroudColor);
			}
		};
	}


	public void valueReplaced(Value oldValue, Value newValue) {
		mEvaluateOnlyValue = (BooleanValue) ValueManager.synchronizeValue(mEvaluateOnlyValue, oldValue, newValue, this);
	}

}
