package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import de.fhg.iais.outdoor.simba.communication.NetFileFilter;
import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;
import de.fhg.iais.outdoor.simba.value.ObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.BoxLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.FrameVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.StaseButton;
import de.fhg.iais.outdoor.simba.visualization.TabbedVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.statistics.StatisticsHistoryPlotPanel;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanel;

public class EvolutionParameterPanel extends FrameVisualizationArea {
	
	private TabbedVisualizationArea mTabbedArea;
	private JButton mSaveButton;
	private JButton mLoadButton;
	private JButton mApplyButton;
	private JButton mUpdateButton;
	
	private JPanel mLowerPanel;
	private Vector mValuePanels = new Vector();
	private JFileChooser mFileChooser;
	
	public EvolutionParameterPanel(int width, int height, int x, int y) {
		super("Evolution", width, height, x, y);
		
		mFileChooser = new JFileChooser();
		mFileChooser.setFileFilter(new NetFileFilter("Value Files", "val"));
		
		mTabbedArea = new TabbedVisualizationArea();
		addVisualization(mTabbedArea, BorderLayout.CENTER);
		
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		
		Vector ignoreList = new Vector();
		ignoreList.add("Active");
		ignoreList.add("Layer");
		
		//General control
		EvolutionControlPanel controlPanel = new EvolutionControlPanel();
		mTabbedArea.addVisualization(controlPanel, "Control");
		mValuePanels.addAll(controlPanel.getValuePanels());
		
		//EvolutionOperators
		Vector operators = em.getEvolutionOperators();
		ParameterPanel panel = new ParameterPanel(operators, ignoreList);
		mTabbedArea.addVisualization(panel, "Operators");
		mValuePanels.addAll(panel.getValuePanels());
		
		
		//Fitness
		Vector fitnessFunctions = Simulator.getInstance().getFitnessManager().getFitnessFunctions();
		ParameterPanel fitnessPanel = new ParameterPanel(fitnessFunctions, ignoreList);
		mTabbedArea.addVisualization(fitnessPanel, "Fitness");
		mValuePanels.addAll(fitnessPanel.getValuePanels());
		
		//Selection
		Vector selectionMethods = Simulator.getInstance().getEvolutionManager().getSelectionMethods();
		ParameterPanel selectionPanel = new ParameterPanel(selectionMethods, ignoreList);
		mTabbedArea.addVisualization(selectionPanel, "Selection");
		mValuePanels.addAll(selectionPanel.getValuePanels());
		
		//Environment
		Vector environments = new Vector();
		environments.add(Simulator.getInstance().getScheduler().getMasterEnvironment());
		ParameterPanel environmentPanel = new ParameterPanel(environments, ignoreList);
		mTabbedArea.addVisualization(environmentPanel, "Environment");
		mValuePanels.addAll(environmentPanel.getValuePanels());
		
		//Misc adjustments
		Vector miscValues = Simulator.getInstance().getValueManager().getValuesStartingWith("/Adjustments/Misc");
		Vector miscParameterElements = new Vector();
		for(Iterator i = miscValues.iterator(); i.hasNext();) {
			Object o = i.next();
			if(o instanceof ObjectValue) {
				ObjectValue v = (ObjectValue) o;
				if(v != null && v.get() != null && v.get() instanceof ParameterElement) {
					miscParameterElements.add((ParameterElement) v.get());
				}
			}
		}
		if(!miscParameterElements.isEmpty()) {
			ParameterPanel miscPanel = new ParameterPanel(miscParameterElements, ignoreList);
			mTabbedArea.addVisualization(miscPanel, "Misc");
			mValuePanels.addAll(miscPanel.getValuePanels());
		}
		
		//History Plotter
		BoxLayoutVisualizationArea plotterArea = new BoxLayoutVisualizationArea();
		mTabbedArea.addVisualization(plotterArea, "Statistic Plots");
		for(int i = 0; i < 3; ++i) {
			StatisticsHistoryPlotPanel plotPanel = new StatisticsHistoryPlotPanel();
			plotterArea.addVisualization(plotPanel, "");
		}
		
		
		BoxLayoutVisualizationArea buttonArea = new BoxLayoutVisualizationArea(SwingConstants.HORIZONTAL);
		addVisualization(buttonArea, BorderLayout.SOUTH);
		
		
		//Start Evolution Button.
		RunEvolutionButton runEvolutionButton = new RunEvolutionButton();
		buttonArea.addVisualization(runEvolutionButton, "");
		
		//StaseButton
		StaseButton staseButton = new StaseButton();
		buttonArea.addVisualization(staseButton, "");
		
		//Moveable Plotter
		FrameVisualizationArea framePlotArea = new FrameVisualizationArea("Plotter", 700, 250, 200, 400);
		StatisticsHistoryPlotPanel plotPanel = new StatisticsHistoryPlotPanel();
		framePlotArea.addVisualization(plotPanel, BorderLayout.CENTER);
		buttonArea.addVisualization(framePlotArea, "");
		
		//Buttons
		mSaveButton = new JButton("Save");
		mLoadButton = new JButton("Load");
		mApplyButton = new JButton("Apply");
		mUpdateButton = new JButton("Update");
		
		mSaveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveParameters();
			}
		});
		
		mLoadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadParameters();
			}
		});
		
		mApplyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				applyAllValueChanges();
			}
		});
		
		mUpdateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateAllValues();
			}
		});
		
		mLowerPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		mLowerPanel.add(mSaveButton);
		mLowerPanel.add(mLoadButton);
		mLowerPanel.add(mUpdateButton);
		mLowerPanel.add(mApplyButton);
		
		buttonArea.addVisualization(new Visualization() {

			public JPanel getPanel() {
				return mLowerPanel;
			}

			public boolean cleanUp() {
				return true;
			}

			public boolean init() {
				return true;
			}
		}, BorderLayout.SOUTH);
	}
	
	public void saveParameters() {
		//Choose directory.
		Properties props = Simulator.getInstance().getSystemProperties();
		String directory = props.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Save");
		
		//Save parameters
		if(mFileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
			File file = mFileChooser.getSelectedFile();
			if(!file.getName().endsWith(".val")) {
				file = new File(file.getAbsolutePath() + ".val");
			}
			
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			props.put("/FileChooser/CurrentDirectory", directory);
			
			if(file.exists()) {
				int res = JOptionPane.showConfirmDialog(null, "A file with name " + file.getName() + " already exists.\nDo you want to overwrite it?", 
								"File exists warning", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
				if(res == JOptionPane.CANCEL_OPTION) {
					return;
				}
			}
			savePropertiesInFile(file, true);
		}
		else {
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			props.put("/FileChooser/CurrentDirectory", directory);
		}
	}
	
	private void savePropertiesInFile(File file, boolean warnAtFailure) {
		Properties props = new Properties();
		for(Iterator i = mValuePanels.iterator(); i.hasNext();) {
			ValuePanel panel = (ValuePanel) i.next();
			if(panel == null || panel.getValue() == null) {
				continue;
			}
			props.setProperty(panel.getValueName(), panel.getValue().getValueAsString());
		}
		
		try {
			FileOutputStream output = new FileOutputStream(file);
			props.store(output, "Saved Values for the Simba Simulator.");
			output.close();
			Simulator.getInstance().reportStatus("Evolution parameters saved.");
		} catch (FileNotFoundException e) {
			if(warnAtFailure) {
				JOptionPane.showMessageDialog(null, "Could not save the Values.", 
						"Save error", JOptionPane.WARNING_MESSAGE);
			}
	
		} catch (IOException e) {
			if(warnAtFailure) {
				JOptionPane.showMessageDialog(null, "Could not save the Values.", 
						"Save error", JOptionPane.WARNING_MESSAGE);
			}
		}
	}
	
	public void loadParameters() {
		//choose directory.
		Properties props = Simulator.getInstance().getSystemProperties();
		String directory = props.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Load");
		
		//load parameters.
		if(mFileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
			File file = mFileChooser.getSelectedFile();
			if(!file.getName().endsWith(".val")) {
				file = new File(file.getAbsolutePath() + ".val");
			}
			
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			props.put("/FileChooser/CurrentDirectory", directory);
			
			String errorMessage = loadParametersFromFile(file, true); 
			if(errorMessage.length() > 0) {
				JOptionPane.showMessageDialog(null, "Could not find the following Values:\n\n" + errorMessage, 
						"Warning", JOptionPane.WARNING_MESSAGE);
				Simulator.getInstance().reportStatus("Evolution parameters loaded with warnings.");
			}
			else {
				Simulator.getInstance().reportStatus("Evolution parameters loaded.");
			}
		}
		else {
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			props.put("/FileChooser/CurrentDirectory", directory);
		}
	}
	
	private String loadParametersFromFile(File file, boolean warnAtFailure) {
		Properties props = new Properties();
		try {
			FileInputStream inputStream = new FileInputStream(file);
			props.load(inputStream);
		} catch (FileNotFoundException e1) {
			if(warnAtFailure) {
				JOptionPane.showMessageDialog(null, "Could not find the specified file" + file.getName(), 
						"File does not exist", JOptionPane.WARNING_MESSAGE);
			}
			return "";
		} catch (IOException e1) {
			if(warnAtFailure) {
				JOptionPane.showMessageDialog(null, "Could not load the Values.", 
						"Load error", JOptionPane.WARNING_MESSAGE);
			}
			return "";
		}
		
		String errorMessage = "";
		ValueManager vm = Simulator.getInstance().getValueManager();
		Enumeration keys = props.keys();
		for(; keys.hasMoreElements();) {
			String valueName = (String) keys.nextElement();
			
			Value v = vm.getValue(valueName);
			if(v == null) {
				errorMessage += "- " + valueName + "\n";
				continue;
			}
			v.setValueFromString(props.getProperty(valueName));
		}
		return errorMessage;
	}
	
	public void applyAllValueChanges() {
		for(Iterator i = mValuePanels.iterator(); i.hasNext();) {
			ValuePanel vp = (ValuePanel) i.next();
			if(vp.checkContentFieldChanged()) {
				vp.setValueFromContentField();
			}
		}
	}
	
	public void updateAllValues() {
		for(Iterator i = mValuePanels.iterator(); i.hasNext();) {
			ValuePanel vp = (ValuePanel) i.next();
			if(vp.checkContentFieldChanged()) {
				vp.setContentFieldFromValue();
			}
		}
	}

	public boolean cleanUp() {
		boolean ok = super.cleanUp();
		
		File file = new File(Simulator.getInstance().getConfigDirectoryName() + "evolution.conf");
		savePropertiesInFile(file, false);
		
		return ok;
	}

	public boolean init() {
		boolean ok = super.init();
		
		File file = new File(Simulator.getInstance().getConfigDirectoryName() + "evolution.conf");
		loadParametersFromFile(file, false);
		
		return ok;
	}
	
	

}
