package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionIndividual;
import de.fhg.iais.outdoor.simba.evolution.EvolutionListener;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.TableSorter;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class GenerationSummaryPanel implements Visualization,
		GenerationListener, EvolutionListener, ValueChangedListener {

	
	private JPanel mDisplay;
	private JButton mRestartGenerationButton;
	private JButton mUpdateMarkedButton;
	private JButton mMarkAllButton;
	private JToggleButton mHoldResumeButton;
	private JTable mIndividualTable;
	private IndividualSummaryTableModel mTableModel;
	private BooleanValue mEvaluateOnlyValue;
	private BooleanValue mEvolutionRunningValue;
	private NetworkRunner mRunner;
	private TableSorter mTableSorter;
	
	public GenerationSummaryPanel() {
		mDisplay = new JPanel(new BorderLayout());
		
		mRestartGenerationButton = new JButton("Restart All");
		mRestartGenerationButton.setEnabled(false);
		
		
		mUpdateMarkedButton = new JButton("Update Table");
		mUpdateMarkedButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mTableModel.updateContent();
			}
		});
		
		mMarkAllButton = new JButton("Mark All");
		mMarkAllButton.setEnabled(false);
		
		JPanel holdResumeButtonPanel = new JPanel();
		mHoldResumeButton = new JToggleButton("Hold");
		holdResumeButtonPanel.add(mHoldResumeButton);
		
		mHoldResumeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!mHoldResumeButton.isSelected() && mRunner != null && mRunner.isAlive()) {
		            if(mRunner != null) {
		            	mRunner.terminateRunner();
		            	try {
							mRunner.join();
						} catch (InterruptedException e1) {
						}
		            }
		            mRunner = null;
				}
				Simulator.getInstance().getEvolutionManager().getRunEvolutionValue().set(!mHoldResumeButton.isSelected());
				if(!mHoldResumeButton.isSelected()) {
					mIndividualTable.setEnabled(false);
				}
				else {
					mIndividualTable.setEnabled(!Simulator.getInstance().getEvolutionManager().getEvolutionRunningValue().get());
				}
			}
		});
		  
		mTableModel = new IndividualSummaryTableModel();
		mTableSorter = new TableSorter(mTableModel); 
		mIndividualTable = new JTable(mTableSorter);
		mTableSorter.setTableHeader(mIndividualTable.getTableHeader());
		mTableSorter.setColumnComparator(Double.class, TableSorter.COMPARABLE_COMAPRATOR);
		mTableSorter.setColumnComparator(Integer.class, TableSorter.COMPARABLE_COMAPRATOR);
		
		
		JScrollPane scrollPane = new JScrollPane(mIndividualTable);
		mIndividualTable.setPreferredScrollableViewportSize(new Dimension(200, 100));
		
		mIndividualTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ListSelectionModel rowSM = mIndividualTable.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
		        //Ignore extra messages.
		        if(e.getValueIsAdjusting()) {
		        	return;
		        }
		        
		        //so not change the individual during pause mode!
		        //refactor: this is not thread safe!
		        if(Simulator.getInstance().isPaused()) {
		        	return;
		        }

		        ListSelectionModel lsm = (ListSelectionModel) e.getSource();
		        if (lsm.isSelectionEmpty()) {
		        } 
		        else {
		            int selectedRow = lsm.getMinSelectionIndex();
		            //System.out.println("Selected line " + selectedRow);
		            
		            //get the row in the unsorted model
		            selectedRow = mTableSorter.modelIndex(selectedRow);
		            
		            EvolutionIndividual ind = mTableModel.getIndividual(selectedRow);
		            if(ind == null) {
		            	return;
		            }
		            NeuralNet net = (NeuralNet) ind.getGenom();
		            if(net == null) {
		            	return;
		            }

		            Scheduler scheduler = Simulator.getInstance().getScheduler();
		            if(mRunner != null && mRunner.isAlive()) {
		            	mRunner.terminateRunner();
		            	try {
							mRunner.join();
						} catch (InterruptedException e1) {
							System.out.println("Join was interrupted.");
						}
		            }
		            mRunner = new NetworkRunner(net, selectedRow, scheduler.getNumberOfTries(), scheduler.getNumberOfSteps());
		            
		            mRunner.start();
		        }
		    }
		});
		
		mIndividualTable.setEnabled(false);
		
		
		JPanel buttonPanel = new JPanel();
		//buttonPanel.add(mMarkAllButton);
		//buttonPanel.add(mRestartGenerationButton);
		buttonPanel.add(mUpdateMarkedButton);
		
		JPanel lowerPanel = new JPanel(new BorderLayout());
		lowerPanel.add(BorderLayout.WEST, buttonPanel);
		lowerPanel.add(BorderLayout.EAST, holdResumeButtonPanel);
		
		mDisplay.add(BorderLayout.CENTER, scrollPane);
		mDisplay.add(BorderLayout.SOUTH, lowerPanel);
		
		Simulator.getInstance().getEvolutionManager().addEvolutionListener(this);
		Simulator.getInstance().getScheduler().addGenerationListener(this);
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {

		mEvaluateOnlyValue = Simulator.getInstance().getEvolutionManager().getEvaluateOnlyModeValue();
		mEvolutionRunningValue = Simulator.getInstance().getEvolutionManager().getEvolutionRunningValue();
		
		if(mEvaluateOnlyValue != null) {
			mEvaluateOnlyValue.addValueChangedListener(this);
		}
		if(mEvolutionRunningValue != null) {
			mEvolutionRunningValue.addValueChangedListener(this);
		}
		
		return true;
	}

	public void generationCompleted(int generationId) {
	}

	public void generationStarted(int generationId) {
	}

	public void notifyIndividualsCreated(Vector individuals, int generation) {
	}

	public void notifyIndividualsEvaluated(Vector individuals, int generation) {
		mTableModel.updateContent();
		if(mHoldResumeButton.isSelected()) {
			//mIndividualTable.setEnabled(true);
			Simulator.getInstance().getEvolutionManager().getRunEvolutionValue().set(false);
		}
	}

	public void notifyParentsSelected(Vector parents, int generation) {
	}

	
	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(mEvaluateOnlyValue == value) {
			if(!mEvaluateOnlyValue.get()) {
				mHoldResumeButton.setSelected(false);
			}
		}
		else if(mEvolutionRunningValue == value) {
			if(!mEvolutionRunningValue.get() && mHoldResumeButton.isSelected()) {
				mIndividualTable.setEnabled(true);
			}
			else {
				if(mRunner != null && mRunner.isAlive()) {
					mRunner.terminateRunner();
					try {
						mRunner.join();
					} catch (InterruptedException e) {
					}
					mRunner = null;
				}
				mIndividualTable.setEnabled(false);
			}
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mEvaluateOnlyValue = (BooleanValue) ValueManager.synchronizeValue(mEvaluateOnlyValue, oldValue, newValue, this);
		mEvolutionRunningValue = (BooleanValue) ValueManager.synchronizeValue(mEvolutionRunningValue, oldValue, newValue, this);
	}
	
	
	
	

}
