package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionIndividual;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;

public class IndividualSummaryTableModel extends AbstractTableModel {

	public static final long serialVersionUID = 12345L;
	private String mColumNames[] = {"#", "ID", "Fitness", "#Neurons", "#Synapses", "age"};
	private Vector mGeneration = new Vector();
	
	
	public IndividualSummaryTableModel() {
		
	}
	
	public int getColumnCount() {
		return mColumNames.length;
	}

	public int getRowCount() {
		return mGeneration.size();
	}

	public synchronized Object getValueAt(int rowIndex, int columnIndex) {
		if(rowIndex >= 0 && rowIndex < mGeneration.size()) {
			EvolutionIndividual ind = (EvolutionIndividual) mGeneration.elementAt(rowIndex);
			if(columnIndex == 0) {
				return new Integer(rowIndex);
			}
			else if(columnIndex == 1) {
				return new Integer(ind.getId());
			}
			else if(columnIndex == 2) {
				Vector fitnessValues = ind.getFitnessValues();
				if(fitnessValues.isEmpty()) {
					return new Double(-1.0);
				}
				else {
					return new Double(((Double) fitnessValues.elementAt(0)).doubleValue());
				}
			}
			else if(columnIndex == 3) {
				NeuralNet net = (NeuralNet) ind.getGenom();
				if(net != null) {
					return new Integer(net.getNeurons().size());
				}
			}
			else if(columnIndex == 4) {
				NeuralNet net = (NeuralNet) ind.getGenom();
				if(net != null) {
					return new Integer(net.getSynapses().size());
				}
			}
			else if(columnIndex == 5) {
				return new Integer(ind.getAge());
			}
			
		}
		return null;
	}

	public String getColumnName(int columnIndex) {
		if(columnIndex >= 0 && columnIndex < mColumNames.length) {
			return mColumNames[columnIndex];
		}
		return "";
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
	
	public synchronized void updateContent() {
		mGeneration = Simulator.getInstance().getEvolutionManager().getGeneration();
		fireTableStructureChanged();
	}
	
	public EvolutionIndividual getIndividual(int row) {
		if(row >= 0 && row < mGeneration.size()) {
			return (EvolutionIndividual) mGeneration.elementAt(row);
		}
		return null;
	}

	public Class getColumnClass(int columnIndex) {
		switch(columnIndex) {
		case 0:
		case 1:
		case 3:
		case 4:
		case 5:
			return Integer.class;
		case 2:
			return Double.class;
		default:
			return super.getColumnClass(columnIndex);
		}
	}

	
	
}
