package de.fhg.iais.outdoor.simba.visualization.evolution;

import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;

public class NetworkRunner extends Thread {
	
	private NeuralNet mNet;
	private int mNumberOfSteps;
	private int mNumberOfTries;
	private int mId;
	private boolean mTerminate;
	
	public NetworkRunner(NeuralNet net, int id, int numberOfTries, int numberOfSteps) {
		mNet = net;
		mNumberOfSteps = numberOfSteps;
		mNumberOfTries = numberOfTries;
		mId = id;
		mTerminate = false;
	}

	public void run() {
		if(mNet == null) {
			return;
		}
		
		Simulator.getInstance().getNeuralNetManager().setCurrentNet(mNet);
		Scheduler scheduler = Simulator.getInstance().getScheduler();
		
		if(mTerminate) {
			return;
		}
		
		scheduler.prepareNextIndividual(mId);
		for(int tries = 0; tries < mNumberOfTries; ++tries) {
			scheduler.prepareNextTry(tries);
			for(int steps = 0; steps < mNumberOfSteps; ++steps) {
				scheduler.executeSimulationStep(steps);
				if(mTerminate) {
					return;
				}
				if(scheduler.shouldCurrentTryBeTerminated()) {
					break;
				}
			}
		}
	}

	public void terminateRunner() {
		mTerminate = true;
	}
}
