package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanel;

public class ParameterElementPanel extends JPanel implements ValueChangedListener {

	public static final long serialVersionUID = 12345L;
	
	private ParameterElement mParameterElement;
	private JCheckBox mEnableCheckBox;
	private BooleanValue mEnabledValue;
	private JPanel mParameterPanel;
	private Vector mValuePanels = new Vector();
	
	public ParameterElementPanel(ParameterElement element) {
		this(element, null);
	}
	
	public ParameterElementPanel(ParameterElement element, Vector ignoreList) {
		if(element == null) {
			throw new IllegalArgumentException("The ParameterElement must not be null!");
		}
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createRaisedBevelBorder());
		
		JPanel borderPanel = new JPanel(new BorderLayout());
		add(borderPanel, BorderLayout.CENTER);
		
		mParameterElement = element;
		borderPanel.setBorder(BorderFactory.createTitledBorder(mParameterElement.getName()));
		
		mEnabledValue = element.getActiveValue();
		mEnableCheckBox = new JCheckBox();
		mEnableCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mEnabledValue.set(mEnableCheckBox.isSelected());
			}
		});
		mEnableCheckBox.setSelected(mEnabledValue.get());
		mEnabledValue.addValueChangedListener(this);
		borderPanel.add(mEnableCheckBox, BorderLayout.WEST);
		
		mParameterPanel = new JPanel(new GridLayout(0, 3));
		borderPanel.add(mParameterPanel, BorderLayout.CENTER);
		
		Vector parameters = element.getParameters();
		for(Iterator i = parameters.iterator(); i.hasNext();) {
			Value param = (Value) i.next();
			ValuePanel panel = new ValuePanel(param, element.getPrefix() + element.getParameterName(param), element.getParameterName(param), 4, false, true);
			mValuePanels.add(panel);
			
			if(ignoreList != null) {
				boolean ignore = false;
				String parameterName = element.getParameterName(param);
				for(Iterator j = ignoreList.iterator(); j.hasNext();) {
					if(parameterName.equals((String) j.next())) {
						ignore = true;
						break;
					}
				}
				if(ignore) {
					continue;
				}
			}
			panel.setBorder(BorderFactory.createEtchedBorder());
			mParameterPanel.add(panel);
			
		}
		
	}
	
	public void synchronizeActivity() {
		if(mEnabledValue.get()) {
			mEnableCheckBox.setSelected(true);
			for(Iterator i = mValuePanels.iterator(); i.hasNext();) {
				((ValuePanel) i.next()).setEnabled(true);
			}
		}
		else {
			mEnableCheckBox.setSelected(false);
			mParameterPanel.setEnabled(false);
			for(Iterator i = mValuePanels.iterator(); i.hasNext();) {
				((ValuePanel) i.next()).setEnabled(false);
			}
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mEnabledValue = (BooleanValue) ValueManager.synchronizeValue(mEnabledValue, oldValue, newValue, this);
	}

	public void valueChanged(Value value) {
		if(value != null) {
			if(value == mEnabledValue) {
				synchronizeActivity();
			}
		}
	}
	
	public Vector getValuePanels() {
		return new Vector(mValuePanels);
	}
	
	

}
