package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.communication.NetFileFilter;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.GenerationFileHandler;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class SaveAndLoadGenerationButton implements Visualization {

	private JPanel mDisplay;
	private JButton mSaveButton;
	private JButton mLoadButton;
	private JFileChooser mFileChooser;
	
	
	public SaveAndLoadGenerationButton() {
		mDisplay = new JPanel();
		mSaveButton = new JButton("Save Generation");
		
		mSaveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveGeneration();
			}
		});
		
		mLoadButton = new JButton("Load Generation");
		mLoadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadGeneration();
			}
		});
		
		mFileChooser = new JFileChooser();
		mFileChooser.setFileFilter(new NetFileFilter("Generations", "gen"));
		
		
		mDisplay.add(mSaveButton);
		mDisplay.add(mLoadButton);
		
	}
	
	public void saveGeneration() {
		//choose directory.
		Properties sysProps = Simulator.getInstance().getSystemProperties();
		String directory = sysProps.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Save");
		
		
		int returnVal = mFileChooser.showOpenDialog(null);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	       
	    	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
			
	    	File file = mFileChooser.getSelectedFile();
	    	
			if(!file.getName().endsWith(".gen")) {
				file = new File(file.getAbsolutePath() + ".gen");
			}
	        
	        if(!file.isDirectory()) {
	        	if(file.exists()) {
	        		int result = JOptionPane.showConfirmDialog(null, 
	        				"The file with name " + file.getName() + " already exists.\n" +
	        						"Do you really want to overwrite this file?");
	        		if(result != JOptionPane.YES_OPTION) {
	        			return;
	        		}
	        	}
	        	GenerationFileHandler fileHandler = new GenerationFileHandler();
	        	if(fileHandler.saveGeneration(file.getAbsolutePath())) {
	        		Simulator.getInstance().reportStatus("Generation saved.");
	        	}
	        	else {
	        		Simulator.getInstance().reportStatus("Could not save generation.");
	        	}
	        }
	    }
        else {
        	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
        }
	}
	
	public void loadGeneration() {
		//choose directory.
		Properties sysProps = Simulator.getInstance().getSystemProperties();
		String directory = sysProps.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Load");

		int returnVal = mFileChooser.showOpenDialog(null);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
			
			File file = mFileChooser.getSelectedFile();

			if(!file.isDirectory()) {
				if(file.exists()) {
					GenerationFileHandler fileHandler = new GenerationFileHandler();
		        	if(fileHandler.loadGeneration(file.getAbsolutePath())) {
		        		Simulator.getInstance().reportStatus("Generation loaded.");
		        		Simulator.getInstance().getEvolutionManager().getEvaluateOnlyModeValue().set(true);
		        		return;
		        	}
				}
			}
		}
		else {
        	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
        }
		Simulator.getInstance().reportStatus("Could not load generation.");;
		
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}

}
