package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunction;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class NetworkTableTexConverter implements Visualization {

	private JPanel mDisplay;
	private JButton mGenerateButton;
	private JTextArea mCodeArea;
	private JTextField mRowSize;
	private JTextField mFirstRowSize;
	private JTextField mNetworkName;
	
	public NetworkTableTexConverter() {
		mDisplay = new JPanel(new BorderLayout());
		
		mGenerateButton = new JButton("Generate");
		mCodeArea = new JTextArea();
		mCodeArea.setEditable(false);
		
		mRowSize = new JTextField("9");
		mFirstRowSize = new JTextField("7");
		mNetworkName = new JTextField("Network");
		
		JScrollPane scrollPane = new JScrollPane(mCodeArea);
		
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(BorderLayout.NORTH, mNetworkName);
		JPanel panel2 = new JPanel(new BorderLayout());
		panel.add(BorderLayout.SOUTH, mRowSize);
		panel2.add(BorderLayout.NORTH, panel);
		panel2.add(BorderLayout.SOUTH, mFirstRowSize);
		
		mDisplay.add(BorderLayout.CENTER, scrollPane);
		mDisplay.add(BorderLayout.SOUTH, mGenerateButton);
		mDisplay.add(BorderLayout.NORTH, panel2);
		
		
		
		mGenerateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int size = 8;
				int firstSize = 6;
				try {
					size = Integer.parseInt(mRowSize.getText());
					firstSize = Integer.parseInt(mFirstRowSize.getText());
				} catch (NumberFormatException e1) {
				}
				generateLatexCodeForNetwork(size, firstSize);
			}
			
		});
	}
	
	public void generateLatexCodeForNetwork(int rowSize, int firstSize) {
		NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
		
		synchronized(net) {
			
			int maxNeuronsPerTableRest = rowSize;
			
			int currentStartNeuron = 0;
			int maxNeuronsFirstPart = firstSize;
			int maxNeuronsPerTable = maxNeuronsFirstPart;
			int partNumber = 1;
		
			if(net == null) {
				mCodeArea.setText("");
				return;
			}
			
			Vector neurons = net.getNeurons();
			
			String code = "";
			
			while(currentStartNeuron < neurons.size()) {
				
				boolean firstNeuron = true;
				code += "\\begin{table}[htbp]\n";
				code += "\t\\begin{tabular} {";
				for(int i = currentStartNeuron; (i <= neurons.size()) && (i < (currentStartNeuron + 1 + maxNeuronsPerTable)); ++i) {
					if(!firstNeuron) {
						code +="|";
					}
					else {
						firstNeuron = false;
					}
					code +="l";
				;}
				code +="}\n";
				code += "\t\t";
				
				for(int i = currentStartNeuron; (i < neurons.size()) && (i < (currentStartNeuron + maxNeuronsPerTable)); ++i) {
					code += "& N" + i + " ";
				}
				code +="\\\\\n";
	
				code += "\t\t\\hline\n";
				
				code += "\t\tBias ";
				for(int j = currentStartNeuron; (j < neurons.size()) && (j < (currentStartNeuron + maxNeuronsPerTable)); ++j) {
					Neuron n = (Neuron) neurons.elementAt(j);
					code += " & " + n.getBias();
				;}
				code += "\\\\\n";
				
				code += "\t\tTF ";
				for(int j = currentStartNeuron; (j < neurons.size()) && (j < (currentStartNeuron + maxNeuronsPerTable)); ++j) {
					Neuron n = (Neuron) neurons.elementAt(j);
					code += " & " + getTransferFunctionAbbreviation(n);
				;}
				code += "\\\\\n";
				
				code += "\t\tType";
				for(int j = currentStartNeuron; (j < neurons.size()) && (j < (currentStartNeuron + maxNeuronsPerTable)); ++j) {
					Neuron n = (Neuron) neurons.elementAt(j);
					if(n.getType().equals(Neuron.INPUT)) {
						code += " & I ";
					}
					else if(n.getType().equals(Neuron.OUTPUT)) {
						code += " & O ";
					}
					else {
						code += " & H ";
					}
					
				;}
				code += "\\\\\n";
				
				code += "\t\tName";
				for(int j = currentStartNeuron; (j < neurons.size()) && (j < (currentStartNeuron + maxNeuronsPerTable)); ++j) {
					Neuron n = (Neuron) neurons.elementAt(j);
					if(n.getType().equals(Neuron.INPUT)) {
						code += " & " + n.getName();
					}
					else if(n.getType().equals(Neuron.OUTPUT)) {
						code += " & " + n.getName();
					}
					else {
						code += " &";
					}
					
				;}
				code += "\\\\\n";
				
				code += "\t\t\\hline\n";
				
				
				
				
				for(int i = 0; i < neurons.size(); ++i) {
					String line = "";
					boolean hasEntry = false;
					
					line += "\t\tN" + i;
					Neuron n = (Neuron) neurons.elementAt(i);
					for(int j = currentStartNeuron; (j < neurons.size()) && (j < (currentStartNeuron + maxNeuronsPerTable)); ++j) {
						Neuron target = (Neuron) neurons.elementAt(j);
						Vector synapses = target.getSynapses();
						Synapse s = null;
						boolean contained = false;
						for(int k = 0; k < synapses.size(); ++k) {
							s = (Synapse) synapses.elementAt(k);
							if(s.getSourceNeuron() == n) {
								contained = true;
								break;
							}
						}
						if(contained && s != null) {
							line += " & " + s.getStrength();
							hasEntry = true;
						}
						else {
							line += " & ";
						}
					;}
					if(i < neurons.size() -1) {
						line += "\\\\";
					}
					line += "\n";
					
					if(hasEntry) {
						code += line;
					}
				}
				
				
				
				code += "\t\t\\end{tabular}\n";
				code += "\t\t\\caption{Connection matrix of \\textbf{" + mNetworkName.getText() + "} (chapter \\ref{})";
				if(maxNeuronsPerTable < neurons.size()) {
					int leftNeurons = neurons.size() - maxNeuronsFirstPart;
					int totalParts = 1;
					totalParts += (leftNeurons / maxNeuronsPerTableRest);
					code += " Part: " + partNumber + "/" + totalParts + " ";
				}
				code += "}\n";
				code += "\t\\label{tab:Networks:Network_" + mNetworkName.getText() + partNumber + "}\n";
				code += "\\end{table}\n";
				
				currentStartNeuron += maxNeuronsPerTable;
				maxNeuronsPerTable = maxNeuronsPerTableRest;
				partNumber++;
			}
			
			mCodeArea.setText(code);
			
		
		}
	}
	
	
	public String getTransferFunctionAbbreviation(Neuron neuron) {
		TransferFunction tf = neuron.getTransferFunction();
		String tfa = "";
		if(tf != null) {
			Vector transferFunctions = Simulator.getInstance().getNeuralNetManager().getTransferFunctionPrototypes();
			for(int i = 0; i < transferFunctions.size(); ++i) {
				TransferFunction ltf = (TransferFunction) transferFunctions.elementAt(i);
				if(ltf.getName().equals(tf.getName())) {
					tfa = tf.getName().substring(0, Math.min(1, tf.getName().length())) + (i + 1);
				}
			}
		}
		return tfa;
	}
	
	
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}
	
	

}
