package de.fhg.iais.outdoor.simba.visualization.neuron;


import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.visualization.SimulationVisualization;
import de.fhg.iais.outdoor.simba.visualization.VisibilityListener;

public class NetworkViewer extends SimulationVisualization implements NeuralNetListener,
								VisibilityListener {
	
	private NeuronVisualizationEnvironment mNeuronEnvironment;
	private boolean mOptimized;
	private boolean mVisible;
	private boolean mNetworkInvalid;
	
	public NetworkViewer(NeuronVisualizationEnvironment environmentToVisualize) {
		super("/Visualization/Neurons/Control", environmentToVisualize);
		mNeuronEnvironment = environmentToVisualize;
		Simulator.getInstance().getScheduler().removeStepListener(this);
		mNetworkInvalid = true;
	}
	
	
	public boolean init() {
		boolean ok = super.init();
		
		Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		
		return ok;
	}


	/**
	 * Repaints the visualization after each step.
	 */
	public void repaintNetwork() {
		mVisualizationIntervalCounter++;
		if(mShowSimulation.get()) {
			if(mVisualizationIntervalCounter >= mVisualizationInterval.get()) {
				mVisualizationIntervalCounter = 0;
				mVisuArea.repaint();
			}
		}
	}


	public void optimizePerformance(boolean optimize) {
		super.optimizePerformance(optimize);
		Simulator.getInstance().getScheduler().removeStepListener(this); //always remove.
		if(!optimize && mVisible) {
			//Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
			if(mNetworkInvalid && mNeuronEnvironment != null) {
				mNeuronEnvironment.resetEnvironmentObjects();
				mNetworkInvalid = false;
			}
		}
		else {
			//Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
		mOptimized = optimize;
	}


	public void currentNeuralNetChanged(NeuralNet currentNet) {
		if(!mVisible || mOptimized) {
			mNetworkInvalid = true;
			return;
		}
		if(mNeuronEnvironment != null) {
			mNeuronEnvironment.resetEnvironmentObjects();
		}
		repaintNetwork();
	}


	public void networkActivationsUpdated(NeuralNet currentNet) {
		if(!mVisible || mOptimized) {
			return;
		}
		mModelToVisualize.initEnvironment();
		mModelToVisualize.executeSimulationStep();
		repaintNetwork();
	}


	public void networkStructureChanged(NeuralNet currentNet) {
		if(!mVisible || mOptimized) {
			mNetworkInvalid = true;
			return;
		}
		if(mNeuronEnvironment != null) {
			mNeuronEnvironment.resetEnvironmentObjects();
		}
		repaintNetwork();
	}


	public void visibilityChanged(boolean visible, Object source) {
		if(!mOptimized && visible) {
			//Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
			//mNeuronEnvironment.resetEnvironmentObjects();
			if(mNetworkInvalid && mNeuronEnvironment != null) {
				mNeuronEnvironment.resetEnvironmentObjects();
				mNetworkInvalid = false;
			}
		}
		else {
			//Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
		mVisible = visible;
	}
	
	
	
	

}
