package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;

import javax.swing.JScrollPane;
import javax.swing.SwingConstants;


import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.visualization.BorderLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.BoxLayoutVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.FrameVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.OptimizePerformanceListener;
import de.fhg.iais.outdoor.simba.visualization.ScrollPaneVisualizationArea;
import de.fhg.iais.outdoor.simba.visualization.VisibilityListener;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.VisualizationIntervalController;

public class NeuralNetworkEditor extends BorderLayoutVisualizationArea implements VisibilityListener,
									OptimizePerformanceListener 
{
	
	private NeuronVisualizationEnvironment mNeuralNetworkEnvironment;
	private NeuronEditorPanel mNeuronEditorPanel;
	private SynapseEditorPanel mSynapseEditorPanel;
	private SaveAndLoadNetworkPanel mSaveAndLoadPanel;
	private NetworkModificationPanel mNetworkModificationPanel;
	private ResetNeuralNetworkButton mResetNeuralNetworkButton;
	private TransferFunctionSelectionPanel mTransferFunctionSelector;
	private NetworkViewer mNetworkViewer;
	private boolean mVisible;
	private boolean mOptimized;
	private ScrollPaneVisualizationArea mControlArea;
	private NeuronRelationHighlighter mNeuronRelationHighlighter;
	
	private FrameVisualizationArea mPlotArea;
	
	public NeuralNetworkEditor() {
		mNeuralNetworkEnvironment = new NeuronVisualizationEnvironment();
		mNetworkViewer = new NetworkViewer(mNeuralNetworkEnvironment);
		mNetworkViewer.setOffset(-120, -630);
		mNetworkViewer.setDefaultView();
		addVisualization(mNetworkViewer, BorderLayout.CENTER);
		
		mControlArea = new ScrollPaneVisualizationArea(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		addVisualization(mControlArea, BorderLayout.EAST);
		
		BoxLayoutVisualizationArea leftArea = new BoxLayoutVisualizationArea();
		mControlArea.addVisualization(leftArea, "");
		
		mNeuronEditorPanel = new NeuronEditorPanel(mNetworkViewer);
		leftArea.addVisualization(mNeuronEditorPanel, "");
		
		mSynapseEditorPanel = new SynapseEditorPanel(mNetworkViewer);
		leftArea.addVisualization(mSynapseEditorPanel, "");
		
		mNetworkModificationPanel = new NetworkModificationPanel(mNeuralNetworkEnvironment);
		leftArea.addVisualization(mNetworkModificationPanel, "");
		
		BoxLayoutVisualizationArea innerBox = new BoxLayoutVisualizationArea(SwingConstants.HORIZONTAL);
		mTransferFunctionSelector = new TransferFunctionSelectionPanel();
		innerBox.addVisualization(mTransferFunctionSelector, "");
		mResetNeuralNetworkButton = new ResetNeuralNetworkButton();
		innerBox.addVisualization(mResetNeuralNetworkButton, "");
		
		leftArea.addVisualization(innerBox, "");
		
		mNeuronRelationHighlighter = new NeuronRelationHighlighter(mNetworkViewer, mNeuralNetworkEnvironment);
		leftArea.addVisualization(mNeuronRelationHighlighter, "");
		
		mSaveAndLoadPanel = new SaveAndLoadNetworkPanel(mNeuralNetworkEnvironment, mNetworkViewer);
		leftArea.addVisualization(mSaveAndLoadPanel, "");
		
		BoxLayoutVisualizationArea plotterButtons = new BoxLayoutVisualizationArea(SwingConstants.HORIZONTAL);
		addVisualization(plotterButtons, BorderLayout.SOUTH);
		
		RearrangeSynapsesPanel rearrangePanel = new RearrangeSynapsesPanel(mNeuralNetworkEnvironment);
		plotterButtons.addVisualization(rearrangePanel, "");
		
		
		mPlotArea = new FrameVisualizationArea("Plot1", 600, 200, 50, 50);
		mPlotArea.addVisualization(new NeuronPlotPanel(), BorderLayout.CENTER);
		plotterButtons.addVisualization(mPlotArea, "");
		mPlotArea = new FrameVisualizationArea("Plot2", 600, 200, 50, 250);
		mPlotArea.addVisualization(new NeuronPlotPanel(), BorderLayout.CENTER);
		plotterButtons.addVisualization(mPlotArea, "");
		mPlotArea = new FrameVisualizationArea("Plot3", 600, 200, 50, 450);
		mPlotArea.addVisualization(new NeuronPlotPanel(), BorderLayout.CENTER);
		plotterButtons.addVisualization(mPlotArea, "");
		mPlotArea = new FrameVisualizationArea("Plot4", 600, 200, 50, 650);
		mPlotArea.addVisualization(new NeuronPlotPanel(), BorderLayout.CENTER);
		plotterButtons.addVisualization(mPlotArea, "");
		
		mPlotArea.addVisualization(new NeuronPlotPanel(), BorderLayout.CENTER);
		
		VisualizationIntervalController intervalController = new VisualizationIntervalController("/Visualization/Neurons/Control/VisualizationInterval", SwingConstants.HORIZONTAL);
		plotterButtons.addVisualization(intervalController, "");
		
		//Install short cut keyboard commands for copy, past, delete.
		new NeuronEditFunctions(mNeuralNetworkEnvironment, mNetworkViewer);
		
		
		Simulator.getInstance().addOptimizePerformanceListener(this);
	}

	protected boolean installVisualization(Visualization visu, String positionIdentifier) {
		boolean ok = super.installVisualization(visu, positionIdentifier);
		
		return ok;
	}

	
	public void visibilityChanged(boolean visible, Object source) {
		if(visible && !mOptimized) {
			mNetworkViewer.showSimulation(true);
		}
		else {
			mNetworkViewer.showSimulation(false);
		}
		mVisible = visible;
		notifyVisibilityChanged(visible);
	}

	public void optimizePerformance(boolean optimize) {
		if(!optimize && mVisible) {
			
		}
		else {
			
		}
		mOptimized = optimize;
	}
	
	

	
	

}
