package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunction;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleWrapper;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.OptimizePerformanceListener;
import de.fhg.iais.outdoor.simba.visualization.VisibilityListener;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanel;

public class NeuronEditorPanel implements Visualization, ValueChangedListener, 
							NeuralNetListener, VisibilityListener, OptimizePerformanceListener {
	
	
	private JPanel mDisplay;
	private ValuePanel mNamePanel;
	private ValuePanel mActivityPanel;
	private ValuePanel mBiasPanel;
	private ValuePanel mMaxIncomingSynapsesPanel;
	private ValuePanel mMaxOutgoingSynapsesPanel;
	private SimObjectValue mSelectedObject;
	private NeuronObject mLastSelectedObject;
	private boolean mOptimizePerformance;
	private boolean mVisible;
	private JComboBox mTransferFunction;
	private JCheckBox mAllTransferFunctionsCheckBox;
	private Vector mTransferFunctionPrototypes = new Vector();
	private Vector mProtectionPrototypes = new Vector();
	private JComboBox mProtectionMode;
	private NetworkViewer mNetworkViewer;
	private ActionListener mTransferFunctionListener;
	private ActionListener mProtectionModeListener;
	
	public NeuronEditorPanel(NetworkViewer viewer) {
		mNetworkViewer = viewer;
		
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("Selected Neuron"));
		Box boxLeft = Box.createVerticalBox();
		mDisplay.add(boxLeft, BorderLayout.CENTER);
		Box boxRight = Box.createVerticalBox();
		mDisplay.add(boxRight, BorderLayout.EAST);
		
		mNamePanel = new ValuePanel(null, "Name:", 15, false, true);
		mActivityPanel = new ValuePanel(null, "Activity", 5, false, true);
		mBiasPanel = new ValuePanel(null, "Bias:", 5, false, true);
		mMaxIncomingSynapsesPanel = new ValuePanel(null, "Max In:", 4, false, true);
		mMaxOutgoingSynapsesPanel = new ValuePanel(null, "Max Out:", 4, false, true);
		
		mDisplay.add(mNamePanel, BorderLayout.NORTH);
		boxLeft.add(mActivityPanel);
		boxLeft.add(mBiasPanel);
		boxRight.add(mMaxIncomingSynapsesPanel);
		boxRight.add(mMaxOutgoingSynapsesPanel);
		
		JPanel lowerPanel = new JPanel(new BorderLayout());
		mTransferFunction = new JComboBox();
		mTransferFunction.setEnabled(false);
		mAllTransferFunctionsCheckBox = new JCheckBox("All Neurons");
		mAllTransferFunctionsCheckBox.setSelected(false);
		
		JPanel transferPanel = new JPanel(new BorderLayout());
		
		transferPanel.add(mTransferFunction, BorderLayout.EAST);
		transferPanel.add(new JLabel("TransferFunction:"), BorderLayout.CENTER);
		
		//TODO check if all transfer functions should be changeable from here.
//		transferFunctionPanel.add(mAllTransferFunctionsCheckBox, BorderLayout.WEST);
		
		mTransferFunctionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateTransferFunction();
			}
		};
		mTransferFunction.addActionListener(mTransferFunctionListener);
		
		
		mProtectionMode = new JComboBox();
		mProtectionMode.setEnabled(false);
		mProtectionModeListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateProtectionMode();
			}
		};
		mProtectionMode.addActionListener(mProtectionModeListener);
		
		JPanel protectionPanel = new JPanel(new BorderLayout());
		protectionPanel.add(mProtectionMode, BorderLayout.EAST);
		protectionPanel.add(new JLabel("Protection:"), BorderLayout.CENTER);
		
		lowerPanel.add(transferPanel, BorderLayout.NORTH);
		lowerPanel.add(protectionPanel, BorderLayout.SOUTH);
		mDisplay.add(lowerPanel, BorderLayout.SOUTH);
		
		
		
				
		Simulator.getInstance().addSystemObject(this);
		Simulator.getInstance().addOptimizePerformanceListener(this);
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mSelectedObject = vm.getSimObjectValue("/Visualization/Neurons/Control/LastSelectedObject");
		if(mSelectedObject != null) {
			mSelectedObject.addValueReplacedListener(this);
			mSelectedObject.addValueChangedListener(this);
		}
		
		Vector transferFunctions = Simulator.getInstance().getNeuralNetManager().getTransferFunctionPrototypes();
		for(Iterator i = transferFunctions.iterator(); i.hasNext();) {
			TransferFunction tf = (TransferFunction) i.next();
			mTransferFunctionPrototypes.add(tf);
			mTransferFunction.addItem(tf.getName());
		}
		
		mProtectionPrototypes.clear();
		new Neuron(""); //make sure that the prototypes are created.
		mProtectionPrototypes.addAll(Neuron.mProtectionModes);
		for(Iterator i = mProtectionPrototypes.iterator(); i.hasNext();) {
			mProtectionMode.addItem(i.next());
		}
		
		
		return true;
	}
	
	public void synchronizeWithSelectedObject() {
		
		Vector selectedObjects = mNetworkViewer.getSelectedObjects();
		if(mSelectedObject != null && mSelectedObject.get() != null) {
			if(mSelectedObject.get() instanceof NeuronObject) {
				NeuronObject no = (NeuronObject) mSelectedObject.get();
				mLastSelectedObject = no;
			}
		}
		else {
			mLastSelectedObject = null;
		}
		if(mLastSelectedObject == null || !selectedObjects.contains(mLastSelectedObject)) {
			mLastSelectedObject = null;
			for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
				SimObject obj = (SimObject) i.next();
				if(obj instanceof NeuronObject) {
					mLastSelectedObject = (NeuronObject) obj;
					break;
				}
			}
		}
		
		if(Simulator.getInstance().getNeuralNetManager().getCurrentNet() == null
				|| mSelectedObject == null || mSelectedObject.get() == null
				|| mLastSelectedObject == null || mNetworkViewer == null) 
		{
			mLastSelectedObject = null;
			mNamePanel.setValue(null);
			if(mBiasPanel.getValue() != null) {
				mBiasPanel.getValue().removeValueChangedListener(this);
			}
			mBiasPanel.setValue(null);
			mActivityPanel.setValue(null);
			if(mMaxIncomingSynapsesPanel.getValue() != null) {
				mMaxIncomingSynapsesPanel.getValue().removeValueChangedListener(this);
			}
			mMaxIncomingSynapsesPanel.setValue(null);
			if(mMaxOutgoingSynapsesPanel.getValue() != null) {
				mMaxOutgoingSynapsesPanel.getValue().removeValueChangedListener(this);
			}
			mMaxOutgoingSynapsesPanel.setValue(null);
		}
		else if(mLastSelectedObject != null) {
			if(mLastSelectedObject.isActive()) {
				Neuron neuron = mLastSelectedObject.getNeuron();
				if(neuron == null) {
					mLastSelectedObject = null;
					return;
				}
				mNamePanel.setValue(neuron.getNameValue());
				mBiasPanel.setValue(neuron.getBiasValue());
				neuron.getBiasValue().addValueChangedListener(this);
				mActivityPanel.setValue(new NormalizedDoubleWrapper(neuron.getActivationValue()));
				mMaxIncomingSynapsesPanel.setValue(neuron.getMaxIncomingSynapsesValue());
				neuron.getMaxIncomingSynapsesValue().addValueChangedListener(this);
				mMaxOutgoingSynapsesPanel.setValue(neuron.getMaxOutgoingSynapsesValue());
				neuron.getMaxOutgoingSynapsesValue().addValueChangedListener(this);
				mNamePanel.setActivationCheckboxSelected(true);
				mBiasPanel.setActivationCheckboxSelected(true);
				mActivityPanel.setActivationCheckboxSelected(true);
				mMaxIncomingSynapsesPanel.setActivationCheckboxSelected(true);
				mMaxOutgoingSynapsesPanel.setActivationCheckboxSelected(true);
				
				//transferfunction
				int index = 0;
				mTransferFunction.removeActionListener(mTransferFunctionListener);
				mTransferFunction.setEnabled(true);
				for(; index < mTransferFunctionPrototypes.size(); ++index) {
					TransferFunction tf = (TransferFunction) mTransferFunctionPrototypes.elementAt(index);
					if(neuron.getTransferFunction().getName().equals(tf.getName())) {
						break;
					}
				}
				mTransferFunction.setSelectedIndex(index);
				mTransferFunction.addActionListener(mTransferFunctionListener);
				
				//Protection mode
				index = 0;
				mProtectionMode.removeActionListener(mProtectionModeListener);
				mProtectionMode.setEnabled(true);
				for(; index < mProtectionPrototypes.size(); ++index) {
					String protection = (String) mProtectionPrototypes.elementAt(index);
					if(neuron.getProtectionMode().equals(protection)) {
						break;
					}
				}
				mProtectionMode.setSelectedIndex(index);
				mProtectionMode.addActionListener(mProtectionModeListener);
				
				return;
			}
			else {
				mLastSelectedObject = null;
				mNamePanel.setValue(null);
				mBiasPanel.setValue(null);
				mActivityPanel.setValue(null);
				mMaxIncomingSynapsesPanel.setValue(null);
				mMaxOutgoingSynapsesPanel.setValue(null);
				mTransferFunction.setEnabled(false);
				mProtectionMode.setEnabled(false);
			}
		}
		
	}
	
	public void updateTransferFunction() {
		if(mAllTransferFunctionsCheckBox.isSelected()) {
			String tfName = (String) mTransferFunction.getSelectedItem();
			for(Iterator i = mTransferFunctionPrototypes.iterator(); i.hasNext();) {
				TransferFunction tf = (TransferFunction) i.next();
				if(tf.getName().equals(tfName)) {
					//change transfer function for all neurons.
					NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
					if(net != null) {
						synchronized (net) {
							Vector neurons = net.getNeurons();
							for(Iterator j = neurons.iterator(); j.hasNext();) {
								((Neuron) j.next()).setTransferFunction(tf);
							}
							net.setDefaultTransferFunction(tf);
							return;
						}
					}
				}
			}
		}
		else {
			if(mLastSelectedObject != null && mLastSelectedObject.isActive()) {
				
				Vector selectedObjects = mNetworkViewer.getSelectedObjects();
				if(selectedObjects.contains(mLastSelectedObject)) {
					TransferFunction tf = (TransferFunction) mTransferFunctionPrototypes.elementAt(mTransferFunction.getSelectedIndex());
					for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
						SimObject obj = (SimObject) i.next();
						if(!(obj instanceof NeuronObject)) {
							continue;
						}
						Neuron neuron = ((NeuronObject) obj).getNeuron();
						if(neuron == null) {
							return;
						}
						if(tf != null) {
							neuron.setTransferFunction(tf);
						}
					}
				}
				else {
					Neuron neuron = mLastSelectedObject.getNeuron();
					if(neuron == null) {
						return;
					}
					TransferFunction tf = (TransferFunction) mTransferFunctionPrototypes.elementAt(mTransferFunction.getSelectedIndex());
					if(tf != null) {
						neuron.setTransferFunction(tf);
					}
				}
			}
		}
	}
	
	public void updateProtectionMode() {
		
		
		if(mLastSelectedObject != null && mLastSelectedObject.isActive()) {
			Vector selectedObjects = mNetworkViewer.getSelectedObjects();
			if(selectedObjects.contains(mLastSelectedObject)) {
				String protection = (String) mProtectionMode.getSelectedItem();
				for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
					SimObject obj = (SimObject) i.next();
					if(!(obj instanceof NeuronObject)) {
						continue;
					}
					Neuron neuron = ((NeuronObject) obj).getNeuron();
					if(neuron == null) {
						return;
					}
					if(protection != null) {
						neuron.setProtectionMode(protection);
					}
				}
			}
			else {
				Neuron neuron = mLastSelectedObject.getNeuron();
				if(neuron == null) {
					return;
				}
				String protection = (String) mProtectionMode.getSelectedItem();
				if(protection != null) {
					neuron.setProtectionMode(protection);
				}
			}
		}
	}

	public void valueChanged(Value value) {
		if(value == mSelectedObject) {
			synchronizeWithSelectedObject();
			
		}
		if(mLastSelectedObject != null && mLastSelectedObject.isActive()) {
			if(value == mBiasPanel.getValue()) {
				Vector selectedObjects = mNetworkViewer.getSelectedObjects();
				if(selectedObjects.contains(mLastSelectedObject)) {
					double bias = ((DoubleValue) mBiasPanel.getValue()).get();
					for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
						SimObject obj = (SimObject) i.next();
						if(obj == mLastSelectedObject || !(obj instanceof NeuronObject)) {
							continue;
						}
						Neuron neuron = ((NeuronObject) obj).getNeuron();
						if(neuron == null) {
							return;
						}
						neuron.getBiasValue().set(bias);
					}
				}
			}
			else if(value == mMaxIncomingSynapsesPanel.getValue()) {
				Vector selectedObjects = mNetworkViewer.getSelectedObjects();
				if(selectedObjects.contains(mLastSelectedObject)) {
					int maxIn = ((IntValue) mMaxIncomingSynapsesPanel.getValue()).get();
					for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
						SimObject obj = (SimObject) i.next();
						if(obj == mLastSelectedObject || !(obj instanceof NeuronObject)) {
							continue;
						}
						Neuron neuron = ((NeuronObject) obj).getNeuron();
						if(neuron == null) {
							return;
						}
						neuron.getMaxIncomingSynapsesValue().set(maxIn);
					}
				}
			}
			else if(value == mMaxOutgoingSynapsesPanel.getValue()) {
				Vector selectedObjects = mNetworkViewer.getSelectedObjects();
				if(selectedObjects.contains(mLastSelectedObject)) {
					int maxOut = ((IntValue) mMaxOutgoingSynapsesPanel.getValue()).get();
					for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
						SimObject obj = (SimObject) i.next();
						if(obj == mLastSelectedObject || !(obj instanceof NeuronObject)) {
							continue;
						}
						Neuron neuron = ((NeuronObject) obj).getNeuron();
						if(neuron == null) {
							return;
						}
						neuron.getMaxOutgoingSynapsesValue().set(maxOut);
					}
				}
			}
			mNetworkViewer.getDisplayPanel().repaint();
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mSelectedObject = (SimObjectValue) ValueManager.synchronizeValue(mSelectedObject, oldValue, newValue, this);
	}

	public void currentNeuralNetChanged(NeuralNet currentNet) {
		synchronizeWithSelectedObject();
	}

	public void networkActivationsUpdated(NeuralNet currentNet) {
	}

	public void networkStructureChanged(NeuralNet currentNet) {
		synchronizeWithSelectedObject();
	}
	
	/**
	 * Called whenever the source object gets visible or invisible. Usually this is a
	 * Frame like the FrameVisualizationArea that indicates whether or not the Frame
	 * with all its content is visible. 
	 * In the case that the Frame is not visible, all observations of all ValuePanels
	 * are temporarily disabled to increase the performance (if the frame is not seen
	 * then there is no need to request values).
	 */
	public void visibilityChanged(boolean visible, Object source) {
		mVisible = visible;
		mNamePanel.enableObservation(mVisible && !mOptimizePerformance);
		mActivityPanel.enableObservation(mVisible && !mOptimizePerformance);
		mBiasPanel.enableObservation(mVisible && !mOptimizePerformance);
		if(mVisible && !mOptimizePerformance) {
			Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		}
		else {
			Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
	}
	
	
	/**
	 * Called if the system was set to optimized performance mode.
	 * In this mode this Visualization disables the observation of all ValuePanels in the
	 * PanelWrappers to avoid computations for displaying values.
	 */
	public void optimizePerformance(boolean optimize) {
		mOptimizePerformance = optimize;
		
		mNamePanel.enableObservation(mVisible && !mOptimizePerformance);
		mActivityPanel.enableObservation(mVisible && !mOptimizePerformance);
		mBiasPanel.enableObservation(mVisible && !mOptimizePerformance);
		
		if(mVisible && !mOptimizePerformance) {
			Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		}
		else {
			Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
	}
	

}
