package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.SystemObject;

public abstract class NeuronGraphPlotter implements SystemObject {
	
	private static int mPlotterIndexCounter = 0;
	private int mPlotterIndex;
	private double mScale;
	private String mName;
	private double mVerticalScale;
	private double mWidth;
	private double mHeight;
	
	
	public NeuronGraphPlotter(String name) {
		mName = name;
		mScale = 1.0;
		mPlotterIndex = mPlotterIndexCounter++;
		Simulator.getInstance().addSystemObject(this);
	}
	
	public String getName() {
		return mName;
	}
	
	public abstract void plotHistroy(Graphics2D g2d, Vector neurons, Vector colors);
	
	
	public void plot(Graphics2D g2d, Vector neurons, Vector colors, int width, int height) {
		
		mWidth = width;
		mHeight = height;
		
		//Clear area.
		g2d.setColor(Color.WHITE);
		g2d.fillRect(0, 0, width, height);

		int neuronIndex = 0;

		//Draw the raster of the diagram.
		mVerticalScale = ((double) height - 20) / 2.0;
		g2d.setColor(Color.LIGHT_GRAY);
		g2d.drawLine(0, 9, width, 9);
		g2d.drawLine(0, (int) (mVerticalScale) + 10, width, (int) (1 * mVerticalScale) + 10);
		g2d.drawLine(0, (int) (2 * mVerticalScale) + 11, width, (int) (2 * mVerticalScale) + 11);
		
		
		g2d.drawLine(0, (int) (0.5 * mVerticalScale) + 10, width, (int) (0.5 * mVerticalScale) + 10);
		g2d.drawLine(0, (int) (1.5 * mVerticalScale) + 10, width, (int) (1.5 * mVerticalScale) + 10);
		
		//plot graph
		plotHistroy(g2d, neurons, colors);
		
		//Plot the names of the neurons in the colors of the neuron.
		neuronIndex = 0;
		for(Iterator n = neurons.iterator(); n.hasNext();) {
			
			NeuronObject obj = (NeuronObject) n.next();
			
			g2d.setColor((Color) colors.elementAt(neuronIndex));

			String name = obj.getName();
			int offset = Math.max(0, name.length() - 25);
			g2d.drawChars(name.toCharArray(), offset, name.length() - offset, 10, 20 + (15 * neuronIndex));
			
			neuronIndex++;
		}
		
		g2d.setColor(Color.BLACK);
		//Plot the range
		g2d.setColor(Color.BLACK);
		String one = Double.toString((double) (1.0 / mScale));
		String minusOne = Double.toString((double) (-1.0 / mScale));
		String zero = Integer.toString(0);
		g2d.drawChars(one.toCharArray(), 0, Math.min(4, one.length()), 7, 13);
		g2d.drawChars(zero.toCharArray(), 0, zero.length(), 7, (int) (mVerticalScale) + 14);
		g2d.drawChars(minusOne.toCharArray(), 0, Math.min(5, minusOne.length()), 7, (int) (2 * mVerticalScale) + 14);
	}
	
	
	public double translateValue(double value) {
		return ((((value * mScale) - 1.0) * -1.0) * mVerticalScale) + 10.0;
	}
	
	public void setScale(double scale) {
		mScale = scale;
	}
	
	public double getScale() {
		return mScale;
	}
	
	
	public double getCurrentWidth() {
		return mWidth;
	}
	
	public double getCurrentHeight() {
		return mHeight;
	}

	public boolean cleanUp() {
		
		Properties props = Simulator.getInstance().getSystemProperties();
		props.put("/NeuronGraphPlotters/" + mName + mPlotterIndex + "/Scale", Double.toString(mScale));
		
		return true;
	}

	public boolean init() {
		
		Properties props = Simulator.getInstance().getSystemProperties();
		String scale = props.getProperty("/NeuronGraphPlotters/" + mName + mPlotterIndex + "/Scale");
		if(scale != null) {
			try {
				setScale(Double.parseDouble(scale));
			} catch (NumberFormatException e) {
			}
		}
		
		return true;
	}
	
	

}
