package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.OptimizePerformanceListener;
import de.fhg.iais.outdoor.simba.visualization.VisibilityListener;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class NeuronPlotPanel implements Visualization, VisibilityListener,
					ValueChangedListener, NeuralNetListener, OptimizePerformanceListener
					
{

	private JPanel mDisplay;
	private DiagramPlot mDiagramPlot;
	private JPanel mControlPanel;
	private JToggleButton mAddButton;
	private JButton mRemoveButton;
	private JButton mDefaultScaleButton;
	
	private SimObjectValue mDraggedObject;
	
	private Vector mColors = new Vector();
	private Vector mNeurons = new Vector();
	private Vector mNeuronColors = new Vector();
	private Vector mBufferVector = new Vector();
	private int mCurrentColor;
	private boolean mOptimized;
	private boolean mVisible;
	private JComboBox mPlotModes;
	private JSlider mScaleSlider;
	
	/**
	 * Standard constructor.
	 *
	 */
	public NeuronPlotPanel() {
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createRaisedBevelBorder());
		
		mDiagramPlot = new DiagramPlot(this);
		mDisplay.add(mDiagramPlot, BorderLayout.CENTER);
		
		mControlPanel = new JPanel(new BorderLayout());
		mAddButton = new JToggleButton("Add Neurons");
		
		mRemoveButton = new JButton("Remove");
		mRemoveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				synchronized(this) {
					mNeuronColors.clear();
					mNeurons.clear();
					mCurrentColor = 0;
				}
			}
		});
		
		mScaleSlider = new JSlider(SwingConstants.HORIZONTAL, -2500, 2500, 0);
		mScaleSlider.setPreferredSize(new Dimension(20, 30));
		mScaleSlider.setMinorTickSpacing(1000);
		
		mScaleSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				double scale = ((double) mScaleSlider.getValue());
				
				//System.out.println("scale here " + scale);
				if(scale < 0.0) {
					scale = (-1.0 / ((scale - 100.0) / 100.0));
				}
				else {
					scale = ((scale + 100) / 100.0);
				}
				NeuronGraphPlotter plotter = mDiagramPlot.getCurrentPlotter();
				if(plotter != null) {
					plotter.setScale(scale);
				}
				mDiagramPlot.repaint();
			}
		});
		
		
		mPlotModes = new JComboBox();
		mPlotModes.setPreferredSize(new Dimension(50, 25));
		mPlotModes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mDiagramPlot.setCurrentPlotter((String) mPlotModes.getSelectedItem());
				NeuronGraphPlotter plotter = mDiagramPlot.getCurrentPlotter();
				if(plotter != null) {
					double scale = plotter.getScale();
					if(scale < 1) {
						scale = ((-1.0 / scale) * 100.0) + 100.0;
					}
					else {
						scale = (scale * 100.0) - 100.0;
					}
					
					mScaleSlider.setValue((int) scale);
				}
				mDiagramPlot.repaint();
			}
		});
		
		addGraphPlotter(new NeuronActivityPlotter());
		addGraphPlotter(new NeuronActivityChangePlotter());
		addGraphPlotter(new NeuronActivtySumPlotter());
		addGraphPlotter(new NeuronActivitySumChangePlotter());
		
		
		mDefaultScaleButton = new JButton("1:1");
		mDefaultScaleButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				NeuronGraphPlotter plotter = mDiagramPlot.getCurrentPlotter();
				if(plotter != null) {
					plotter.setScale(1.0);
					mScaleSlider.setValue(0);
				}
				mDiagramPlot.repaint();
			}
		});
		
		
		JPanel buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.add(mAddButton, BorderLayout.NORTH);
		buttonPanel.add(mRemoveButton, BorderLayout.SOUTH);
		
		JPanel scalePanel = new JPanel(new BorderLayout());
		scalePanel.add(mScaleSlider, BorderLayout.NORTH);
		scalePanel.add(mDefaultScaleButton, BorderLayout.SOUTH);
		
		JPanel plotSelectPanel = new JPanel(new BorderLayout());
		plotSelectPanel.add(mPlotModes, BorderLayout.NORTH);
		plotSelectPanel.add(scalePanel, BorderLayout.SOUTH);
		
		
		mControlPanel.add(buttonPanel, BorderLayout.NORTH);
		mControlPanel.add(plotSelectPanel, BorderLayout.SOUTH);
		
		mDisplay.add(mControlPanel, BorderLayout.EAST);

		mColors.add(Color.GREEN);
		mColors.add(Color.RED);
		mColors.add(Color.BLUE);
		mColors.add(Color.BLACK);
		mColors.add(Color.CYAN);
		mColors.add(Color.MAGENTA);
		mColors.add(Color.YELLOW);
		mColors.add(Color.PINK);
		
		
		
		mDiagramPlot.setPreferredSize(new Dimension(200, 150));
		
		Simulator.getInstance().addOptimizePerformanceListener(this);
		
	}


	/**
	 * Make sure that unused, inactive or invalid NeuronObjects are removed from 
	 * the plot list.
	 */
	public void synchronizeWithNetwork() {
		synchronized(this) {
			mBufferVector.clear();
			mBufferVector.addAll(mNeurons);
			for(Iterator i = mBufferVector.iterator(); i.hasNext();) {
				NeuronObject obj = (NeuronObject) i.next();
				NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
				synchronized (net) {
					if(!obj.isActive() || obj.getNeuron() == null
							|| (net != null && !net.getNeurons().contains(obj.getNeuron())))
					{
						mNeurons.remove(obj);
					}
				}
			}
		}
	}
	
	/**
	 * Adds a new NeuronObject to the plot list, if this NeuronObject is not already 
	 * contained. The NeuronObject is associated with a specific Color that is used 
	 * to identify this NeuronObject in the plots.
	 * 
	 * @param neuron the neuronObject to add to the plot list.
	 */
	public void addNeuronToVisualize(NeuronObject neuron) {
		synchronized(this) {
			if(mNeurons.contains(neuron) || neuron.getNeuron() == null) {
				return;
			}
			mNeurons.add(neuron);
			mNeuronColors.add(mColors.elementAt(mCurrentColor % mColors.size()));
		}
		mCurrentColor++;
		
		mDiagramPlot.repaint();
	}
	
	public boolean addGraphPlotter(NeuronGraphPlotter plotter) {
		boolean ok = mDiagramPlot.addGraphPlotter(plotter);
		updatePlotterSelector();
		return ok;
	}
	
	public boolean removeGraphPlotter(NeuronGraphPlotter plotter) {
		boolean ok = mDiagramPlot.removeGraphPlotter(plotter);
		updatePlotterSelector();
		return ok;
	}
	
	public void updatePlotterSelector() {
		Vector plotter = mDiagramPlot.getGraphPlotters();
		mPlotModes.removeAllItems();
		for(Iterator i = plotter.iterator(); i.hasNext();) {
			mPlotModes.addItem(((NeuronGraphPlotter) i.next()).getName().trim());
		}
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		ValueManager vm = Simulator.getInstance().getValueManager(); 
		mDraggedObject = vm.getSimObjectValue("/Visualization/Neurons/Control/DraggedObject");
		if(mDraggedObject != null) {
			mDraggedObject.addValueReplacedListener(this);
			mDraggedObject.addValueChangedListener(this);
		}
		if(mDraggedObject == null) {
			Logger.getLogger("Simulator").warning("NetworkModificationPanel: Could not find all required value objects.");
		}
		
		Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		
		
		return true;
	}

	/**
	 * If the currently selected object of the neuron visualization view changed and
	 * the add button is selected, then the selected neuron is added to the plot list.
	 * This is only true if the object is of type NeuronObject and if this object
	 * is not already in the plot list.
	 */
	public void valueChanged(Value value) {
		if(value != null) {
			if(value == mDraggedObject) {
				if(mAddButton.isSelected() && (mDraggedObject.get() instanceof NeuronObject)) {
					addNeuronToVisualize((NeuronObject) mDraggedObject.get());
				}
			}
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mDraggedObject = (SimObjectValue) ValueManager.synchronizeValue(mDraggedObject, oldValue, newValue, this);
	}


	public void currentNeuralNetChanged(NeuralNet currentNet) {
		synchronizeWithNetwork();
	}

	public void networkActivationsUpdated(NeuralNet currentNet) {
		mDiagramPlot.repaint();
	}

	public void networkStructureChanged(NeuralNet currentNet) {
		synchronizeWithNetwork();
	}
	
	
	
	public void visibilityChanged(boolean visible, Object source) {
		if(!mOptimized && visible) {
			Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
			mDiagramPlot.repaint();
		}
		else {
			Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
		mVisible = visible;
	}

	public void optimizePerformance(boolean optimize) {
		if(!optimize && mVisible) {
			Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
			mDiagramPlot.repaint();
		}
		else {
			Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
		mOptimized = optimize;
	}



	public class DiagramPlot extends JPanel {
		
		public static final long serialVersionUID = 123L; 
		private NeuronPlotPanel mOwner;
		private Vector mGraphPlotters = new Vector();
		private NeuronGraphPlotter mCurrentPlotter;
		
		public DiagramPlot(NeuronPlotPanel owner) {
			mOwner = owner;
			setBorder(BorderFactory.createEtchedBorder());
		}

		protected void paintComponent(Graphics g) {
			Graphics2D g2d = (Graphics2D) g;
			Color c = g2d.getColor();
			
			synchronized(mOwner) {
				if(mCurrentPlotter != null) {
					mCurrentPlotter.plot(g2d, mNeurons, mNeuronColors, getWidth(), getHeight());
				}
				else {
					//Clear area.
					g2d.setColor(Color.WHITE);
					g2d.fillRect(0, 0, getWidth(), getHeight());
				}
			}
			
			g2d.setColor(c);

		}
		
		protected boolean addGraphPlotter(NeuronGraphPlotter plotter) {
			if(plotter == null || mGraphPlotters.contains(plotter)) {
				return false;
			}
			mGraphPlotters.add(plotter);
			if(mCurrentPlotter == null) {
				mCurrentPlotter = plotter;
			}
			return true;
		}
		
		protected boolean removeGraphPlotter(NeuronGraphPlotter plotter) {
			if(plotter == null || !mGraphPlotters.contains(plotter)) {
				return false;
			}
			if(mCurrentPlotter == plotter) {
				if(mGraphPlotters.isEmpty()) {
					mCurrentPlotter = null;
				}
				else {
					mCurrentPlotter = (NeuronGraphPlotter) mGraphPlotters.elementAt(0);
				}
			}
			mGraphPlotters.remove(plotter);
			return true;
		}
		
		public Vector getGraphPlotters() {
			return new Vector(mGraphPlotters);
		}
		
		public NeuronGraphPlotter getCurrentPlotter() {
			return mCurrentPlotter;
		}
		
		public void setCurrentPlotter(String plotterName) {
			for(Iterator i = mGraphPlotters.iterator(); i.hasNext();) {
				NeuronGraphPlotter gp = (NeuronGraphPlotter) i.next();
				if(gp.getName().equals(plotterName)) {
					mCurrentPlotter = gp;
					break;
				}
			}
		}
		
//		private void paintHistory(Graphics2D g2d) {
//
//			//Clear area.
//			g2d.setColor(Color.WHITE);
//			g2d.fillRect(0, 0, getWidth(), getHeight());
//
//			int neuronIndex = 0;
//
//			//Draw the raster of the diagram.
//			double verticalScale = ((double) getHeight() - 20) / 2.0;
//			g2d.setColor(Color.LIGHT_GRAY);
//			g2d.drawLine(0, 9, getWidth(), 9);
//			g2d.drawLine(0, (int) (verticalScale) + 10, getWidth(), (int) (1 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (2 * verticalScale) + 11, getWidth(), (int) (2 * verticalScale) + 11);
//			
//			
//			g2d.drawLine(0, (int) (0.5 * verticalScale) + 10, getWidth(), (int) (0.5 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (1.5 * verticalScale) + 10, getWidth(), (int) (1.5 * verticalScale) + 10);
//			
//			
//			synchronized (mOwner) {
//					
//				//Plot the history of each neuron in the color of that neuron.
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					LinkedList p = new LinkedList();
//					if(obj.getNeuron() != null) {
//						p = obj.getNeuron().getHistory();
//					}
//					synchronized(p) {
//						g2d.setColor((Color) mNeuronColors.elementAt(neuronIndex));				
//						double pointDist = ((double) getWidth()) / ((double) p.size());
//						
//						int pointId = 0;
//						Point point = new Point(0, 0);
//						for(Iterator i = p.iterator(); i.hasNext();) {
//							Double d = (Double) i.next();
//							Point nextPoint = new Point((int) (pointId * pointDist), (int) ((((d.doubleValue() * mScaleMode) - 1.0) * -1) * verticalScale) + 10);
//							if(pointId != 0) {
//								g2d.drawLine(point.x, point.y, nextPoint.x, nextPoint.y);
//							}
//							pointId++;
//							point = nextPoint;
//						}
//					}
//					neuronIndex++;
//				}
//				
//				//Plot the names of the neurons in the colors of the neuron.
//				neuronIndex = 0;
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					g2d.setColor((Color) mNeuronColors.elementAt(neuronIndex));
//
//					String name = obj.getName();
//					int offset = Math.max(0, name.length() - 25);
//					g2d.drawChars(name.toCharArray(), offset, name.length() - offset, 10, 20 + (15 * neuronIndex));
//					
//					neuronIndex++;
//				}
//				
//				g2d.setColor(Color.BLACK);
//				//Plot the range
//				g2d.setColor(Color.BLACK);
//				String one = Double.toString((double) (1.0 / mScaleMode));
//				String minusOne = Double.toString((double) (-1.0 / mScaleMode));
//				String zero = Integer.toString(0);
//				g2d.drawChars(one.toCharArray(), 0, Math.min(4, one.length()), 5, 13);
//				g2d.drawChars(zero.toCharArray(), 0, zero.length(), 5, (int) (verticalScale) + 14);
//				g2d.drawChars(minusOne.toCharArray(), 0, Math.min(5, minusOne.length()), 5, (int) (2 * verticalScale) + 14);
//			}
//
//			
//			
//		}
		
//		private void paintActivityChange(Graphics2D g2d) {
//		
//			//Clear area.
//			g2d.setColor(Color.WHITE);
//			g2d.fillRect(0, 0, getWidth(), getHeight());
//
//			int neuronIndex = 0;
//
//			//Draw the raster of the diagram.
//			double verticalScale = ((double) getHeight() - 20) / 2.0;
//			g2d.setColor(Color.LIGHT_GRAY);
//			g2d.drawLine(0, 9, getWidth(), 9);
//			g2d.drawLine(0, (int) (verticalScale) + 10, getWidth(), (int) (1 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (2 * verticalScale) + 11, getWidth(), (int) (2 * verticalScale) + 11);
//			
//			
//			g2d.drawLine(0, (int) (0.5 * verticalScale) + 10, getWidth(), (int) (0.5 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (1.5 * verticalScale) + 10, getWidth(), (int) (1.5 * verticalScale) + 10);
//			
//			
//			synchronized (mOwner) {
//				
//				//Plot the history of each neuron in the color of that neuron.
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					LinkedList p = new LinkedList();
//					if(obj.getNeuron() != null) {
//						p = obj.getNeuron().getHistory();
//					}
//					synchronized(p) {
//						g2d.setColor((Color) mNeuronColors.elementAt(neuronIndex));				
//						double pointDist = ((double) getWidth()) / ((double) p.size());
//						
//						int pointId = 0;
//						Point point = new Point(0, 0);
//						double lastActivity = 0;
//						if(!p.isEmpty()) {
//							lastActivity = ((Double) p.getFirst()).doubleValue();
//						}
//						for(Iterator i = p.iterator(); i.hasNext();) {
//							double d = ((Double) i.next()).doubleValue();
//							double diff = d - lastActivity;
//							lastActivity = d;
//							Point nextPoint = new Point((int) (pointId * pointDist), (int) ((((diff * mScaleMode) - 1.0) * -1) * verticalScale) + 10);
//							if(pointId != 0) {
//								g2d.drawLine(point.x, point.y, nextPoint.x, nextPoint.y);
//							}
//							pointId++;
//							point = nextPoint;
//						}
//					}
//					neuronIndex++;
//				}
//				
//				//Plot the names of the neurons in the colors of the neuron.
//				neuronIndex = 0;
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					g2d.setColor((Color) mNeuronColors.elementAt(neuronIndex));
//
//					String name = obj.getName();
//					int offset = Math.max(0, name.length() - 25);
//					g2d.drawChars(name.toCharArray(), offset, name.length() - offset, 5, 20 + (15 * neuronIndex));
//					
//					neuronIndex++;
//				}
//			}
//
//			g2d.setColor(Color.BLACK);
//			String one = Double.toString((double) (1.0 / mScaleMode));
//			String minusOne = Double.toString((double) (-1.0 / mScaleMode));
//			String zero = Integer.toString(0);
//			g2d.drawChars(one.toCharArray(), 0, Math.min(4, one.length()), 5, 13);
//			g2d.drawChars(zero.toCharArray(), 0, zero.length(), 5, (int) (verticalScale) + 14);
//			g2d.drawChars(minusOne.toCharArray(), 0, Math.min(5, minusOne.length()), 5, (int) (2 * verticalScale) + 14);
//		
//			
//			
//		}
		
//		private void paintActivitySum(Graphics2D g2d) {
//			
//			//Clear area.
//			g2d.setColor(Color.WHITE);
//			g2d.fillRect(0, 0, getWidth(), getHeight());
//
//			//Draw the raster of the diagram.
//			double verticalScale = ((double) getHeight() - 20) / 2.0;
//			g2d.setColor(Color.LIGHT_GRAY);
//			g2d.drawLine(0, 9, getWidth(), 9);
//			g2d.drawLine(0, (int) (verticalScale) + 10, getWidth(), (int) (1 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (2 * verticalScale) + 11, getWidth(), (int) (2 * verticalScale) + 11);
//			
//			
//			g2d.drawLine(0, (int) (0.5 * verticalScale) + 10, getWidth(), (int) (0.5 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (1.5 * verticalScale) + 10, getWidth(), (int) (1.5 * verticalScale) + 10);
//			
//			
//			synchronized (mOwner) {
//				
//				//Plot the history of each neuron in the color of that neuron.
//				
//				Vector plotLists = new Vector();
//				
//				int dataMinSize = Integer.MAX_VALUE;
//				
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					if(obj.getNeuron() != null) {
//						LinkedList list = obj.getNeuron().getHistory();
//						synchronized(list) {
//							Object[] data = list.toArray();
//							plotLists.add(data);
//							if(dataMinSize > data.length) {
//								dataMinSize = data.length;
//							}
//						}
//					}
//				}
//				if(!plotLists.isEmpty()) {
//
//					int pointId = 0;
//					Point point = new Point(0, 0);
//					g2d.setColor(Color.RED);
//					double pointDist = ((double) getWidth()) / ((double) dataMinSize);
//					
//					for(int i = 0; i < dataMinSize; ++i) {
//						
//						double sum = 0.0;
//						for(Iterator j = plotLists.iterator(); j.hasNext();) {
//							sum += ((Double) ((Object[]) j.next())[i]).doubleValue();
//						}
//						Point nextPoint = new Point((int) (pointId * pointDist), (int) ((((sum * mScaleMode) - 1.0) * -1) * verticalScale) + 10);
//						if(pointId != 0) {
//							g2d.drawLine(point.x, point.y, nextPoint.x, nextPoint.y);
//						}
//						pointId++;
//						point = nextPoint;				
//					}	
//
//				}
//				
//				//Plot the names of the neurons in the colors of the neuron.
//				int neuronIndex = 0;
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					g2d.setColor((Color) mNeuronColors.elementAt(neuronIndex));
//
//					String name = obj.getName();
//					int offset = Math.max(0, name.length() - 25);
//					g2d.drawChars(name.toCharArray(), offset, name.length() - offset, 5, 20 + (15 * neuronIndex));
//					
//					neuronIndex++;
//				}
//			}
//
//			g2d.setColor(Color.BLACK);
//			String one = Double.toString((double) (1.0 / mScaleMode));
//			String minusOne = Double.toString((double) (-1.0 / mScaleMode));
//			String zero = Integer.toString(0);
//			g2d.drawChars(one.toCharArray(), 0, Math.min(4, one.length()), 5, 13);
//			g2d.drawChars(zero.toCharArray(), 0, zero.length(), 5, (int) (verticalScale) + 14);
//			g2d.drawChars(minusOne.toCharArray(), 0, Math.min(5, minusOne.length()), 5, (int) (2 * verticalScale) + 14);
//
//		}
		
		
//		private void paintActivitySumChange(Graphics2D g2d) {
//			
//			//Clear area.
//			g2d.setColor(Color.WHITE);
//			g2d.fillRect(0, 0, getWidth(), getHeight());
//
//			//Draw the raster of the diagram.
//			double verticalScale = ((double) getHeight() - 20) / 2.0;
//			g2d.setColor(Color.LIGHT_GRAY);
//			g2d.drawLine(0, 9, getWidth(), 9);
//			g2d.drawLine(0, (int) (verticalScale) + 10, getWidth(), (int) (1 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (2 * verticalScale) + 11, getWidth(), (int) (2 * verticalScale) + 11);
//			
//			
//			g2d.drawLine(0, (int) (0.5 * verticalScale) + 10, getWidth(), (int) (0.5 * verticalScale) + 10);
//			g2d.drawLine(0, (int) (1.5 * verticalScale) + 10, getWidth(), (int) (1.5 * verticalScale) + 10);
//			
//			
//			synchronized (mOwner) {
//				
//				//Plot the change of the activity sum  of all selected neurons
//				
//				Vector plotLists = new Vector();
//				
//				int dataMinSize = Integer.MAX_VALUE;
//				
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					if(obj.getNeuron() != null) {
//						LinkedList list = obj.getNeuron().getHistory();
//						synchronized(list) {
//							Object[] data = list.toArray();
//							plotLists.add(data);
//							if(dataMinSize > data.length) {
//								dataMinSize = data.length;
//							}
//						}
//						
//					}
//				}
//				if(!plotLists.isEmpty()) {
//
//					int pointId = 0;
//					Point point = new Point(0, 0);
//					g2d.setColor(Color.RED);
//					double pointDist = ((double) getWidth()) / ((double) dataMinSize);
//					double lastSum = 0.0;
//					for(int i = 0; i < dataMinSize; ++i) {
//						
//						double sum = 0.0;
//						for(Iterator j = plotLists.iterator(); j.hasNext();) {
//							sum += ((Double) ((Object[]) j.next())[i]).doubleValue();
//						}
//						double diff = sum - lastSum;
//						
//						Point nextPoint = new Point((int) (pointId * pointDist), (int) ((((diff * mScaleMode) - 1.0) * -1) * verticalScale) + 10);
//						if(pointId != 0) {
//							g2d.drawLine(point.x, point.y, nextPoint.x, nextPoint.y);
//						}
//						pointId++;
//						point = nextPoint;		
//						lastSum = sum;
//					}	
//
//				}
//				
//				//Plot the names of the neurons in the colors of the neuron.
//				int neuronIndex = 0;
//				for(Iterator neurons = mNeurons.iterator(); neurons.hasNext();) {
//					
//					NeuronObject obj = (NeuronObject) neurons.next();
//					
//					g2d.setColor((Color) mNeuronColors.elementAt(neuronIndex));
//
//					String name = obj.getName();
//					int offset = Math.max(0, name.length() - 25);
//					g2d.drawChars(name.toCharArray(), offset, name.length() - offset, 5, 20 + (15 * neuronIndex));
//					
//					neuronIndex++;
//				}
//			}
//
//			g2d.setColor(Color.BLACK);
//			String one = Double.toString((double) (1.0 / mScaleMode));
//			String minusOne = Double.toString((double) (-1.0 / mScaleMode));
//			String zero = Integer.toString(0);
//			g2d.drawChars(one.toCharArray(), 0, Math.min(4, one.length()), 5, 13);
//			g2d.drawChars(zero.toCharArray(), 0, zero.length(), 5, (int) (verticalScale) + 14);
//			g2d.drawChars(minusOne.toCharArray(), 0, Math.min(5, minusOne.length()), 5, (int) (2 * verticalScale) + 14);
//
//		}
		
		
		
	
	}
	


}
