package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class NeuronRelationHighlighter implements Visualization, ValueChangedListener {

	
	private JPanel mDisplay;
	private NetworkViewer mNetworkViewer;
	private NeuronVisualizationEnvironment mNeuronEnvironment;
	private JCheckBox mActivateCheckBox;
	private JCheckBox mIncommingNeurons;
	private JCheckBox mOutgoingNeurons;
	private JTextField mDepthRuler;
	private BooleanValue mSelectedObjectsChangedValue;
	private boolean mIsActive;
	
	public static final int HIGHLIGHT_INCOMMING = 1;
	public static final int HIGHLIGHT_OUTGOING = 2;
	public static final int HIGHLIGHT_ALL = 3;
	
	public NeuronRelationHighlighter(NetworkViewer viewer, NeuronVisualizationEnvironment neuronEnv) {
		mNetworkViewer = viewer;
		mNeuronEnvironment = neuronEnv;
		mSelectedObjectsChangedValue = mNetworkViewer.getSelectedObjectsChangedValue();
		mSelectedObjectsChangedValue.addValueChangedListener(this);
		
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("RelationHighlighter"));
		
		mActivateCheckBox = new JCheckBox("Activate");
		mActivateCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mIncommingNeurons.setEnabled(mActivateCheckBox.isSelected());
				mOutgoingNeurons.setEnabled(mActivateCheckBox.isSelected());
				mDepthRuler.setEnabled(mActivateCheckBox.isSelected());
				updateHighlighter();
			}
		});
		
		mIncommingNeurons = new JCheckBox("In");
		mIncommingNeurons.setSelected(true);
		mIncommingNeurons.setEnabled(false);
		mIncommingNeurons.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateHighlighter();
			}
		});
		
		mOutgoingNeurons = new JCheckBox("Out");
		mOutgoingNeurons.setSelected(true);
		mOutgoingNeurons.setEnabled(false);
		mOutgoingNeurons.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateHighlighter();
			}
		});
		
		mDepthRuler = new JTextField("1");
		mDepthRuler.setColumns(5);
		mDepthRuler.setEnabled(false);
		mDepthRuler.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateHighlighter();
			}
		});
		
		
		JPanel typePanel = new JPanel();
		typePanel.add(mActivateCheckBox);
		typePanel.add(mIncommingNeurons);
		typePanel.add(mOutgoingNeurons);
		typePanel.add(mDepthRuler);
		mDisplay.add(BorderLayout.NORTH, typePanel);

		Simulator.getInstance().addSystemObject(this);
		
		mIsActive = false;
	}
	
	public void updateHighlighter() {
		
		//performance reasons.
		if(!mIsActive && !mActivateCheckBox.isSelected()) {
			return;
		}
		
		if(mActivateCheckBox.isSelected()) {
			mIsActive = true;
			synchronized(mNeuronEnvironment) {
				Vector selectedNeurons = mNetworkViewer.getSelectedObjects();
				Vector availableNeurons = mNeuronEnvironment.getNeuronObjects();
				
				for(Iterator i = availableNeurons.iterator(); i.hasNext();) {
					NeuronObject obj = (NeuronObject) i.next();
					if(obj != null) {
						obj.setShapeTransparency(1.0);
					}
				}
				
				Vector neuronObjects = new Vector();
				for(Iterator i = selectedNeurons.iterator(); i.hasNext();) {
					Object obj = i.next();
					if(obj instanceof NeuronObject) {
						neuronObjects.add(obj);
						availableNeurons.remove(obj);
					}
				}
				
				int mode = HIGHLIGHT_ALL;
				if(mIncommingNeurons.isSelected() && !mOutgoingNeurons.isSelected()) {
					mode = HIGHLIGHT_INCOMMING;
				}
				else if(mOutgoingNeurons.isSelected() && !mIncommingNeurons.isSelected()) {
					mode = HIGHLIGHT_OUTGOING;
				}
				else if (!mOutgoingNeurons.isSelected() && !mIncommingNeurons.isSelected()) {
					mode = 0;
				}
				
				int depth = 0;
				try {
					depth = Integer.parseInt(mDepthRuler.getText());
				}
				catch(NumberFormatException e) {
				}
				
				Vector highlightedNeurons = neuronObjects;
				for(int i = 1; i <= depth; ++i) {
					if(availableNeurons.size() == 0) {
						break;
					}
					else if(highlightedNeurons.size() == 0) {
						highlightNeurons(highlightedNeurons, availableNeurons, 0.0, mode);
						break;
					}
					double transparency = 1.0 - ((1.0 / (((double) depth))) * ((double) i));
					highlightedNeurons = highlightNeurons(highlightedNeurons, availableNeurons, transparency, mode);
				}
				
				
			}
		}
		else {
			mIsActive = false;
			synchronized(mNeuronEnvironment) {
				Vector neuronObjects = mNeuronEnvironment.getNeuronObjects();
				for(Iterator i = neuronObjects.iterator(); i.hasNext();) {
					NeuronObject obj = (NeuronObject) i.next();
					obj.setShapeTransparency(1.0);
				}	
			}
		}
	}
	
	public Vector highlightNeurons(Vector highlighted, Vector availableNeurons, double transparency, int mode) {
		
		Vector neighbours = new Vector();
		
		Vector highlightedNeurons = new Vector();
		Vector inSynapses = new Vector();
		for(Iterator i = highlighted.iterator(); i.hasNext();) {
			try {
				NeuronObject obj = (NeuronObject) i.next();
				Neuron n = obj.getNeuron();
				if(n != null) {
					inSynapses.addAll(n.getSynapses());
					highlightedNeurons.add(n);
				}
			} 
			catch(ClassCastException e) {
				//ignore.
			}
		}
		Vector outSynapses = new Vector();
		for(Iterator i = availableNeurons.iterator(); i.hasNext();) {
			try {
				NeuronObject obj = (NeuronObject) i.next();
				Neuron n = obj.getNeuron();
				if(n != null) {
					Vector synapses = n.getSynapses();
					for(Iterator j = synapses.iterator(); j.hasNext();) {
						Synapse s = (Synapse) j.next();
						if(highlightedNeurons.contains(s.getSourceNeuron())) {
							outSynapses.add(s);
						}
					}
				}
			}
			catch (ClassCastException e) {
				//ignore
			}
		}
		
		Vector neighbourCandidates = new Vector(availableNeurons);
		
		for(Iterator i = neighbourCandidates.iterator(); i.hasNext();) {
			NeuronObject obj = (NeuronObject) i.next();
			Neuron n = obj.getNeuron();
			if(n == null) {
				continue;
			}
			
			boolean foundSynapse = false;
			for(Iterator j = inSynapses.iterator(); j.hasNext();) {
				Synapse s = (Synapse) j.next();
				
				
				if(s.getEnabledValue().get() && (mode == HIGHLIGHT_ALL || mode == HIGHLIGHT_INCOMMING) && s.getSourceNeuron() == n) {
					neighbours.add(obj);
					availableNeurons.remove(obj);
					foundSynapse = true;
					break;
				}
			}
			if(!foundSynapse) {
				for(Iterator j = outSynapses.iterator(); j.hasNext();) {
					Synapse s = (Synapse) j.next();
					
					
					if(s.getEnabledValue().get() && (mode == HIGHLIGHT_ALL || mode == HIGHLIGHT_OUTGOING) && s.getTargetNeuron() == n) {
						neighbours.add(obj);
						availableNeurons.remove(obj);
						foundSynapse = true;
						break;
					}
				}
			}
			if(!foundSynapse) {
				obj.setShapeTransparency(transparency);
			}
		}
		
		return neighbours;
	}
	
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		return true;
	}

	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(value == mSelectedObjectsChangedValue) {
			updateHighlighter();
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mSelectedObjectsChangedValue = (BooleanValue) ValueManager.synchronizeValue(mSelectedObjectsChangedValue, oldValue, newValue, this);
	}
	
	

}
