package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Random;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.SystemObject;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;

public class NeuronVisualizationEnvironment extends EnvironmentModel implements SystemObject  {

	private static int mNeuronVisualizationCounter = 0;
	private int mId;
	private Vector mNeuronObjects = new Vector();
	private Vector mSynapseObjects = new Vector();
	
	public NeuronVisualizationEnvironment() {
		super("NeuronVisualizationEnvironment");
		mId = mNeuronVisualizationCounter++;
		Simulator.getInstance().addSystemObject(this);
		loadInitialPositions(Simulator.getInstance().getConfigDirectoryName() + "netvisu" + mId + ".conf", true);
	}


	public void createEnvironment() {
		resetEnvironmentObjects();
	}

	public synchronized void resetEnvironmentObjects() {
		
		NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
		
		Vector neuronObjects = getNeuronObjects();
		Vector synapseObjects = getSynapseObjects();
		
		if(net == null) {
			for(int i = 0; i < neuronObjects.size(); ++i) {
				((NeuronObject) neuronObjects.elementAt(i)).setNeuron(null);
			}
			//Set the represented synapse and the collaborands to null (the SimObjects become inactive then).
			for(int i = 0; i < synapseObjects.size(); ++i) {
				((SynapseObject) synapseObjects.elementAt(i)).setSynapse(null, null, null);
			}
			return;
		}
		
		synchronized (net) {
	
			
			Vector neurons = net.getNeurons();
			//Set the represented Neuron for all NeuronObjects to null (the NeuronObjects therefore become inactive.)
			for(int i = 0; i < neuronObjects.size(); ++i) {
				NeuronObject no = (NeuronObject) neuronObjects.elementAt(i);
				if(!neurons.contains(no.getNeuron())) {
					no.setNeuron(null);
					no.setActive(false);
				}
			}
			
			Vector synapses = net.getSynapses();
			//Set the represented synapse and the collaborands to null (the SimObjects become inactive then).
			for(int i = 0; i < synapseObjects.size(); ++i) {
				SynapseObject so = (SynapseObject) synapseObjects.elementAt(i);
				if(so.getSynapse() == null || !synapses.contains(so.getSynapse())) {
					so.setSynapse(null, null, null);
					so.setActive(false);
				}
			}
				
			
			Random rand = new Random();
			
	
			NeuronObject neuronObject = null;
			SynapseObject synapseObject = null;
			for(int i = 0; i < neurons.size(); ++i) {
				Neuron neuron = (Neuron) neurons.elementAt(i);
				neuronObject = null;
				
				//check if there is a neuron object that represents this neuron.
				for(Iterator ni = neuronObjects.iterator(); ni.hasNext();) {
					NeuronObject no = (NeuronObject) ni.next();
					if(no.getNeuron() == neuron) {
						neuronObject = no;
						break;
					}
					else if(neuronObject == null && no.getNeuron() == null) {
						neuronObject = no;
					}
				}
				
				if(neuronObject == null) {
					neuronObject = new NeuronObject(rand.nextDouble() * 400.0, rand.nextDouble() * 400.0, 0.0);
					addSimObject(neuronObject);
				}
				
				neuronObject.setNeuron(neuron);
				
				Vector localSynapses = neuron.getSynapses();
				
				for(Iterator j = localSynapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					synapseObject = null;
					
					
					for(Iterator si = synapseObjects.iterator(); si.hasNext();) {
						SynapseObject so = (SynapseObject) si.next();
						if(so.getSynapse() == synapse) {
							synapseObject = so;
							synapseObject.setSynapse(null, null, null);
							break;
						}
						else if(synapseObject == null && so.getTargetNeuron() == null && so.getSynapse() == null) {
							synapseObject = so;
						}
					}
					
					if(synapseObject == null) {
						synapseObject = new SynapseObject(rand.nextDouble() * 400.0, rand.nextDouble() * 400.0, 0.0);
						addSimObject(synapseObject);
					}
					synapseObject.setSynapse(synapse, null, neuronObject);
				}
			}
			
			neuronObjects = mNeuronObjects;
			synapseObjects = mSynapseObjects;
			
			//Find the NeuronObjects that represent the source neurons of the synapses.
			for(Iterator i = synapseObjects.iterator(); i.hasNext();) {
				SynapseObject synapseObj = (SynapseObject) i.next();
				Synapse synapse = (Synapse) synapseObj.getSynapse();
				if(synapse == null) {
					continue;
				}
				Neuron source = synapse.getSourceNeuron();
				if(source == null) {
					continue;
				}
				for(Iterator j = neuronObjects.iterator(); j.hasNext();) {
					NeuronObject neuronObj = (NeuronObject) j.next();
					if(neuronObj.getNeuron() == source) {
						//if(synapseObject.getSourceNeuron() != neuronObj) {
							synapseObj.setSynapse(synapse, neuronObj, synapseObj.getTargetNeuron());
							synapseObj.centerPosition();
						//}
					}
				}
			}
		}
	}

	public void updateRandomizationParameters() {
	}
	
	
	/**
	 * Sorts the NeuronObjects to the same order as the Neuron objects in the
	 * current Neuralnet. This is important for layout informations.
	 */
	public void sortNeuronObjects() {
		Vector neuronObjs = new Vector();
		LinkedList oldNeuronObjs = new LinkedList(mNeuronObjects);
		
		NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
		
		if(net == null) {
			return;
		}
		
		synchronized (net) {
			Vector neurons = net.getNeurons();
			
			for(Iterator i = neurons.iterator(); i.hasNext();) {
				Neuron n = (Neuron) i.next();
				
				for(int j = 0; j < oldNeuronObjs.size(); ++j) {
					NeuronObject obj = (NeuronObject) oldNeuronObjs.get(j);
					
					if(obj.getNeuron() == n) {
						neuronObjs.add(obj);
						oldNeuronObjs.remove(obj);
						break;
					}
				}
			}
		}
		//add the rest to the end.
		neuronObjs.addAll(oldNeuronObjs);
		
		mNeuronObjects = neuronObjs;
	}
	
	
	
	
	public synchronized boolean addSimObject(SimObject object) {
		if(!super.addSimObject(object)) {
			return false;
		}
		
		if(object instanceof NeuronObject) {
			mNeuronObjects.add(object);
		}
		else if(object instanceof SynapseObject) {
			mSynapseObjects.add(object);
		}
		return true; 
	}


	/**
	 * Returns a Vector that contains only the SimObjects that are derived from 
	 * class SynapseObject.
	 * 
	 * @return a Vector with all SynapseObjects.
	 */
	public Vector getSynapseObjects() {
		updateContent();
//		Vector objs = new Vector();
//		for(Iterator i = mLayerSortedSimObjects.iterator(); i.hasNext();) {
//			Object obj = i.next();
//			if(obj instanceof SynapseObject) {
//				objs.add(obj);
//			}
//		}
//		return objs;
		return new Vector(mSynapseObjects);
	}
	
	
	/**
	 * Returns a Vector that contains only the NeuronObjects that are derived from 
	 * class SynapseObject.
	 * 
	 * @return a Vector with all NeuronObjects.
	 */
	public Vector getNeuronObjects() {
		updateContent();
//		Vector objs = new Vector();
//		for(Iterator i = mLayerSortedSimObjects.iterator(); i.hasNext();) {
//			Object obj = i.next();
//			if(obj instanceof NeuronObject) {
//				objs.add(obj);
//			}
//		}
//		return objs;
		return new Vector(mNeuronObjects);
		
	}

	
	/**
	 * Loads the previously saved positions of the neurons and synapses at startup.
	 */
	public boolean loadInitialPositions(String fileName, boolean createNewNeurons) {
		
		sortNeuronObjects();
		
		Properties props = new Properties();
		try {
			File file = new File(fileName);
			FileInputStream stream = new FileInputStream(file);
			props.load(stream);
			stream.close();
		} catch (FileNotFoundException e) {
			return false;
		} catch (IOException e) {
			return false;
		}
		
		Object entries[] = props.keySet().toArray();
		Arrays.sort(entries);
		
		Vector neurons = getNeuronObjects();
		for(int i = 0; i < entries.length - 1; i = i + 2) {
			String entryX = (String) entries[i];
			String entryY = (String) entries[i + 1];
			
			if(entryX.startsWith("Neuron") && entryX.endsWith("X")
					&& entryY.startsWith("Neuron") && entryY.endsWith("Y")) 
			{
				try {
					double x = Double.parseDouble(props.getProperty(entryX));
					double y = Double.parseDouble(props.getProperty(entryY));
					
					if((i / 2) < neurons.size()) {
						NeuronObject neuron = (NeuronObject) neurons.elementAt(i / 2);
						neuron.setPosition(x, y);
						neuron.setAngle(0.0);
					}
					else if(createNewNeurons) {
						NeuronObject no = new NeuronObject(x, y, 0.0);
						no.setActive(false);
						addSimObject(no);
					}
				} catch (NumberFormatException e) {
				}
			}
		}
		return true;
	}
	
	public boolean saveLayout(String fileName, int maxNumberOfNeurons, boolean activeOnly) {
		
		sortNeuronObjects();
		
		//Save the neuron and synapse positions for recovery at the next start.
		Properties props = new Properties();
		Vector neurons = mNeuronObjects;

		if(maxNumberOfNeurons == -1) {
			maxNumberOfNeurons = Integer.MAX_VALUE;
		}
		for(int i = 0; i < neurons.size() && i < maxNumberOfNeurons; ++i) {
			NeuronObject n = (NeuronObject) neurons.elementAt(i);
			if(activeOnly && !n.isActive()) {
				continue;
			}
			String prefix = "";
			if(i < 10) {
				prefix = "000";
			}
			else if(i < 100) {
				prefix = "00";
			}
			else if(i < 1000) {
				prefix = "0";
			}
			Point2D.Double pos = n.getPosition();
			props.put("Neuron" + prefix + i + "X", Double.toString(pos.x));
			props.put("Neuron" + prefix + i + "Y", Double.toString(pos.y));
		}
		
		try {
			File file = new File(fileName);
			FileOutputStream stream = new FileOutputStream(file);
			props.store(stream, "Network Layout");
			stream.close();
		} catch (FileNotFoundException e) {
			return false;
		} catch (IOException e) {
			return false;
		}
		return true;
	}

	public boolean init() {
		return true;
	}


	public boolean cleanUp() {
		
		saveLayout(Simulator.getInstance().getConfigDirectoryName() + "netvisu" + mId + ".conf", 20, false);
		
//		//Save the neuron and synapse positions for recovery at the next start.
//		Properties props = new Properties();
//		Vector neurons = getNeuronObjects();
//		//log the positions of up to 20 neurons.
//		for(int i = 0; i < neurons.size() && i < 25; ++i) {
//			NeuronObject n = (NeuronObject) neurons.elementAt(i);
//			Point2D.Double pos = n.getPosition();
//			props.put("Neuron" + i + "X", Double.toString(pos.x));
//			props.put("Neuron" + i + "Y", Double.toString(pos.y));
//		}
//		
////		Vector synapses = getSynapseObjects();
////		for(int i = 0; i < synapses.size() && i < 10; ++i) {
////			SynapseObject n = (SynapseObject) synapses.elementAt(i);
////			Point2D.Double pos = n.getPosition();
////			props.put("Synapse" + i + "X", Double.toString(pos.x));
////			props.put("Synapse" + i + "Y", Double.toString(pos.y));
////		}
//		
//		try {
//			File file = new File(Simulator.getInstance().getConfigDirectoryName() + "netvisu" + mId + ".conf");
//			FileOutputStream stream = new FileOutputStream(file);
//			props.store(stream, "Properties of the NeuronVisualizationEnvironment");
//			stream.close();
//		} catch (FileNotFoundException e) {
//		} catch (IOException e) {
//		}
		
		return true;
	}
	
	
	
	

}
