package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunction;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class TransferFunctionSelectionPanel implements Visualization, NeuralNetListener {
	
	private JPanel mDisplay;
	private JComboBox mTransferFunctionSelection;
	private JCheckBox mActivateSelectionBox;
	private Vector mTransferFunctions = new Vector();
	
	public TransferFunctionSelectionPanel() {
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("Transfer Function"));
		
		mTransferFunctionSelection = new JComboBox();
		mTransferFunctionSelection.setEnabled(false);
		mActivateSelectionBox = new JCheckBox();
		mActivateSelectionBox.setEnabled(false);
		mActivateSelectionBox.setSelected(false);
		
		mActivateSelectionBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mActivateSelectionBox.isSelected()) {
					mTransferFunctionSelection.setEnabled(true);
				}
				else {
					mTransferFunctionSelection.setEnabled(false);
				}
			}
		});
		
		mTransferFunctionSelection.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String tfName = (String) mTransferFunctionSelection.getSelectedItem();
				Vector transferFunctions = Simulator.getInstance().getNeuralNetManager().getTransferFunctionPrototypes();
				for(Iterator i = transferFunctions.iterator(); i.hasNext();) {
					TransferFunction tf = (TransferFunction) i.next();
					if(tf.getName().equals(tfName)) {
						//change transfer function for all neurons.
						NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
						if(net != null) {
							synchronized (net) {
//								Vector neurons = net.getNeurons();
//								for(Iterator j = neurons.iterator(); j.hasNext();) {
//									((Neuron) j.next()).setTransferFunction(tf);
//								}
								net.setDefaultTransferFunction(tf);
								return;
							}
						}
					}
				}
			}
		});
		
		mDisplay.add(mActivateSelectionBox, BorderLayout.NORTH);
		mDisplay.add(mTransferFunctionSelection, BorderLayout.CENTER);
		
		Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		Vector transferFunctions = Simulator.getInstance().getNeuralNetManager().getTransferFunctionPrototypes();
		for(Iterator i = transferFunctions.iterator(); i.hasNext();) {
			String name = ((TransferFunction) i.next()).getName();
			mTransferFunctionSelection.addItem(name);
			mTransferFunctions.add(name);
		}
		
		return true;
	}

	public void currentNeuralNetChanged(NeuralNet currentNet) {
		if(currentNet == null) {
			mActivateSelectionBox.setSelected(false);
			mActivateSelectionBox.setEnabled(false);
			mTransferFunctionSelection.setEnabled(false);
		}
		else {
			mActivateSelectionBox.setEnabled(true);
			if(mActivateSelectionBox.isSelected()) {
				mTransferFunctionSelection.setEnabled(true);
			}
			for(int i = 0; i < mTransferFunctions.size(); ++i) {
				String name = (String) mTransferFunctions.elementAt(i);
				if(name.equals(currentNet.getDefaultTransferFunction().getName())) {
					mTransferFunctionSelection.setSelectedIndex(i);
					break;
				}
			}
		}
	}

	public void networkActivationsUpdated(NeuralNet currentNet) {
	}

	public void networkStructureChanged(NeuralNet currentNet) {
		
	}
	
	

}
