package de.fhg.iais.outdoor.simba.visualization.valuepanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;;

public class PanelWrapper extends JPanel {
	
	public static final long serialVersionUID = 1002L;
	private JPanel mPanel;
	private JButton mUpButton;
	private JButton mDownButton;
	private JButton mDeleteButton;
	
	
	/**
	 * A PanelWrapper wrapps a JPanel with a border and some buttons to remove or move
	 * that panel in a ValuePanelDisplay. 
	 * Pressing the up button will move the entire panel one position upwards 
	 * in the ValuePanelDisplay. The opposite is true for the Dn button.
	 * The X button removes the panel.
	 */
	private ValuePanelDisplay mValuePanelDisplay; //TODO generalize with interface.
	
	/**
	 * Constructor.
	 * 
	 * @param panel the panel to wrap.
	 * @param valuePanelDisplay the valuePanelDisplay where this wrapper is used in.
	 */
	public PanelWrapper(JPanel panel, ValuePanelDisplay valuePanelDisplay) {
		setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		mValuePanelDisplay = valuePanelDisplay;
		mPanel = panel;
		setLayout(new BorderLayout());
		add(panel, BorderLayout.CENTER);
		
		JPanel controlPanel = new JPanel();
		mUpButton = new JButton("Up");
		mDownButton = new JButton("Dn");
		mDeleteButton = new JButton("X");
		mUpButton.setMargin(new Insets(0, 0, 0, 0));
		mDownButton.setMargin(new Insets(0, 0, 0, 0));
		mDeleteButton.setMargin(new Insets(0, 0, 0, 0));
		
		mUpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				moveWrapper(true);
			}
		});
		
		mDownButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				moveWrapper(false);
			}
		});
		
		mDeleteButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeWrapper();
			}
		});
		
		controlPanel.add(mUpButton);
		controlPanel.add(mDownButton);
		controlPanel.add(mDeleteButton);
		
		add(controlPanel, BorderLayout.EAST);
	}
	
	/**
	 * Moves the wrapper panel one position up or down in the ValuePanelDisplay 
	 * it is used in.
	 * 
	 * @param up up if true, down if false.
	 */
	private void moveWrapper(boolean up) {
		mValuePanelDisplay.moveWrapper(up, this);
	}
	
	/**
	 * Removes the wrapper panel from the ValuePanelDisplay it is used in.
	 */
	private void removeWrapper() {
		mValuePanelDisplay.removeWrapper(this);
	}
	
	
	/**
	 * Returns the JPanel object that is wrapped by this wrapper.
	 * @return
	 */
	public JPanel getWrappedPanel() {
		return mPanel;
	}

}
