package de.fhg.iais.outdoor.simba.xml;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


import org.w3c.dom.DOMException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


import com.sun.org.apache.xerces.internal.dom.DocumentImpl;
import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunction;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunctionSigmoid;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunctionTanh;


/**
 * TODO Redo and protect against wrong parameters and runtime exceptions.
 * @author Chris
 *
 */
public class NeuralNetParser {

	
	public static int getParentId(NeuralNet net, Neuron parent) {
		Vector neurons = net.getNeurons();
		
		for(int i = 0; i < neurons.size(); ++i) {
			if(parent == neurons.elementAt(i)) {
				return i;
			}
		}
		return -1;
	}
	
	
	public static String wrapEvosunEvolutionXML(String netXML) {
		StringBuffer output = new StringBuffer();
		output.append("<?xml version=\"1.0\" encoding=\"LATIN1\"?>\n<Evolution>\n" +
				"  <Generation Index=\"0\">\n    <Population Name=\"pop1\" Index=\"0\">\n" +
						"       <Comment> no comment </Comment>\n" +
						"          <EvoObject  Index=\"0\" OutPerf=\"0.0\" SysPerf=\"0.0\" Age=\"0\">\n");
		int index = netXML.indexOf("<Net");
		output.append(netXML.substring(index));
		output.append("          </EvoObject>\n    </Population>\n  </Generation>\n</Evolution>");
		
		return output.toString();
	}
	
	
	/**
	 * Removes the XML header before the net tag, which is included automatically by the 
	 * DOM XML generatior.
	 * @param net
	 * @return
	 */
	public static String createPureSimbaXMLFromNet(NeuralNet net) {
		String xml = createSimbaXMLFromNet(net);
		int index = xml.indexOf("<net");
		if(index == -1) {
			return "";
		}
		return xml.substring(index);
	}
	
	
	public static String createSimbaXMLFromNet(NeuralNet net) {
		try {
			Document doc = new DocumentImpl();
			
			Element netRoot = doc.createElement("net");
			netRoot.setAttribute("defaultTransferFunction", net.getDefaultTransferFunction().getName());
				
			Vector neurons = net.getNeurons();
			for(Iterator i = neurons.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				Element neuronElem = doc.createElement("neuron");
				netRoot.appendChild(neuronElem);
				neuronElem.setAttribute("name", neuron.getName());
				neuronElem.setAttribute("layer", neuron.getType());
				neuronElem.setAttribute("bias", String.valueOf(neuron.getBias()));
				neuronElem.setAttribute("protection", neuron.getProtectionMode());
				neuronElem.setAttribute("transferFunction", neuron.getTransferFunction().getName());
				neuronElem.setAttribute("maxIn", Integer.toString(neuron.getMaxIncomingSynapsesValue().get()));
				neuronElem.setAttribute("maxOut", Integer.toString(neuron.getMaxOutgoingSynapsesValue().get()));
				
				
				Vector synapses = neuron.getSynapses();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					Element synapseElem = doc.createElement("synapse");
					neuronElem.appendChild(synapseElem);
					synapseElem.setAttribute("source", String.valueOf(NeuralNetParser.getParentId(net, synapse.getSourceNeuron())));
					synapseElem.setAttribute("protection", synapse.getProtectionMode());
					synapseElem.setAttribute("strength", String.valueOf(synapse.getStrength()));
					
				}
			}
	
			doc.appendChild(netRoot);
			StringWriter fos = new StringWriter();

			OutputFormat of = new OutputFormat();
			of.setIndent(1);
			of.setIndenting(true);
			//of.setDoctype(null,"users.dtd");
			XMLSerializer serializer = new XMLSerializer(fos, of);

			serializer.asDOMSerializer();
			serializer.serialize( doc.getDocumentElement() );
			fos.close();
			return fos.toString();
			
		} catch (DOMException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return "";
	}
	
	
	public static String createPureEvosunNetXML(NeuralNet net) {
		String xml = createEvosunXMLFromNet(net);
		int index = xml.indexOf("<Net");
		if(index == -1) {
			return "";
		}
		return xml.substring(index);
	}
	
	
	public static String createEvosunXMLFromNet(NeuralNet net) {
		
		try {
			Document doc = new DocumentImpl();
			
			Element netRoot = doc.createElement("Net");
			netRoot.setAttribute("SynapseMode", "conventional");
			netRoot.setAttribute("Transferfunction", net.getDefaultTransferFunction().getName());
			
			Vector neurons = net.getNeurons();
			for(Iterator i = neurons.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				Element neuronElem = doc.createElement("Neuron");
				netRoot.appendChild(neuronElem);
				neuronElem.setAttribute("Layer", neuron.getType());
				neuronElem.setAttribute("Process", "dynamic");
				neuronElem.setAttribute("Bias", String.valueOf(neuron.getBias()));
				
				Vector synapses = neuron.getSynapses();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					Element synapseElem = doc.createElement("Synapse");
					neuronElem.appendChild(synapseElem);
					synapseElem.setAttribute("Source", String.valueOf(NeuralNetParser.getParentId(net, synapse.getSourceNeuron())));
					synapseElem.setAttribute("Process", "dynamic");
					synapseElem.setAttribute("Strength", String.valueOf(synapse.getStrength()));
					
				}
			}
	
			doc.appendChild(netRoot);
			StringWriter fos = new StringWriter();

			OutputFormat of = new OutputFormat();
			of.setIndent(1);
			of.setIndenting(true);
			//of.setDoctype(null,"users.dtd");
			XMLSerializer serializer = new XMLSerializer(fos, of);

			serializer.asDOMSerializer();
			serializer.serialize( doc.getDocumentElement() );
			fos.close();
			return fos.toString();
			
		} catch (DOMException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return "";
	}
	
	public static NeuralNet createNetFromSimbaXMLFile(String fileName, NeuralNetAgentInterface agentInterface) {
		StringBuffer buffer = new StringBuffer();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(fileName));
			
			String line = reader.readLine();
			while(line != null) {
				buffer.append(line);
				line = reader.readLine();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		return createNetFromSimbaXML(buffer.toString(), agentInterface);
	}
	
	public static NeuralNet createNetFromSimbaXML(String xml, NeuralNetAgentInterface agentInterface) {
		
		
		
		if(agentInterface == null) {
			return null;
		}
		
		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document doc = builder.parse(new InputSource(new StringReader(xml)));
					
			NodeList netList = doc.getElementsByTagName("net");
			Element netRoot = null;
			
			for(int i = 0; i < netList.getLength(); ++i) {
				Node node = netList.item(i);
				if(node.getNodeType() == Node.ELEMENT_NODE) {
					netRoot = (Element) node;
					break;
				}
			}
			if(netRoot == null) {
				return null;
			}
			return createNetFromSimbaNetNode(netRoot, agentInterface);
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
			
	}
	
	public static NeuralNet createNetFromSimbaNetNode(Element netRoot, NeuralNetAgentInterface agentInterface) {
		
		NeuralNet net = new NeuralNet();

		int outputNeuronCount = 0;
		
		Vector neuronBuffer = new Vector();
		Hashtable synapseBuffer = new Hashtable();
		
		String transferFunction = netRoot.getAttribute("defaultTransferFunction");
		
		Vector transferFunctionPrototypes = Simulator.getInstance().getNeuralNetManager().getTransferFunctionPrototypes();
		for(Iterator i = transferFunctionPrototypes.iterator(); i.hasNext();) {
			TransferFunction tf = (TransferFunction) i.next();
			if(tf.getName().equals(transferFunction)) {
				net.setDefaultTransferFunction(tf);
				break;
			}
		}
		
		NodeList neurons = netRoot.getChildNodes();
		Element neuronElem = null;
		
		for(int i = 0; i < neurons.getLength(); ++i) {
			neuronElem = null;
			Node node = neurons.item(i);
			if(node.getNodeType() == Node.ELEMENT_NODE) {
				neuronElem = (Element) node;
				
				String name = neuronElem.getAttribute("name");
				String layer = neuronElem.getAttribute("layer");
				String bias = neuronElem.getAttribute("bias");
				String protection = neuronElem.getAttribute("protection");
				String transFct = neuronElem.getAttribute("transferFunction");
				String maxIn = neuronElem.getAttribute("maxIn");
				String maxOut = neuronElem.getAttribute("maxOut");
				
				if(layer.equals("") || bias.equals("") || name.equals("")
						|| protection.equals("") || transFct.equals("") 
						|| maxIn.equals("") || maxOut.equals("")) 
				{
					Logger.getLogger("Simulator").warning("NeuralNetParser: Could not find one of the required attributes.");
					continue;
				}
				
				TransferFunction neuronTf = null;
				for(Iterator j = transferFunctionPrototypes.iterator(); j.hasNext();) {
					TransferFunction tf = (TransferFunction) j.next();
					if(tf.getName().equals(transFct)) {
						neuronTf = tf;
						break;
					}
				}

				
				Neuron neuron = new Neuron(name, neuronTf, layer);
				neuron.setProtectionMode(protection);
				
				try {
					neuron.setBias(Double.parseDouble(bias));
				} catch (NumberFormatException e) {
				}
				try {
					neuron.getMaxIncomingSynapsesValue().set(Integer.parseInt(maxIn));
				} catch (NumberFormatException e) {
				}
				try {
					neuron.getMaxOutgoingSynapsesValue().set(Integer.parseInt(maxOut));
				} catch (NumberFormatException e) {
				}
				
				net.addNeuron(neuron);
				neuronBuffer.add(neuron);
				
				NodeList synapseList = neuronElem.getChildNodes();
				synapseBuffer.put(neuron, synapseList);
			}
		}
		
		//add synapses
		Set keys = synapseBuffer.keySet();
		for(Iterator i = keys.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			NodeList synapseList = (NodeList) synapseBuffer.get(neuron);
		
			for(int j = 0; j < synapseList.getLength(); ++j) {
				Node node = (Node) synapseList.item(j);
				if(node.getNodeType() == Node.ELEMENT_NODE) {
					Element synapseElem = (Element) node;
					String strength = synapseElem.getAttribute("strength");
					String source = synapseElem.getAttribute("source");
					String protection = synapseElem.getAttribute("protection");
					
					if(strength.equals("") || source.equals("") || protection.equals("")) {
						Logger.getLogger("Simulator").warning("NeuralNetParser: Could not find strength, source or protection in xml for synapse.");
						continue;
					}
					int sourceId = Integer.parseInt(source);
					if(sourceId >= 0 && sourceId < neuronBuffer.size()) {
						Synapse synapse = new Synapse((Neuron) neuronBuffer.elementAt(sourceId));
						synapse.setStrength(Double.parseDouble(strength));
						synapse.setProtectionMode(protection);
						
						neuron.addSynapse(synapse);
					}
					
				}
			}
		}
		
		//connect input and output neurons
		Vector inputNeurons = agentInterface.getInputNeurons();
		//System.out.println("AgentInterfeace: inputs: " + inputNeurons.size());
		Iterator inputIterator = inputNeurons.iterator();
		for(Iterator i = neuronBuffer.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			if(neuron.getType().equals(Neuron.INPUT)) {
				if(inputIterator.hasNext()) {
					NeuronWrapper wrapper = (NeuronWrapper) inputIterator.next();
					neuron.setActivationValue(wrapper);
					neuron.setName(wrapper.getName());
				}
			}
		}
		if(inputIterator.hasNext()) { //too less input neurons...
			while(inputIterator.hasNext()) {
				NeuronWrapper wrapper = (NeuronWrapper) inputIterator.next();
				Neuron n = new Neuron(wrapper.getName(), null, Neuron.INPUT);
				n.setActivationValue(wrapper);
				net.addNeuron(n);
			}
		}
		Vector outputNeurons = agentInterface.getOutputNeurons();
		Iterator outputIterator = outputNeurons.iterator();
		for(Iterator i = neuronBuffer.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			if(neuron.getType().equals(Neuron.OUTPUT)) {
				if(outputIterator.hasNext()) {
					NeuronWrapper wrapper = (NeuronWrapper) outputIterator.next();
					neuron.setActivationValue(wrapper);
					neuron.setName(wrapper.getName());
				}
			}
		}
		if(outputIterator.hasNext()) { //too less output neurons...
			while(outputIterator.hasNext()) {
				NeuronWrapper wrapper = (NeuronWrapper) outputIterator.next();
				Neuron n = new Neuron("OutputNeuron" + ++outputNeuronCount, null, Neuron.OUTPUT);
				n.setActivationValue(wrapper);
				n.setName(wrapper.getName());
				net.addNeuron(n);
			}
		}

		return net;
	}
		
	
	
	public static NeuralNet createNetFromEvosunXMLFile(String fileName, NeuralNetAgentInterface agentInterface) {
		
		StringBuffer buffer = new StringBuffer();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(fileName));
			
			String line = reader.readLine();
			while(line != null) {
				buffer.append(line);
				line = reader.readLine();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		return createNetFromEvosunXML(buffer.toString(), agentInterface);
	}
	
	public static NeuralNet createNetFromEvosunXML(String xml, NeuralNetAgentInterface agentInterface) {
		
		NeuralNet net = new NeuralNet();
		
		if(agentInterface == null) {
			return null;
		}
		
		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document doc = builder.parse(new InputSource(new StringReader(xml)));
			
			int inputNeuronCount = 0;
			int outputNeuronCount = 0;
			int hiddenNeuronCount = 0;
			
			Vector neuronBuffer = new Vector();
			Hashtable synapseBuffer = new Hashtable();
			
			NodeList netList = doc.getElementsByTagName("Net");
			Element netRoot = null;
			
			for(int i = 0; i < netList.getLength(); ++i) {
				Node node = netList.item(i);
				if(node.getNodeType() == Node.ELEMENT_NODE) {
					netRoot = (Element) node;
					break;
				}
			}
			if(netRoot == null) {
				return null;
			}
			
			String transferFunction = netRoot.getAttribute("Transferfunction");
			
			if(transferFunction.equals("tanh")) {
				net.setDefaultTransferFunction(new TransferFunctionTanh());
			}
			else if(transferFunction.equals("sigmoid")) {
				net.setDefaultTransferFunction(new TransferFunctionSigmoid());
			}
			
			NodeList neurons = netRoot.getChildNodes();
			Element neuronElem = null;
			
			for(int i = 0; i < neurons.getLength(); ++i) {
				neuronElem = null;
				Node node = neurons.item(i);
				if(node.getNodeType() == Node.ELEMENT_NODE) {
					neuronElem = (Element) node;
					
					String layer = neuronElem.getAttribute("Layer");
					String bias = neuronElem.getAttribute("Bias");
					
					if(layer.equals("") || bias.equals("")) {
						Logger.getLogger("Simulator").warning("NeuralNetParser: Could not find layer or bias of neuron.");
						continue;
					}
					
					String name = "";
					if(layer.equals(Neuron.INPUT)) {
						name = "InputNeuron" + ++inputNeuronCount;
					}
					else if(layer.equals(Neuron.OUTPUT)) {
						name = "OutputNeuron" + ++outputNeuronCount;
					}
					else {
						name = "HiddenNeuron" + ++hiddenNeuronCount;
					}
					
					Neuron neuron = new Neuron(name, null, layer);
					neuron.setBias(Double.parseDouble(bias));
					net.addNeuron(neuron);
					neuronBuffer.add(neuron);
					
					NodeList synapseList = neuronElem.getChildNodes();
					synapseBuffer.put(neuron, synapseList);
				}
			}
			
			//add synapses
			Set keys = synapseBuffer.keySet();
			for(Iterator i = keys.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				NodeList synapseList = (NodeList) synapseBuffer.get(neuron);
			
				for(int j = 0; j < synapseList.getLength(); ++j) {
					Node node = (Node) synapseList.item(j);
					if(node.getNodeType() == Node.ELEMENT_NODE) {
						Element synapseElem = (Element) node;
						String strength = synapseElem.getAttribute("Strength");
						String source = synapseElem.getAttribute("Source");
						if(strength.equals("") || source.equals("")) {
							Logger.getLogger("Simulator").warning("NeuralNetParser: Could not find strength of source in xml for synapse.");
							continue;
						}
						int sourceId = Integer.parseInt(source);
						if(sourceId >= 0 && sourceId < neuronBuffer.size()) {
							Synapse synapse = new Synapse((Neuron) neuronBuffer.elementAt(sourceId));
							synapse.setStrength(Double.parseDouble(strength));
							neuron.addSynapse(synapse);
						}
						
					}
				}
			}
			
			//connect input and output neurons
			Vector inputNeurons = agentInterface.getInputNeurons();
			//System.out.println("AgentInterfeace: inputs: " + inputNeurons.size());
			Iterator inputIterator = inputNeurons.iterator();
			for(Iterator i = neuronBuffer.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				if(neuron.getType().equals(Neuron.INPUT)) {
					if(inputIterator.hasNext()) {
						NeuronWrapper wrapper = (NeuronWrapper) inputIterator.next();
						neuron.setActivationValue(wrapper);
						neuron.setName(wrapper.getName());
					}
				}
			}
			if(inputIterator.hasNext()) { //too less input neurons...
				while(inputIterator.hasNext()) {
					NeuronWrapper wrapper = (NeuronWrapper) inputIterator.next();
					Neuron n = new Neuron(wrapper.getName(), null, Neuron.INPUT);
					n.setActivationValue(wrapper);
					net.addNeuron(n);
				}
			}
			Vector outputNeurons = agentInterface.getOutputNeurons();
			Iterator outputIterator = outputNeurons.iterator();
			for(Iterator i = neuronBuffer.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				if(neuron.getType().equals(Neuron.OUTPUT)) {
					if(outputIterator.hasNext()) {
						NeuronWrapper wrapper = (NeuronWrapper) outputIterator.next();
						neuron.setActivationValue(wrapper);
						neuron.setName(wrapper.getName());
					}
				}
			}
			if(outputIterator.hasNext()) { //too less input neurons...
				while(outputIterator.hasNext()) {
					NeuronWrapper wrapper = (NeuronWrapper) outputIterator.next();
					Neuron n = new Neuron("OutputNeuron" + ++outputNeuronCount, null, Neuron.OUTPUT);
					n.setActivationValue(wrapper);
					n.setName(wrapper.getName());
					net.addNeuron(n);
				}
			}
			
			
			
			
			
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
		return net;
	}
	
	public static void main(String args[]) {
		NeuralNet net = new NeuralNet();

		Neuron inputLeftScanner = new Neuron("LeftScanner", new TransferFunctionTanh(), Neuron.INPUT);
		Neuron inputRightScanner = new Neuron("RightScanner", new TransferFunctionTanh(), Neuron.INPUT);
		Neuron inputMidScanner = new Neuron("MidScanner", new TransferFunctionTanh(), Neuron.INPUT);
		Neuron outputLeftWheel = new Neuron("LeftWheel", new TransferFunctionTanh(), Neuron.OUTPUT);
		Neuron outputRightWheel = new Neuron("RightWheel", new TransferFunctionTanh(), Neuron.OUTPUT);
		
		net.addNeuron(inputLeftScanner);
		net.addNeuron(inputRightScanner);
		net.addNeuron(inputMidScanner);
		net.addNeuron(outputLeftWheel);
		net.addNeuron(outputRightWheel);
		
		inputMidScanner.setBias(-1.0);
		outputRightWheel.addSynapse(new Synapse(inputLeftScanner, 1.0));
		outputLeftWheel.addSynapse(new Synapse(inputRightScanner, 1.0));
		outputRightWheel.addSynapse(new Synapse(inputMidScanner, 1.0));
		outputLeftWheel.addSynapse(new Synapse(inputMidScanner, 1.0));
		
		//more
		Neuron hidden1 = new Neuron("Hidden1");
		Neuron hidden2 = new Neuron("Hidden2");
		Neuron hidden3 = new Neuron("Hidden3");
		
		net.addNeuron(hidden1);
		net.addNeuron(hidden2);
		net.addNeuron(hidden3);
		outputRightWheel.addSynapse(new Synapse(outputRightWheel, 0.55));
		outputRightWheel.addSynapse(new Synapse(hidden1, 0.3));
		
		hidden1.addSynapse(new Synapse(hidden2, 2.0));
		hidden1.addSynapse(new Synapse(inputLeftScanner, 0.1));
		hidden1.addSynapse(new Synapse(hidden3, 5.0));
		
		hidden2.addSynapse(new Synapse(inputMidScanner, 0.881));
		hidden2.addSynapse(new Synapse(hidden2, 10.5));
		
		hidden3.addSynapse(new Synapse(inputRightScanner, 0.294));
		
		try {
			FileWriter writer = new FileWriter("d:\\tmp\\indy\\testnet.xml");
			String xml = NeuralNetParser.createEvosunXMLFromNet(net);
			writer.write(xml);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
}
